#!/bin/bash
source /opt/iacheck/bin/common_def/GlobalVariable.sh
source ${DATA_PATH}/bin/collect_log/CommonUtils.sh
source ${DATA_PATH}/bin/collect_log/GetLogUtils.sh

#################################################通用环境参数获取###################################
declare chip_ids_array=() # 芯片编号数组

#根据IP获取芯片数
## $1 板卡IP
getChipNumByIp()
{
	local boardIp=$1
	local chip_num=$(Get_Chip_Num "$boardIp")
	echo $chip_num
}
#获取全部板卡的数
getAllBoardNum()
{
	local boardNum=0
	#获取挂载的板卡IP列表
	local str_engine_ssh_info=` sqlite3 ${CHECK_DB_FILE} "select * from ia_engine;" `
    IFS=$'\n' read -d '' -ra arr_engine_ssh_info <<< "$str_engine_ssh_info"
	boardNum=${#arr_engine_ssh_info[@]}
	echo $boardNum
}
#获取全部板卡的芯片数
getAllChipNum()
{
	local chipNum=0
	#获取挂载的板卡IP列表
	local str_engine_ssh_info=` sqlite3 ${CHECK_DB_FILE} "select * from ia_engine;" `
    IFS=$'\n' read -d '' -ra arr_engine_ssh_info <<< "$str_engine_ssh_info"
	for engien_ssh_info_line in "${arr_engine_ssh_info[@]}"
	do
		local boardIp=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $1}')
		local num=$(getChipNumByIp "$boardIp")
		chipNum=$(expr $chipNum + $num)
	done
	echo $chipNum
}
#获取芯片的编号数组
## $1 板卡IP
getChipIdArry()
{
	# 清空数组
	unset chip_ids_array
	chip_ids_array=() # 重新初始化数组
	local boardIp="$1"
	#获取板卡IP对应的芯片数量，生成对应的芯片号数组
	local chip_num=$(Get_Chip_Num "$boardIp")
	local index=0
	for ((; index < chip_num; index++))
	do	
		chip_ids_array+=("$index")
	done
	LOG "$1" "[$1]获取芯片数组为: ${chip_ids_array[*]}"
	LOG "$1" "[$1]获取芯片数组长度为: ${#chip_ids_array[@]}"
}

#提示用户输入
getUserInput()
{
    # 提示用户输入板卡IP
	local boardIp
    read -p "请输入收集的板卡IP（默认为all，可填入多个用英文分号分割，如206.206.124.1;206.206.124.2;）: " boardIp
	if [ -z "${boardIp}" ]; then
		#获取板卡IP对应的芯片数量，生成对应的芯片号数组
		local chip_num=$(Get_Chip_Num "$boardIp")
		local index=0
		for ((; index < chip_num; index++))
		do	
			chip_ids_array+=index
		done
	else
		IFS=';' read -r -a board_ip_array <<< "$boardIp"
    fi

    # 提示用户输入芯片号，默认为all
	local chipIds
    read -p "请输入收集的芯片号（默认为all，可填入多个用英文逗号分割，如0,1,2,3）: " chipIds
	#整理芯片号
	if [ -z "$chipIds" ]; then
		#获取板卡IP对应的芯片数量，生成对应的芯片号数组
		local chip_num=$(Get_Chip_Num "$boardIp")
		local index=0
		for ((; index < chip_num; index++))
		do	
			chip_ids_array+=index
		done
	else
		IFS=',' read -r -a chip_ids_array <<< "$chip_ids"
    fi

    # 提示用户输入备份日志时间，默认不收集
    read -p "请输入收集的备份日志时间（0：不收集；1：1天内；2:2天内 3：3天内）: " log_days
}
#################################################实时日志获取（全部板卡的实时日志）###################################
CollectAllRealtimeLog()
{
	#获取全部板卡的芯片数，用于计算实时日志所需的空间大小
	local chipNum=$(getAllChipNum)
	#校验IAM的空间是否满足
	ret=$(checkIamLogDirSize ${chipNum} 0)
	# 检查SshCmd是否成功执行
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM空间检查异常，无法执行日志收集操作。"
		return 1
	fi
	#开始收集IAM日志
	ret=$(getIamRealLog) #内部存在日志
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM实时日志收集失败。"
		return 1
	fi
	#开始收集板卡日志
	local str_engine_ssh_info=` sqlite3 ${CHECK_DB_FILE} "select * from ia_engine;" `
    IFS=$'\n' read -d '' -ra arr_engine_ssh_info <<< "$str_engine_ssh_info"
	for engien_ssh_info_line in "${arr_engine_ssh_info[@]}"
	do
		local ssh_ip=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $1}')
		local ssh_port=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $2}')
		local ssh_user_name=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $3}')
		local ssh_passwd=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $4}')
		#获取板卡芯片列表
		getChipIdArry "${ssh_ip}"
		if [ ${#chip_ids_array[@]} -eq 0 ]; then
			LOG "IAM" "获取板卡[${ssh_ip}]的芯片列表失败，不进行实时日志收集"
		else
			LOG "IAM" "板卡[${ssh_ip}]将收集芯片[${chip_ids_array[*]}]的实时日志"
			#校验IAE的实时日志目录是否满足
			ret=$(checkBoardRealLogDirSize ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd} "${#chip_ids_array[@]}")
			if [ $? -ne 0 ]; then
				LOG "IAM" "${ssh_ip}实时日志目录剩余空间不满足条件，无法收集。"
				continue
			fi
			#iae的实时日志
			ret=$(getIaeRealLog ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd} "${chip_ids_array[*]}") 
			if [ $? -ne 0 ]; then
				LOG "IAM" "${ssh_ip} IAE实时日志收集失败。"
			else
				LOG "IAM" "${ssh_ip} IAE实时日志收集完成。"
			fi
			#bw的实时日志
			ret=$(getBwRealLog ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd}) 
			if [ $? -ne 0 ]; then
				LOG "IAM" "${ssh_ip} BW实时日志收集失败。"
			else
				LOG "IAM" "${ssh_ip} BW实时日志收集完成。"
			fi
			
		fi
	done
}


#################################################实时日志获取（指定板卡的实时日志）###################################
#实时日志获取（指定板卡的实时日志）
## $1 板卡ip
CollectRealtimeLogByIp()
{
	#获取全部板卡的芯片数，用于计算实时日志所需的空间大小
	local chipNum=$(getChipNumByIp $1)
	#校验IAM的空间是否满足
	ret=$(checkIamLogDirSize ${chipNum} 0)
	# 检查SshCmd是否成功执行
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM空间检查异常，无法执行日志收集操作。"
		return 1
	fi
	#开始收集IAM日志
	ret=$(getIamRealLog) #内部存在日志
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM实时日志收集失败。"
		return 1
	fi
	#开始收集板卡日志
	local str_engine_ssh_info=` sqlite3 ${CHECK_DB_FILE} "select * from ia_engine;" `
    IFS=$'\n' read -d '' -ra arr_engine_ssh_info <<< "$str_engine_ssh_info"
	for engien_ssh_info_line in "${arr_engine_ssh_info[@]}"
	do
		local ssh_ip=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $1}')
		if [ ${ssh_ip} == "$1" ]; then
			local ssh_port=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $2}')
			local ssh_user_name=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $3}')
			local ssh_passwd=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $4}')
			#获取板卡芯片列表
			ret=$(getChipIdArry "${ssh_ip}")
			if [ ${#chip_ids_array[@]} -eq 0 ]; then
				LOG "IAM" "获取板卡[${ssh_ip}]的芯片列表失败，不进行实时日志收集"
			else
				LOG "IAM" "板卡[${ssh_ip}]将收集芯片[${chip_ids_array[@]}]的实时日志"
				#校验IAE的实时日志目录是否满足
				ret=$(checkBoardRealLogDirSize ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd} "${#chip_ids_array[@]}")
				if [ $? -ne 0 ]; then
					LOG "IAM" "${ssh_ip}实时日志目录剩余空间不满足条件，无法收集。"
					break
				fi
				#iae的实时日志
				ret=$(getIaeRealLog ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd} "${chip_ids_array[*]}") 
				if [ $? -ne 0 ]; then
					LOG "IAM" "${ssh_ip}iae实时日志收集失败。"
				fi
				#bw的实时日志
				ret=$(getBwRealLog ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd}) 
				if [ $? -ne 0 ]; then
					LOG "IAM" "${ssh_ip}bw实时日志收集失败。"
				fi
			fi
			break
		fi
	done
}
#################################################实时日志获取（指定板卡和芯片号的实时日志）###################################
#实时日志获取
##$1 板卡ip
##$2 芯片号数组
CollectRealtimeLogByIpAndChipNum()
{
	# 使用 eval 导入数组
    eval "chip_array=($1)"
	#获取全部板卡的芯片数，用于计算实时日志所需的空间大小
	local chipNum=${#chip_array[@]}
	#校验IAM的空间是否满足
	ret=$(checkIamLogDirSize ${chipNum} 0)
	# 检查SshCmd是否成功执行
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM空间检查异常，无法执行日志收集操作。"
		return 1
	fi
	#开始收集IAM日志
	ret=$(getIamRealLog) #内部存在日志
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM实时日志收集失败。"
		return 1
	fi
	#开始收集板卡日志
	local str_engine_ssh_info=` sqlite3 ${CHECK_DB_FILE} "select * from ia_engine;" `
    IFS=$'\n' read -d '' -ra arr_engine_ssh_info <<< "$str_engine_ssh_info"
	for engien_ssh_info_line in "${arr_engine_ssh_info[@]}"
	do
		local ssh_ip=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $1}')
		if [ ${ssh_ip} == "$1" ]; then
			local ssh_port=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $2}')
			local ssh_user_name=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $3}')
			local ssh_passwd=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $4}')
			#获取板卡芯片列表
			ret=$(getChipIdArry "${ssh_ip}")
			if [ ${#chip_array[@]} -eq 0 ]; then
				LOG "IAM" "获取板卡[${ssh_ip}]的芯片列表失败，不进行实时日志收集"
			else
				LOG "IAM" "板卡[${ssh_ip}]将收集芯片[${chip_ids_array[@]}]的实时日志"
				#校验IAE的实时日志目录是否满足
				ret=$(checkBoardRealLogDirSize ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd} "${#chip_array[@]}")
				if [ $? -ne 0 ]; then
					LOG "IAM" "${ssh_ip}实时日志目录剩余空间不满足条件，无法收集。"
					break
				fi
				#iae的实时日志
				ret=$(getIaeRealLog ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd} "${chip_array[*]}") 
				if [ $? -ne 0 ]; then
					LOG "IAM" "${ssh_ip}iae实时日志收集失败。"
				fi
				#bw的实时日志
				ret=$(getBwRealLog ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd}) 
				if [ $? -ne 0 ]; then
					LOG "IAM" "${ssh_ip}bw实时日志收集失败。"
				fi
			fi
			break
		fi
	done
}

#################################################备份日志获取（全部板卡的备份日志，按照时间间隔1、2、3）###################################
#获取备份日志
##$1 备份日志的时间（1、2、3）
CollectAllBackupLog()
{
	#获取全部板卡的芯片数，用于计算实时日志所需的空间大小
	local boardNum=$(getAllBoardNum)
	LOG "IAM" "获取全部板卡的数为: ${boardNum}"
	#校验IAM的空间是否满足
	ret=$(checkIamLogDirSize ${boardNum} $1)
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM空间检查异常，无法执行日志收集操作。"
		return 1
	fi
	#开始收集IAM日志
	ret=$(getIamBackupLog $1) #内部存在日志
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM备份日志收集失败。"
	else
		LOG "IAM" "IAM备份日志收集完成。"
	fi
	#开始收集板卡日志
	local str_engine_ssh_info=` sqlite3 ${CHECK_DB_FILE} "select * from ia_engine;" `
    IFS=$'\n' read -d '' -ra arr_engine_ssh_info <<< "$str_engine_ssh_info"
	for engien_ssh_info_line in "${arr_engine_ssh_info[@]}"
	do
		local ssh_ip=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $1}')
		local ssh_port=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $2}')
		local ssh_user_name=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $3}')
		local ssh_passwd=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $4}')
		ret=$(getBoardBackupLog ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd} $1) 
		if [ $? -ne 0 ]; then
			LOG "${ssh_ip}" "${ssh_ip} IAE备份日志收集失败。"
		else
			LOG "${ssh_ip}" "${ssh_ip} IAE备份日志收集完成。"
		fi
	done
	
}

#################################################备份日志获取（指定板卡的备份日志，按照时间间隔1、2、3）###################################
#获取备份日志
##$1 备份日志的时间（1、2、3）
##$2 板卡IP
CollectAllBackupLogbyIp()
{
	#获取全部板卡的芯片数，用于计算实时日志所需的空间大小
	local boardNum=1
	#校验IAM的空间是否满足
	ret=$(checkIamLogDirSize ${boardNum} $1)
	# 检查SshCmd是否成功执行
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM空间检查异常，无法执行日志收集操作。"
		return 1
	fi
	#开始收集IAM日志
	ret=$(getIamBackupLog $1) #内部存在日志
	if [ $? -ne 0 ]; then
		LOG "IAM" "IAM备份日志收集失败。"
		return 1
	fi
	#开始收集板卡日志
	local str_engine_ssh_info=` sqlite3 ${CHECK_DB_FILE} "select * from ia_engine;" `
    IFS=$'\n' read -d '' -ra arr_engine_ssh_info <<< "$str_engine_ssh_info"
	for engien_ssh_info_line in "${arr_engine_ssh_info[@]}"
	do
		local ssh_ip=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $1}')
		if [ ${ssh_ip} == "$2" ]; then
			local ssh_port=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $2}')
			local ssh_user_name=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $3}')
			local ssh_passwd=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $4}')
			ret=$(getBoardBackupLog ${ssh_ip} ${ssh_port} ${ssh_user_name} ${ssh_passwd} $1) 
			if [ $? -ne 0 ]; then
				LOG "IAM" "${ssh_ip}iae备份日志收集失败。"
			fi
			break
		fi
		
	done
}


#################################################函数入口###################################
# 初始化相关操作
CollectLogFileInit
LOG "IAM" "初始化日志目录完成"
	
LOG "IAM" "开始加载IAM数据库文件"
LoadCheckDbFile
LOG "IAM" "加载IAM数据库文件完成"
	
LOG "IAM" "开始加载板卡登录信息"
LoadIaeIpAndPasswd
LOG "IAM" "加载板卡登录信息完成"

LOG "IAM" "开始加载KUNLUN2的IP信息"
LoadKunlun2Tx1IpInfo
LOG "IAM" "结束加载KUNLUN2的IP信息"

#收集全部的实时日志和备份日志
CollectAllRealtimeLog

# 提示用户是否收集备份日志
read -p "是否收集备份日志 y/n: " response

# 检查用户输入
if [[ "$response" == "y" ]]; then
    # 提示用户输入天数，默认是1
    read -p "请输入收集备份日志的天数，默认是1，最大为3: " days

    # 如果用户没有输入天数，则默认为1
    if [[ -z "$days" ]]; then
        days=1
    fi
	
	# 检查输入的天数是否大于3
    if (( days > 3 )); then
        echo "输入的天数不能大于3,当前仅收集3天内的备份日志，强制设置为3"
		days=3
    fi
	
    # 输出用户选择的天数
	# 获取当前日期
	current_date=$(date +%Y-%m-%d)
	# 计算天数前的日期
	start_date=$(date -d "$current_date - $days days" +%Y-%m-%d)
    echo "备份日志选取的时间为: 【$start_date】到【$current_date】"	
	
	CollectAllBackupLog $days
else
    echo "未选择收集备份日志"
fi

#红色标识的字体
RED_BLINK='\e[31;5m' #红色闪烁
RED='\e[31m' #红色
NC='\e[0m'  # No Color
echo -e "${RED_BLINK}请注意：${NC}${RED}将${COLLECT_LOG_ROOT_PATH}目录拷贝给对应研发人员，并手动删除该目录${NC}"




