#!/bin/bash

### 获取BW版本号
GetBwVersion()
{
    LOG 1 "开始获取【$1】服务器上的BW版本/补丁号"
    local bw_version=""
	local patch_exist=` SshCmd $1 $2 $3 $4 "[ -f '/usr/local/svconfig/patch/iapatchsys.conf' ] && echo 'true' || echo 'false'" `
	if [ "${patch_exist}" == "true" ]
	then
	    local cmd="displayiapatch.sh | grep PHVERO | grep IA[0-9]"
	    local bw_version_extend_info=` SshCmd $1 $2 $3 $4 "displayiapatch.sh | grep PHVERO | grep IA[0-9]" `
		RecordCommndInfo "$1" "${cmd}" "${bw_version_extend_info}"
		local bw_version_cut_head=${bw_version_extend_info##*-}
		local bw_version=${bw_version_cut_head%_*}
	else
	    local cmd="bwcfgtool.sh -v | grep HCVERO"
	    local bw_version_extend_info=` SshCmd $1 $2 $3 $4 "bwcfgtool.sh -v | grep HCVERO" `
		RecordCommndInfo "$1" "${cmd}" "${bw_version_extend_info}"
	    local bw_version_cut_head=${bw_version_extend_info##*-}
	    local bw_version=${bw_version_cut_head%_*}
	fi
	echo "${bw_version}"
}

### 获取IAE版本号
GetIaeVersion()
{
    LOG 1 "开始获取【$1】服务器上的IAE版本/补丁号"
    local iae_version=""
	local patch_exist=` SshCmd $1 $2 $3 $4 "[ -f '/usr/local/svconfig/patch/iapatchsys.conf' ] && echo 'true' || echo 'false'" `
	if [ "${patch_exist}" == "true" ]
	then
	    local cmd="displayiapatch.sh | grep PHVERO | grep IA[0-9]"
	    local iae_version_extend_info=` SshCmd $1 $2 $3 $4 "displayiapatch.sh | grep PHVERO | grep IA[0-9]" `
		RecordCommndInfo "$1" "${cmd}" "${iae_version_extend_info}"
		local iae_version_cut_head=${iae_version_extend_info##*-}
		local iae_version=${iae_version_cut_head%_*}
	else
	    local cmd="iaecfgtool.sh -v | grep HCVERO"
	    local iae_version_extend_info=` SshCmd $1 $2 $3 $4 "iaecfgtool.sh -v | grep HCVERO" `
		RecordCommndInfo "$1" "${cmd}" "${iae_version_extend_info}"
	    local iae_version_cut_head=${iae_version_extend_info##*-}
	    local iae_version=${iae_version_cut_head%_*}
	fi
	echo "${iae_version}"
}

### 服务状态检查
### $1 bw/iae 进程服务状态
### 返回值0：状态检查正常，1：状态检查异常
GetErrorServerStatus()
{
	local error_process=""
	while read iae_server_status_line
	do
	    cmd=` grep Running <<< "${iae_server_status_line}" `
	    if [ $? -ne 0 ]
		then
			if [ -z "$error_process" ];then #第一次进入
				error_process=$(echo "${iae_server_status_line}" | awk ' {print $1} ')
			else
				local error_process_name=$(echo "${iae_server_status_line}" | awk ' {print $1} ')
				error_process="${error_process}，${error_process_name}"
			fi
	    fi
	done <<< "$1"
	echo "${error_process}"
}

### iae服务状态检查
### $1 板卡服务器IP
### $2 板卡服务器登录端口
### $3 板卡服务器登录用户名
### $4 板卡服务器登录密码
IaeServerStatusCheck()
{
    LOG 1 "开始检查IAE服务状态"
	local cmd="iaeserver.sh status | grep Service"
    local str_iae_server_status=` SshCmd $1 $2 $3 $4 "iaeserver.sh status | grep Service"`
	RecordCommndInfo "$1" "${cmd}" "${str_iae_server_status}"
    if [ -z "${str_iae_server_status}" ]
    then
        ResultTxtOutput "WARNING" "【$1】 IAE服务状态查询异常"
    else 
        local error_msg=` GetErrorServerStatus "${str_iae_server_status}" `
        if [ -z "${error_msg}" ]
        then
		    ResultTxtOutput "INFO" "【$1】 IAE服务状态正常"
        else
			error_msg=` echo -e "IAE服务状态异常：${error_msg}" `
            ResultTxtOutput "ERROR" "【$1】 ${error_msg}"
        fi
    fi
	
}

### bw服务状态检查
### $1 板卡服务器IP
### $2 板卡服务器登录端口
### $3 板卡服务器登录用户名
### $4 板卡服务器登录密码
BwServerStatusCheck()
{
    LOG 1 "开始检查BW服务状态"
	local cmd="bwserver.sh status | grep Service"
    local str_bw_server_status=` SshCmd $1 $2 $3 $4 "bwserver.sh status | grep Service"`
	RecordCommndInfo "$1" "${cmd}" "${str_bw_server_status}"
    if [ -z "${str_bw_server_status}" ]
    then
        ResultTxtOutput "WARNING" "【$1】 BW服务状态查询异常"
    else
        local error_msg=` GetErrorServerStatus "${str_bw_server_status}" `
        if [ -z "${error_msg}" ]
        then
			ResultTxtOutput "INFO" "【$1】 BW服务状态正常"
        else
           error_msg=` echo -e "BW服务状态异常:${error_msg}" `
           ResultTxtOutput "ERROR" "【$1】 ${error_msg}"
        fi
    fi
}

############################ iam 检 查 测 试 代 码 ############################
# IaeServerStatusCheck "206.206.124.89" "22" "root" "Zhineng_1"
# BwServerStatusCheck "206.206.124.89" "22" "root" "Zhineng_1"
# GetIaeVersion "206.206.124.89" "22" "root" "Zhineng_1"
# GetBwVersion "206.206.124.89" "22" "root" "Zhineng_1"
# GetIaeVersion "206.206.124.175" "22" "root" "admin_123"
# GetBwVersion "206.206.124.175" "22" "root" "admin_123"
############################ iam 检 查 测 试 代 码 ############################