#!/bin/bash

### iam业务数据库访问操作
### $1 数据库类型，sqlite/pg
### $2 sql查询语句
SelectDB()
{
	
	local result=""
	case "$1" in
		"pg")
			result=` /home/postgres/pgsql/bin/psql "host=127.0.0.1 port=${IAM_PG_PORT} user=postgres password=${IAM_PG_PASSWD} dbname=ias" << EOF
$2;
EOF`
			;;
		"sqlite")
			result=` sqlite3 /opt/wise/ia12k/db/ias.db << EOF
$2;
EOF`
			;;
		*)
			echo "the db($1) is not supported"
			;;
	esac
    echo "${result}"
	LOG "IAM" "${result}"
}

### 远程命令封装
### $1 远程ip
### $2 远程端口
### $3 远程用户
### $4 远程密码
### $5 远程命令
SshCmd ()
{
	cmd=$(timeout 40s ${SSHPASS} -p $4 ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p $2 $3@$1 "$5" 2>/dev/null)
	timeout_exit_status=$?
	# 根据退出状态码输出相应的日志
	if [ "$timeout_exit_status" -ne 0 ]; then
		LOG "$1" "sshpass执行命令【$5】返回码【$timeout_exit_status】" "${cmd}"
		cmd=""
	fi
	echo "${cmd}"
    return $timeout_exit_status
}

#获取板卡类型
# $1 板卡IP
Get_Engine_Name()
{
        #获取板卡类型
        engine_type=$(sqlite3 /opt/wise/ia12k/db/ias.db "select engine_version from ia_engine where address = '$1' group by engine_version")

        case $engine_type in
        "1")
                echo "昆仑2"
        ;;
        "2")
                echo "昆仑3"
        ;;
        "3")
                echo "GPU"
        ;;
        "4")
                echo "寒武纪"
        ;;
    "7")
    echo "AX"
    ;;
        esac
}


#获取板卡芯片数量
# $1 板卡IP
Get_Chip_Num()
{
	chipNum=$(sqlite3 /opt/wise/ia12k/db/ias.db "select count(*) from ia_engine where address = '$1';")
	chipNum=$((chipNum-1))
	echo "$chipNum"
}

### 文件和文件夹初始化
CollectLogFileInit()
{
	ret=` rm -rf ${COLLECT_LOG_ROOT_PATH}/$1 2>/dev/null `
    # 日志收集的目录，不需要创建，收集过程中会自动清空删除
    [ ! -d "${DATA_PATH}" ] && mkdir "${DATA_PATH}"
    [ ! -d "${COLLECT_LOG_ROOT_PATH}" ] && mkdir -p "${COLLECT_LOG_ROOT_PATH}"
	[ ! -f "${CLLOECT_LOG_FILE}" ] && touch "${CLLOECT_LOG_FILE}"
	# 清空日志文件
	> "${CLLOECT_LOG_FILE}"
}

#########获取芯片ip映射#######
##$1 板卡列表信息列表
Get_Chip_Ip()
{
	#遍历板卡列表
	for engien_ssh_info_line in "$@"
	do
		local ssh_ip=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $1}')
		local ssh_port=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $2}')
		local ssh_user_name=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $3}')
		local ssh_passwd=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $4}')
		local engine_name=$(Get_Engine_Name "$ssh_ip")
		if [ "${engine_name}" == "昆仑2" ];then
			# 获取昆仑2板卡芯片数
			local prototype_name=` SshCmd $ssh_ip $ssh_port $ssh_user_name $ssh_passwd " cat /tmp/.manuinfo | grep PROTOTYPE_NAME" 2>/dev/null `
			if [ -z "${prototype_name}" ];then
				LOG "IAM" "昆仑2板卡IP【${ssh_ip}】获取板卡类型${prototype_name}为空，昆仑2板卡【${ssh_ip}】无法获取实时日志失败"
				return 1
			fi
			LOG "IAM" "昆仑2 【$ssh_ip】prototype_name的值是${prototype_name}"
			if [ "$prototype_name" != "" ];then
				local prototypeRet=$(echo $prototype_name | grep "FB-IA12500-B")
				if [ "$prototypeRet" != "" ];then
					prototype_name="tx1_5"
				elif [ -n "$(echo $prototype_name | grep "FB-IA12500-S")" ];then
					prototype_name="tx1_2"
				else
					prototype_name="tx1_10"
				fi
				LOG "IAM" "昆仑2板卡IP【${ssh_ip}】获取板卡类型为${prototype_name}"
			else
				LOG "IAM" "昆仑2板卡IP【${ssh_ip}】获取板卡类型${prototype_name}为空，昆仑2板卡【${ssh_ip}】无法获取实时日志失败"
			fi
			
			if [ "${prototype_name}" == "tx1_5" ]
			then    # 5芯片昆仑2，第一颗芯片ip：192.168.255.11
				BOARD_CHIP_IP[$ssh_ip-0]="192.168.255.11"
				BOARD_CHIP_IP[$ssh_ip-1]="192.168.255.13"
				BOARD_CHIP_IP[$ssh_ip-2]="192.168.255.15"
				BOARD_CHIP_IP[$ssh_ip-3]="192.168.255.17"
				BOARD_CHIP_IP[$ssh_ip-4]="192.168.255.19"
			elif [ "${prototype_name}" == "tx1_2" ]
			then	# 2芯片昆仑2，第一颗芯片ip：192.168.255.17
				BOARD_CHIP_IP[$ssh_ip-0]="192.168.255.17"
				BOARD_CHIP_IP[$ssh_ip-1]="192.168.255.19"
			else
				BOARD_CHIP_IP[$ssh_ip-0]="192.168.255.1"
				BOARD_CHIP_IP[$ssh_ip-1]="192.168.255.3"
				BOARD_CHIP_IP[$ssh_ip-2]="192.168.255.5"
				BOARD_CHIP_IP[$ssh_ip-3]="192.168.255.7"
				BOARD_CHIP_IP[$ssh_ip-4]="192.168.255.9"
				BOARD_CHIP_IP[$ssh_ip-5]="192.168.255.11"
				BOARD_CHIP_IP[$ssh_ip-6]="192.168.255.13"
				BOARD_CHIP_IP[$ssh_ip-7]="192.168.255.15"
				BOARD_CHIP_IP[$ssh_ip-8]="192.168.255.17"
				BOARD_CHIP_IP[$ssh_ip-9]="192.168.255.19"
			fi
		fi	
	done
}

### 加载数据库文件
LoadCheckDbFile()
{
   LOG "IAM" "开始加载数据库文件"
   local db_engine_load_result=` sqlite3 ${CHECK_DB_FILE} "CREATE TABLE IF NOT EXISTS ia_engine(ip TEXT PRIMARY KEY, port TEXT, username TEXT, passwd TEXT);" `
   local db_config_load_result=` sqlite3 ${CHECK_DB_FILE} "CREATE TABLE IF NOT EXISTS check_config(item TEXT PRIMARY KEY, value TEXT);" `
   LOG "IAM" "数据库文件加载完成"
}

### 移除用不到的ssh登录信息
RemoveSshInfo()
{
    local remove_ssh=""
    local iae_ip_in_iam=` sqlite3 ${IAS_DB} "SELECT address FROM ia_engine GROUP BY address" `
    local arr_iae_ip_in_iam=(${iae_ip_in_iam})
	local hitsory_iae_ip=` sqlite3 ${CHECK_DB_FILE} "SELECT ip FROM ia_engine" `
    local arr_history_iae_ip_info=(${hitsory_iae_ip})
    for history_iae_ip in "${arr_history_iae_ip_info[@]}"
    do
        local need_remove_ssh_info="true"
        for iae_ip_in_iam in "${arr_iae_ip_in_iam[@]}"
        do
            if [ "${history_iae_ip}" == "${iae_ip_in_iam}" ]
            then
                need_remove_ssh_info="false"
            fi
        done
        if [ "true" == "${need_remove_ssh_info}" ]
        then
            cmd=` sqlite3 ${CHECK_DB_FILE} "DELETE FROM ia_engine WHERE ip='${history_iae_ip}';" `
            LOG "IAM" "${history_iae_ip} 登录信息已被移除"
        fi
    done
}

### 加载ssh登录信息
LoadIaeIpAndPasswd()
{ 
    LOG "IAM" "开始加载ssh登录信息"
	# 移除iam上未挂载的板卡登录信息
	RemoveSshInfo
    local iae_in_iam=` sqlite3 ${IAS_DB} "SELECT address FROM ia_engine GROUP BY address" `
	local iae_address_vec=(${iae_in_iam})
    local ssh_port=0
    local ssh_user_name=""
    local ssh_password=""
    # 检验板卡登录信息
	for((index=0; index<${#iae_address_vec[*]}; index++))
	do 
        retry_num=0
	    local current_iae_ip="${iae_address_vec[$index]}"
	    local current_iae_login_info=` sqlite3 ${CHECK_DB_FILE} "SELECT * FROM ia_engine WHERE ip = '${current_iae_ip}';" `
		if [ -z "${current_iae_login_info}" ]
	    then
            # 尝试用上一次输入的密码进行登录，如果登录成功，不在要求用户手动输入登录信息
            ${SSHPASS} -p ${ssh_password} ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o ConnectTimeout=10 -p ${ssh_port} ${ssh_user_name}@${current_iae_ip} " " &> /dev/null
            if [ $? -eq 0 ]
            then
                local insert_result=` sqlite3 ${CHECK_DB_FILE} "INSERT INTO ia_engine(ip,port,username,passwd) VALUES('${current_iae_ip}','${ssh_port}','${ssh_user_name}','${ssh_password}')" `
                continue
            fi
            
		    # 输入ssh登录信息
		    echo "${current_iae_ip} login info is empty, please input"
		    echo "please input ssh_port: "
			read ssh_port
			echo "please input ssh_user_name: "
		    read ssh_user_name
			echo "please input ssh_password: "
			read ssh_password
			while :
			do
			    ${SSHPASS} -p ${ssh_password} ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o ConnectTimeout=10 -p ${ssh_port} ${ssh_user_name}@${current_iae_ip} " " &> /dev/null
			    if [ $? -ne 0 ]
		        then
                    retry_num=$(expr $retry_num + 1)
                    if [ $retry_num -ge 2 ]
                    then
                        echo "${current_iae_ip} login failed num: ${retry_num}，will don't be saved"
                        break
                    fi
                    
    			    echo "${current_iae_ip} login info is error, please reset"
	    			echo "please update ssh_port: "
		    	    read ssh_port
			        echo "please update ssh_user_name: "
		            read ssh_user_name
			        echo "please update ssh_password: "
			        read ssh_password
			    else
				    echo "${current_iae_ip} login info will be saved"
				    break
			    fi
                
			done
            if [ $retry_num -lt 2 ]
            then 
                local insert_result=` sqlite3 ${CHECK_DB_FILE} "INSERT INTO ia_engine(ip,port,username,passwd) VALUES('${current_iae_ip}','${ssh_port}','${ssh_user_name}','${ssh_password}')" `
            fi
		else
            # 尝试用上一次输入的密码进行登录，如果登录成功，不在要求用户手动输入登录信息
            ${SSHPASS} -p ${ssh_password} ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o ConnectTimeout=10 -p ${ssh_port} ${ssh_user_name}@${current_iae_ip} " " &> /dev/null
            if [ $? -eq 0 ]
            then
                local update_result=` sqlite3 ${CHECK_DB_FILE} "UPDATE ia_engine SET port='${ssh_port}', username='${ssh_user_name}', passwd='${ssh_password}' WHERE ip='${current_iae_ip}'" `
                continue
            fi
            
			# iae 登录信息已经保存在了数据库里，则校验ssh登录密码，如不正确，需要手动修改
			local current_iae_login_info_arr=(${current_iae_login_info//|/ })
			local ssh_port=${current_iae_login_info_arr[1]}
			local ssh_user_name=${current_iae_login_info_arr[2]}
			local ssh_user_password=${current_iae_login_info_arr[3]}
			${SSHPASS} -p "${ssh_user_password}" ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o ConnectTimeout=10 -p "${ssh_port}" "${ssh_user_name}"@"${current_iae_ip}" " " &> /dev/null
			if [ $? -ne 0 ]
		    then
			    echo "${current_iae_ip} login info is error, please update"
				echo "please update ssh_port: "
			    read ssh_port
			    echo "please update ssh_user_name: "
		        read ssh_user_name
			    echo "please update ssh_password: "
			    read ssh_password
				while :
			    do
			        ${SSHPASS} -p "${ssh_password}" ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o ConnectTimeout=10 -p "${ssh_port}" "${ssh_user_name}"@"${current_iae_ip}" " " &> /dev/null
			        if [ $? -ne 0 ]
		            then
                        if [ $retry_num -ge 2 ]
                        then
                            echo "${current_iae_ip} login failed num: %{retry_num}，will don't be updated"
                            break
                        fi
    			        echo "${current_iae_ip} login info is error, please reset"
	    			    echo "please update ssh_port: "
		    	        read ssh_port
			            echo "please update ssh_user_name: "
		                read ssh_user_name
			            echo "please update ssh_password: "
			            read ssh_password
					else
					    echo "${current_iae_ip} login info will be updated"
					    break
			        fi
			    done
                if [ $retry_num -lt 2 ]
                then
                    echo "will udpate db"
                    local update_result=` sqlite3 ${CHECK_DB_FILE} "UPDATE ia_engine SET port='${ssh_port}', username='${ssh_user_name}', passwd='${ssh_password}' WHERE ip='${current_iae_ip}'" `
                fi
			fi
		fi
	done
	
}

### 昆仑2的芯片IP
LoadKunlun2Tx1IpInfo()
{
	local str_engine_ssh_info=` sqlite3 ${CHECK_DB_FILE} "select * from ia_engine;" ` 
    IFS=$'\n' read -d '' -ra arr_engine_ssh_info <<< "$str_engine_ssh_info"
	#初始化KUNLUN2的板卡登录信息
	LOG "IAM" "加载昆仑2的板卡登录信息"
	Get_Chip_Ip "${arr_engine_ssh_info[@]}"
	# 判断数组是否为空
	if [ ${#BOARD_CHIP_IP[@]} -eq 0 ]; then
		LOG "IAM" "加载昆仑2的板卡登录信息为空"
	else
		LOG "IAM" "加载昆仑2的板卡登录信息数量: ${#BOARD_CHIP_IP[@]}"
	fi
	LOG "IAM" "加载昆仑2的板卡登录信息完成"
}

### 日志输出
### $1 IP地址（IAM或者板卡IP地址）
### $2 日志内容
LOG()
{
	current_time=` date "+%Y-%m-%d %H:%M:%S" `
	echo "${current_time} [$1] $2"
	echo -e "${current_time} [$1] $2" >> "${CLLOECT_LOG_FILE}"
}


# 处理目录函数
### $1 目录地址
handleDir() 
{
    local dir="$1"
    if [ -d "$dir" ]; then
        # 如果目录存在，清空目录内容
        rm -rf "$dir"/*
    else
        # 如果目录不存在，创建目录
        mkdir -p "$dir"
    fi
}

### 远程rsync命令封装,不进行二次校验
### $1 远程ip
### $2 远程端口
### $3 远程用户
### $4 远程密码
### $5 远方文件绝对路径
### $6 本地目录
rsyncCmd ()
{
	cmd=$(timeout 40s ${SSHPASS} -p $4 rsync -az -e "ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p $2" $3@$1:$5 $6 2>/dev/null)
	timeout_exit_status=$?
	# 根据退出状态码输出相应的日志
	if [ "$timeout_exit_status" -ne 0 ]; then
		LOG "$1" "rsync命令拷贝服务器[$1]的[$5]到本地[$6]失败，返回错误码【$timeout_exit_status】"
	fi
    return $timeout_exit_status
}

### 远程scp命令封装,不进行二次校验
### $1 远程ip
### $2 远程端口
### $3 远程用户
### $4 远程密码
### $5 远方文件绝对路径
### $6 本地目录
scpCmd ()
{
	cmd=$(timeout 40s ${SSHPASS} -p $4 scp -r -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -P $2 $3@$1:$5 $6 2>/dev/null)
	timeout_exit_status=$?
	# 根据退出状态码输出相应的日志
	if [ "$timeout_exit_status" -ne 0 ]; then
		LOG "$1" "scp命令拷贝服务器[$1]的[$5]到本地[$6]失败，返回错误码【$timeout_exit_status】"
	fi
    return $timeout_exit_status
}