#!/bin/bash
declare COLLECT_LOG_ROOT_PATH="${DATA_PATH}/collect_log" # 收集日志根目录
declare COLLECT_IAE_REALTIME_LOG_PATH="${COLLECT_LOG_ROOT_PATH}/iae_realtime_log" # 收集日志IAE实时日志目录
declare COLLECT_BW_REALTIME_LOG_PATH="${COLLECT_LOG_ROOT_PATH}/bw_realtime_log" # 收集日志BW实时日志目录
declare COLLECT_IAM_REALTIME_LOG_PATH="${COLLECT_LOG_ROOT_PATH}/iam_realtime_log" # 收集日志BW实时日志目录
declare COLLECT_IAE_BACKUP_LOG_PATH="${COLLECT_LOG_ROOT_PATH}/backup_log/iae" # 收集IAE备份日志目录(包括bw和iae)
declare COLLECT_IAM_BACKUP_LOG_PATH="${COLLECT_LOG_ROOT_PATH}/backup_log/iam" # 收集IAM备份日志目录
#定义日志目录，不同版本稍有不同，当前按照58版本给出
declare BORAD_BACKUP_LOG_PATH="/var/backuplog" # 板卡备份日志目录
declare IAM_BACKUP_LOG_PATH="/var/backuplog" # 板卡备份日志目录
declare IAM_REALTIME_LOG_PATH="/var/log/iamlog" # iam实时日志目录
declare BW_REALTIME_LOG_PATH="/var/log/proxylog" # bw实时日志目录
declare IAE_REALTIME_LOG_PATH="/var/log/ialog" # IAW实时日志目录
#目录校验使用
declare IAE_SIGNALE_TX1_REALLOG_NEED_SIZE=80 #单芯片实时日志所需要的空间默认80M(压缩后的日志大小)
declare IAE_ONEDAY_BACKUPLOG_NEED_SIZE=300 #单芯片实时日志所需要的空间默认300M
#日志收集脚本输出的日志文件
declare CLLOECT_LOG_FILE="${COLLECT_LOG_ROOT_PATH}/run.log" # 脚本运行日志文件


#################################################实时日志获取逻辑###################################
### 芯片内部实时日志压缩相关日志文件到映射目录
### $1板卡类型名称
### $2芯片编号
### $3板卡IP
getTx1ReallogCopyCommand()
{
	local command=""
	#获取芯片系统IP后缀
	local ipLastIndex=$((($2+1)*2-1))
	#组织命令
    case $1 in
    "GPU")
        LOG "GPU不需要进入芯片拷贝实时日志"
        ;;
    "AX")
        command=" ssh 192.168.254.${ipLastIndex} \"cd ${IAE_REALTIME_LOG_PATH};tar -zcf /opt/wise_iae/ia12k/data$2/log/$2-$(date +%Y%m%d%H%M%S)-realtime.tar.gz iae*.log IntelligentAnalyzeSystemLogs\" " 
        ;;
    "寒武纪")
		command=" ssh 192.168.255.${ipLastIndex} \"cd ${IAE_REALTIME_LOG_PATH};tar -zcf /opt/wise_iae/ia12k/data$2/log/$2-$(date +%Y%m%d%H%M%S)-realtime.tar.gz iae*.log IntelligentAnalyzeSystemLogs\" "
        ;;
    "昆仑2")
		command=" ssh ${BOARD_CHIP_IP[$3-$2]} \"cd ${IAE_REALTIME_LOG_PATH};mkdir -p /program/nfs_share/collect_log;tar --warning=no-file-changed -zcf /program/nfs_share/collect_log/$2-$(date +%Y%m%d%H%M%S)-realtime.tar.gz iae*.log IntelligentAnalyzeSystemLogs\" "
        ;;
    "昆仑3")
		command=" ssh 192.168.255.${ipLastIndex} \"cd ${IAE_REALTIME_LOG_PATH};tar -zcf /opt/wise_iae/ia12k/data$2/log/$2-$(date +%Y%m%d%H%M%S)-realtime.tar.gz iae*.log IntelligentAnalyzeSystemLogs\" "
        ;;
    *)
        LOG "不支持的新类型：$1"
	    ;;
   esac
   echo ${command}  
 }
 
# 获取iae芯片对应的日志
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码
##$5 芯片编号数组
getIaeRealLog()
{
	LOG "$1" "开始收集板卡【$1】的IAE实时日志"
	local bRet=0
	# 转换为数组
	local chip_numbers_array=($5)
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)  #######################测试的时候需要临时修改
	#第一步：拷贝芯片内实时日志到板卡服务器目录
	if [ "$engine_name" != "GPU" ];then
		for index in "${chip_numbers_array[@]}"; 
		do
			#获取拷贝命令
			local command=$(getTx1ReallogCopyCommand "$engine_name" "$index" "$1")
			if [ -z "$command" ];then
				LOG "$1" "板卡[$1]芯片[$index]拷贝实时日志命令为空"
				continue
			fi
			#执行命令
			ret=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
			#判断本地文件是否生成，防止异常报错（暂时不增加逻辑，文件不存在下载就会失败后面进行判断，当前仅增加日志提示）
			ret_status=$?
			# 检查输出是否为空
			if [ -n "$ret" ]; then
				LOG "$1" "板卡[$1]芯片[$index]拷贝实时日志命令[$command]执行返回异常信息[$ret]"
				# 检查退出状态码
				if [ $ret_status -ne 0 ]; then
					LOG "$1" "板卡[$1]芯片[$index]拷贝实时日志命令[$command]执行失败，错误码[$ret_status]"
				else
					LOG "$1" "板卡[$1]芯片[$index]拷贝实时日志命令[$command]执行成功，警告信息[$ret]"
				fi
			else
				LOG "$1" "板卡[$1]芯片[$index]拷贝实时日志命令[$command]执行成功"
			fi
		done
	fi
	#第二步：删除IAM对应IP的目录
	ret=` rm -rf ${COLLECT_IAE_REALTIME_LOG_PATH}/$1 2>/dev/null `
	ret=` mkdir -p ${COLLECT_IAE_REALTIME_LOG_PATH}/$1 2>/dev/null `
	#第三步：分芯片拷贝实时日志到IAM的日志目录
	local succFileNum=0
	local failFileNum=0
	local remote_file="" #远方文件
	local local_file=""  #本地文件绝对路径
	for index in "${chip_numbers_array[@]}"; 
	do
		local_file="${COLLECT_IAE_REALTIME_LOG_PATH}/$1"
		if [ "$engine_name" == "昆仑2" ];then
			remote_file="/program/nfs_share/collect_log/${index}-*-realtime.tar.gz"
		elif [ "$engine_name" == "GPU" ];then
			remote_file="/opt/wise_iae/ia12k/data${index}/log/*"
			local_file="${COLLECT_IAE_REALTIME_LOG_PATH}/$1/GPU-${index}"
		else
			remote_file="/opt/wise_iae/ia12k/data${index}/log/${index}-*-realtime.tar.gz"
		fi
		#获取拷贝命令
		ret=$(rsyncCmd $1 $2 $3 $4 "${remote_file}" "${local_file}")
		ret_status=$?
		if [ "$ret_status" -ne 0 ]; then
			LOG "$1" "板卡[$1]芯片[$index]rsync拷贝日志到本地${local_file}失败，错误码[$ret_status]，尝试使用scp命令"
			ret=$(scpCmd $1 $2 $3 $4 "${remote_file}" "${local_file}")
			ret_status=$?
			if [ "$ret_status" -ne 0 ]; then
				LOG "$1" "板卡[$1]芯片[$index]scp拷贝日志到本地${local_file}失败，错误码[$ret_status]"
				failFileNum=$((failFileNum + 1))
				continue
			else
				LOG "$1" "板卡[$1]芯片[$index]scp拷贝日志到本地${local_file}成功"
			fi
		else
			LOG "$1" "板卡[$1]芯片[$index]rsync拷贝日志到本地${local_file}成功"
		fi
		succFileNum=$((succFileNum + 1))
	done
	#如果全部为空直接退出
	if [ "${succFileNum}" -eq 0 ]; then  
		LOG "$1" "板卡[$1]系统拷贝芯片实时日志失败，成功文件数量为[$succFileNum]，失败数量为[${failFileNum}]"
		LOG "$1" "结束收集板卡【$1】的IAE实时日志"
		bRet=1
	fi
	LOG "$1" "移动芯片内实时日志到板卡[$1]系统，成功文件数量为[${succFileNum}],失败数量为[${failFileNum}]"
	#第四步:清空本地缓存
	if [ "$engine_name" == "昆仑2" ];then
		remote_file="/program/nfs_share/collect_log/*-realtime.tar.gz"
	else
		remote_file="/opt/wise_iae/ia12k/data*/log/*-realtime.tar.gz"
	fi
	ret=` SshCmd $1 $2 $3 $4 "rm -rf ${remote_file}" 2>/dev/null `
	ret_status=$?
	if [ "$ret_status" -ne 0 ]; then
		LOG "$1" "板卡[$1]删除临时实时日志文件失败，错误码[$ret_status]"
	fi
	LOG "$1" "结束收集板卡【$1】的IAE实时日志"
	return $bRet
}


# 获取iae芯片对应的日志
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码
getBwRealLog()
{
	LOG "$1" "开始收集板卡【$1】的BW实时日志"
	#第一步：删除IAM对应IP的目录
	ret=` rm -rf ${COLLECT_BW_REALTIME_LOG_PATH}/$1 2>/dev/null `
	ret=` mkdir -p ${COLLECT_BW_REALTIME_LOG_PATH}/$1 2>/dev/null `
	#第二步：压缩BW的日志到临时目录(待定，直接拷贝，不进行压缩)
	
	#第三步：拷贝实时日志到IAM的日志目录
	local remote_file_path="/var/log/proxylog"
	#获取拷贝命令
	ret=$(rsyncCmd $1 $2 $3 $4 "${remote_file_path}" "${COLLECT_BW_REALTIME_LOG_PATH}/$1")
	ret_status=$?
	if [ "$ret_status" -ne 0 ]; then
		LOG "$1" "板卡[$1]rsync拷贝日志到本地${CLLOECT_LOG_PATH}失败，错误码[$ret_status]"
		ret=$(scpCmd $1 $2 $3 $4 "${remote_file_path}" "${COLLECT_BW_REALTIME_LOG_PATH}/$1")
		ret_status=$?
		if [ "$ret_status" -ne 0 ]; then
			LOG "$1" "板卡[$1]scp拷贝日志到本地${COLLECT_BW_REALTIME_LOG_PATH}/$1失败，错误码[$ret_status]"
			return 1
		fi
	fi
	LOG "$1" "结束收集板卡【$1】的BW实时日志"
	return 0
	
}

# 获取iam的实时日志
getIamRealLog()
{
	LOG "IAM" "开始收集Iam的实时日志"
	#第一步：删除IAM对应IP的目录
	ret=` rm -rf ${COLLECT_IAM_REALTIME_LOG_PATH} 2>/dev/null `
	ret=` mkdir -p ${COLLECT_IAM_REALTIME_LOG_PATH} 2>/dev/null `
	#第三步：拷贝实时日志到IAM的日志收集目录
	local remote_file="${IAM_REALTIME_LOG_PATH}/iam*"
	#获取拷贝命令
	ret=` cp ${remote_file} ${COLLECT_IAM_REALTIME_LOG_PATH} 2>/dev/null `
	ret_status=$?
	if [ "$ret_status" -ne 0 ]; then
		LOG "IAM" "拷贝[${remote_file}]日志到本地${COLLECT_IAM_REALTIME_LOG_PATH}失败，错误码[$ret_status]"
		LOG "IAM" "结束收集Iam的实时日志"
		return 1
	fi
	LOG "IAM" "结束收集Iam的实时日志"
	return 0
}

###################################################测试代码###################################
: <<'COMMENT'
mkdir /opt/iacheck/collect_log/
touch /opt/iacheck/collect_log/run.log
board_ip="206.206.124.247"
ssh_port="22"
ssh_user="root"
ssh_password="admin_123"
chip_ids=("0" "1" "2")
chip_ids_string=$(IFS=','; echo "${chip_ids[*]}")
getIaeRealLog "$board_ip" "$ssh_port" "$ssh_user" "$ssh_password" "$chip_ids_string"
COMMENT
###################################################测试代码###################################

#################################################备份日志获取逻辑###################################
# 获取备份日志(不区分bw和iae，只看时间)
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码
##$5 备份日志的时间周期（1、2、3）
getBoardBackupLog()
{
	LOG "$1" "开始收集板卡【$1】的备份日志"
	#第一步：删除IAM对应IP的目录
	ret=` rm -rf ${COLLECT_IAE_BACKUP_LOG_PATH}/$1 2>/dev/null `
	ret=` mkdir -p ${COLLECT_IAE_BACKUP_LOG_PATH}/$1 2>/dev/null `
	#第二步：获取当前时间关键字
	local dateKeyWord=` date "+%Y%m%d" `
	#第三步：遍历日期组织key值进行下载
	#遍历芯片备份日志拷贝
	local succNum=0
	local failNum=0
	# 遍历指定的天数
	for ((dayIndex = 0; dayIndex <= $5; dayIndex++)); do
		# 计算当前日期关键字
		local currDateKeyWord=$((dateKeyWord - dayIndex))
		
		# 构建文件关键字
		local fileKeyWord="${currDateKeyWord}"
		
		# 尝试使用 rsync 拷贝文件
		rsyncCmd "$1" "$2" "$3" "$4" "${BORAD_BACKUP_LOG_PATH}/*${fileKeyWord}*.tar.gz" "${COLLECT_IAE_BACKUP_LOG_PATH}/$1"
		local ret_status=$?
		if [ "$ret_status" -ne 0 ]; then
			# rsync 失败，尝试使用 scp 拷贝文件
			LOG "$1" "板卡[$1] rsync 拷贝备份日志[*${fileKeyWord}*.tar.gz]到本地 ${COLLECT_IAE_BACKUP_LOG_PATH}/$1 失败，错误码 [$ret_status]"
			scpCmd "$1" "$2" "$3" "$4" "${BORAD_BACKUP_LOG_PATH}/*${fileKeyWord}*.tar.gz" "${COLLECT_IAE_BACKUP_LOG_PATH}/$1"
			local ret_status_scp=$?
			if [ "$ret_status_scp" -ne 0 ]; then
				LOG "$1" "板卡[$1] scp 拷贝备份日志[*${fileKeyWord}*.tar.gz]到本地 ${COLLECT_IAE_BACKUP_LOG_PATH}/$1失败，错误码 [$ret_status_scp]"
				failNum=$((failNum + 1))
				continue
			fi
		fi
		# 拷贝成功
		succNum=$((succNum + 1))
	done
	LOG "$1" "板卡[$1]拷贝备份芯片日志，成功天数为[$succNum]，失败天数为[$failNum]"
	LOG "$1" "结束收集板卡【$1】的IAE备份日志"
	if [ "${succNum}" -eq 0 ]; then  
		
		return 1
	fi
	return 0
}

# 获取备份日志IAM
##$1 备份日志的时间周期（1、2、3）
getIamBackupLog()
{
	LOG "IAM" "开始收集IAM的备份日志"
	#第一步：删除IAM对应IP的目录
	ret=` rm -rf ${COLLECT_IAM_BACKUP_LOG_PATH} 2>/dev/null `
	ret=` mkdir -p ${COLLECT_IAM_BACKUP_LOG_PATH} 2>/dev/null `
	#第二步：获取当前时间关键字
	local dateKeyWord=` date "+%Y%m%d" `
	#第三步：遍历日期组织key值进行下载
	#遍历芯片备份日志拷贝
	local succNum=0
	local failNum=0
	# 遍历指定的天数
	for ((dayIndex = 0; dayIndex <= $1; dayIndex++)); do
		# 计算当前日期关键字
		local currDateKeyWord=$((dateKeyWord - dayIndex))
		
		# 构建文件关键字
		local fileKeyWord="${currDateKeyWord}"
		
		# 尝试使用 rsync 拷贝文件
		ret=` cp ${IAM_BACKUP_LOG_PATH}/*${fileKeyWord}*.tar.gz ${COLLECT_IAM_BACKUP_LOG_PATH} 2>/dev/null `
		local ret_status=$?
		if [ "$ret_status" -ne 0 ]; then
			# rsync 失败，尝试使用 scp 拷贝文件
			LOG "IAM" "IAM拷贝备份日志[*${fileKeyWord}*.tar.gz]到本地 ${COLLECT_IAM_BACKUP_LOG_PATH}/$1 失败，错误码 [$ret_status]"				
			failNum=$((failNum + 1))
			continue
		fi
		# 拷贝成功
		succNum=$((succNum + 1))
	done
	LOG "IAM" "IAM拷贝备份芯片日志，成功天数为[$succNum]，失败天数为[$failNum]"
	LOG "IAM" "结束收集的IAM备份日志"
	return 0
}
###################################################备份日志测试代码###################################
: <<'COMMENT'
mkdir /opt/iacheck/collect_log/206.206.124.247
board_ip="206.206.124.247"
ssh_port="22"
ssh_user="root"
ssh_password="admin_123"
chip_ids=("0" "1" "2")
chip_ids_string=$(IFS=','; echo "${chip_ids[*]}")
getIaeBackupLog "$board_ip" "$ssh_port" "$ssh_user" "$ssh_password" "$chip_ids_string" 1
COMMENT
###################################################备份日志测试代码###################################


###################################################目录校验逻辑###################################
#根据获取的条件，简单计算目录大小
##$1 芯片个数（仅在备份日志的时间阈值不为0的时候为板卡IP个数）
##$2 收集的备份日志的时间阈值 1、2、3 （0表示不收集）
calculateNeedSize()
{
	CHIP_COUNT=$1
	LOG_THRESHOLD=$2
    local needSize=2048 #默认2G
 	if [ "$LOG_THRESHOLD" -ne 0 ]; then  # 收集备份日志
        needSize=$((IAE_ONEDAY_BACKUPLOG_NEED_SIZE * LOG_THRESHOLD * CHIP_COUNT))
    else  # 仅获取实时日志
        needSize=$((IAE_SIGNALE_TX1_REALLOG_NEED_SIZE * CHIP_COUNT))
    fi

	echo $needSize
}


# 实时日志板卡映射目录大小是否正常，目录都是挂在一个磁盘分区中，获取一次即可
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码
##$5 芯片个数
checkBoardRealLogDirSize()
{
	CHIP_COUNT=$5
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)  #######################测试的时候需要临时修改
	#GPU不进行校验板卡映射目录的校验
	if [ "$engine_name" != "GPU" ];then
		local dirPath=""
		if [ "$engine_name" == "昆仑2" ];then #判断的目录是
			dirPath="/program/nfs_share"
		else
			dirPath="/opt"
		fi
		local cmd="df -B1 ${dirPath} | awk 'NR==2 {print \$4}' | awk '{printf \"%d\", \$1 / 1024 / 1024}'"
		free_space_mb=` SshCmd $1 $2 $3 $4 "${cmd}" 2>/dev/null `
		
		# 检查SshCmd是否成功执行
        if [ $? -ne 0 ]; then
            LOG "无法从板卡[$1]获取可用空间信息"
            return 1
        fi
		# 计算所需的最小空间（以MB为单位）
		local needSize=$((IAE_SIGNALE_TX1_REALLOG_NEED_SIZE * CHIP_COUNT))
		
		# 检查可用空间是否满足要求
        if [ "$free_space_mb" -lt "$needSize" ]; then
            LOG "板卡[$1] 实时日志目录/opt剩余空间[${free_space_mb}]MB, 日志拷贝至少需要[${needSize}]MB, 不满足"
            return 1
        fi
	fi
	return 0
}


#校验IAM缓存的日志目录大小是否正常
##$1 芯片个数（仅在备份日志的时间阈值不为0的时候为板卡IP个数）
##$2 收集的备份日志的时间阈值 1、2、3 （0表示不收集）
checkIamLogDirSize()
{
	#获取IAM的临时目录剩余空间（以MB为单位）
	local free_space_mb=` df -B1 "${DATA_PATH}" | awk 'NR==2 {print $4}' | awk '{printf "%d", $1 *0.8 / 1024 / 1024}' ` #剩余0.2的空间保证IAM正常运行
	# 计算所需的最小空间（以MB为单位）
	local needSize=$(calculateNeedSize $1 $2)
	# 检查可用空间是否满足要求
	if [ "$free_space_mb" -lt "$needSize" ]; then
		LOG "IAM服务器的${DATA_PATH}剩余空间[${free_space_mb}]MB, 日志拷贝至少需要[${needSize}]MB, 请删除无用文件后再执行"
		return 1
	fi
	return 0
}

