#!/bin/bash

declare SOFT_VERSION="V2.1.0 2024/12/13"
declare SHELL_DIR=$(dirname $(readlink -f "$0"))	#shell脚本当前目录
declare DATA_PATH="/opt/iacheck" # 运行目录
declare CHECK_RESULT_EXCEL_FILE="${DATA_PATH}/log/check_report.xlsx" # 检查报告
declare CHECK_RESULT_JSON_FILE="${DATA_PATH}/log/check_result.json" # 检查结果输出文件
declare CHECK_INFO_FILE="${DATA_PATH}/log/command_info.log" # 收集信息输出文件
declare CHECK_LOG_FILE="${DATA_PATH}/log/run.log" # 日志记录
declare CHECK_DB_FILE="${DATA_PATH}/db/ia_check.db" # 脚本运行依赖的数据库
declare IAS_DB="/opt/wise/ia12k/db/ias.db" # iam sqlite数据库地址
# iam的pg数据库端口
iam_pg_port_tmp=5432
dev_type_info=` iamcfg list | grep VMDevType `
dev_type=$(echo ${dev_type_info} | awk -F " " '{print $3}')
case "${dev_type}" in
    *"VDS"*)
        iam_pg_port_tmp=5442
        ;;
    *)
		iam_pg_port_tmp=5432
        ;;
esac
declare IAM_PG_PORT=${iam_pg_port_tmp}
arch=$(uname -m)
declare SSHPASS=""
declare UTIL_FOR_REPORT=""
case "$arch" in
    x86_64|amd64)
        # 免交互ssh工具
        SSHPASS="${DATA_PATH}/bin/third_party/sshpass_x64"
        # 报告生成工具
        UTIL_FOR_REPORT="${DATA_PATH}/bin/third_party/format_report_x64"
        ;;
    arm*|aarch64)
        # 免交互ssh工具
        SSHPASS="/${DATA_PATH}/bin/third_party/sshpass_arm"
        # 报告生成工具
        UTIL_FOR_REPORT="${DATA_PATH}/bin/third_party/format_report_arm64"
        ;;
    *)
        echo "未知的系统架构: $arch"
        ;;
esac

declare PGSQL="/home/postgres/pgsql/bin/psql"
cmd=` ls /home/postgres/pgsql/bin/psql `
if [ 0 -ne $? ]
then
	PGSQL="/program/bin/psql"
fi

#python 脚本
declare OUTPUT_TO_CONSOLE_FILE="${DATA_PATH}/bin/python/output_to_console.py" # python控制台打印
declare MQ_DATA_DELAY_CHECK_FILE="${DATA_PATH}/bin/python/mq_data_delay_check.py" # python检查MQ数据积压
declare MQ_QEUE_CHECK_FILE="${DATA_PATH}/bin/python/mq_qeue_num_check.py" # python检查MQ队列数配置
declare MQ_DATA_UPDATE_FILE="${DATA_PATH}/bin/python/mq_data_update_check.py" # python检查MQ数据更新
declare MQ_DATA_COUNT_STATISTICS="${DATA_PATH}/bin/python/mq_data_count_check.py" # python统计接入数据量
declare MQ_BALANCE_DATA_FORMAT_CHECK_FILE="${DATA_PATH}/bin/python/mq_balance_data_format_check.py" # python检查MQ数据格式
declare MQ_DATA_LOSS_CHECK_FILE="${DATA_PATH}/bin/python/mq_loss_data_check.py" # python检查MQ数据丢失

declare ASSIGN_RESOURCE_CHECK_FILE="${DATA_PATH}/bin/python/assign_resource_check.py" # python检查芯片资源分配检查
declare AUTHORIZE_CHECK_FILE="${DATA_PATH}/bin/python/authorize_check.py" # python检查授权检查
declare ASSIGN_AND_AUTHORIZE_CHECK_FILE="${DATA_PATH}/bin/python/authorize_resource_assign_check.py" # python检查资源分配和授权是否匹配检查
declare AUTHORIZE_TIME_CHECK_FILE="${DATA_PATH}/bin/python/authorize_time_check.py" # python检查授权时间检查
declare TASK_RESOURCE_MATCH_CHECK_FILE="${DATA_PATH}/bin/python/task_resource_match_check.py" # python检查任务数和资源分配匹配检查




declare MD_BALANCE_IP="0.0.0.0" # MD地址
declare VERSION_LIMIT=3351 # 最低版本限制
declare CHECK_JSON_INFO # 缓存检查结果（json格式）
declare -A BOARD_CHIP_IP #缓存KUNLUN2的芯片IP
declare -A IAM_PG_PASSWD #IAM PG数据库密码

#json报告相关的全局描述
declare STATUS_NORMAL=1     #正常
declare STATUS_EXCEPTION=2  #异常
declare STATUS_ERROR=3		#错误

#环境检查条目声明
declare CHECK_ENV_ITEM_CORE="core开关和core文件巡检"
declare CHECK_ENV_ITEM_ZOMBIE="僵尸进程巡检"
declare CHECK_ENV_ITEM_CPU="CPU使用率巡检"
declare CHECK_ENV_ITEM_DISK="磁盘容量巡检"
declare CHECK_ENV_ITEM_MEMORY="内存使用巡检"
declare CHECK_ENV_ITEM_NETPORT="端口映射巡检"
declare CHECK_ENV_ITEM_SSD="SSD寿命巡检"

declare CHECK_ENV_ITEM_TEMPERATURE="芯片温度巡检"
declare CHECK_ENV_ITEM_PG="PG数据库压力巡检"
declare CHECK_ENV_ITEM_PORTBLOCK="端口阻塞巡检"
declare CHECK_ENV_ITEM_NG="芯片NG巡检"
declare CHECK_ENV_ITEM_DRIVER="驱动版本巡检"
declare CHECK_ENV_ITEM_CHIPONLINE="芯片在线状态巡检"
declare CHECK_ENV_ITEM_PORTLISTEN="端口监听巡检"

# 通用业务检测项
declare CHECK_VM_DB_PASSWORD="VM数据库密码巡检"
declare CHECK_CDS_IPSAN_CONFIG="CDS/IPSAN组网及IP巡检"
declare CHECK_DBE_IP_STATUS="DBE配置及IP巡检"
declare CHECK_IAE_VERSION="IAE版本巡检"
declare CHECK_IAM_VERSION="IAM版本巡检"
declare CHECK_MD_IP_STATUS="MD配置及IP巡检"
declare CHECK_SERVER_STATUS="服务状态巡检"
declare CHECK_IAE_SERVER_STATUS="IAE服务状态巡检"
declare CHECK_BW_SERVER_STATUS="BW服务状态巡检"
declare CHECK_SOFT_VERSION_LIMIT="最低软件版本限制"
declare CHECK_CHIP_PASSWORD_FREE="芯片免密登录检查"
declare CHECK_ALGO_ATTR_MODE="算法分析模式校验"
declare CHECK_CHIP_RUN_STATUS="芯片状态检查"
declare CHECK_IAM_FIREWALLD_STATUS="iam防火墙状态检查"
declare CHECK_FACE_BALANCE_STATUS="人脸图片流负载均衡状态检查"
declare CHECK_DMESG_LOG="DMESG日志巡检"
declare CHECK_DISK_IO="磁盘IO检查"
declare CHECK_LOG_LEVEL="debug日志开启检查"
declare CHECK_REPLACE_LIB="替换库检查"
declare CHECK_SYNC_TIME="时间同步检查"
declare CHECK_IAM_PG_PASSWD_CONFIG="IAM pg数据库密码配置检查"
declare CHECK_MQ_DATA_DELAY="MQ数据积压检查"
declare CHECK_MQ_QEUE="MQ队列数配置检查"
declare CHECK_MQ_DATA_UPDATA="MQ数据更新检查"
declare CHECK_MQ_DATA_COUNT_STATISTICS="MQ接入数据量统计"
declare CHECK_MQ_BALANCE_DATA_FORMAT="MQ的balance数据格式检查"
declare CHECK_MQ_DATA_LOSS="MQ的数据丢失检查"
declare CHECK_BA_ALARM_THRESHOLD="行为分析告警阈值检查"

# 人脸业务检测项
declare CHECK_STATICFACE_CONFIG="静态库平台巡检"
declare CHECK_DISPOSE_CONFIG="布控组网配置巡检"

# 结构化业务检测项
declare CHECK_VIDEO_STREAM_FORWARD_MD_CONFIG="视频流转发MD配置巡检"
declare CHECK_FORWARD_STATUS="转发配置巡检"

# 授权和资源分配相关检查项
declare CHECK_ASSIGN_RESOURCE="芯片资源分配检查"
declare CHECK_AUTHORIZE="授权检查"
declare CHECK_ASSIGN_AND_AUTHORIZE="资源分配和授权匹配检查"
declare CHECK_AUTHORIZE_TIME="授权时间检查"
declare CHECK_TASK_RESOURCE_MATCH="任务数和资源分配匹配检查"

