#!/bin/bash

### iam业务数据库访问操作
### $1 数据库类型，sqlite/pg
### $2 sql查询语句
SelectDB()
{
	
	local result=""
	case "$1" in
		"pg")
			result=` ${PGSQL} "host=127.0.0.1 port=${IAM_PG_PORT} user=postgres password=${IAM_PG_PASSWD} dbname=ias" << EOF
$2;
EOF`
			;;
		"sqlite")
			result=` sqlite3 /opt/wise/ia12k/db/ias.db << EOF
$2;
EOF`
			;;
		*)
			echo "the db($1) is not supported"
			;;
	esac
    echo "${result}"
	LOG 1 "${result}"
}

### 远程命令封装
### $1 远程ip
### $2 远程端口
### $3 远程用户
### $4 远程密码
### $5 远程命令
SshCmd ()
{
	cmd=$(timeout 40s ${SSHPASS} -p $4 ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p $2 $3@$1 "$5" 2>/dev/null)
	timeout_exit_status=$?
	# 根据退出状态码输出相应的日志
	if [ "$timeout_exit_status" -ne 0 ]; then
		RecordCommndInfo "$1" "sshpass执行命令【$5】返回码【$timeout_exit_status】" "${cmd}"
		cmd=""
	fi
	echo "${cmd}"
    return $timeout_exit_status
}

#获取板卡类型
# $1 板卡IP
Get_Engine_Name()
{
        #获取板卡类型
        engine_type=$(sqlite3 /opt/wise/ia12k/db/ias.db "select engine_version from ia_engine where address = '$1' group by engine_version")

        case $engine_type in
        "1")
                echo "昆仑2"
        ;;
        "2")
                echo "昆仑3"
        ;;
        "3")
                echo "GPU"
        ;;
        "4")
                echo "寒武纪"
        ;;
    "7")
    echo "AX"
    ;;
        esac
}


#获取板卡芯片数量
# $1 板卡IP
Get_Chip_Num()
{
        chipNum=$(sqlite3 /opt/wise/ia12k/db/ias.db "select count(*) from ia_engine where address = '$1';")
        chipNum=$((chipNum-1))
        echo "$chipNum"
}

### 目录/文件初始化
FileInit()
{
    # 创建需要的所有文件
    [ ! -d "${DATA_PATH}" ] && mkdir "${DATA_PATH}"
    [ ! -d "${DATA_PATH}/log" ] && mkdir "${DATA_PATH}/log"
    [ ! -d "${DATA_PATH}/db" ] && mkdir "${DATA_PATH}/db"
    [ ! -d "${DATA_PATH}/bin" ] && mkdir "${DATA_PATH}/bin"
    [ ! -f "${CHECK_RESULT_JSON_FILE}" ] && touch "${CHECK_RESULT_JSON_FILE}"
    [ ! -f "${CHECK_RESULT_EXCEL_FILE}" ] && touch "${CHECK_RESULT_EXCEL_FILE}"
    [ ! -f "${CHECK_INFO_FILE}" ] && touch "${CHECK_INFO_FILE}"
    [ ! -f "${CHECK_LOG_FILE}" ] && touch "${CHECK_LOG_FILE}"
    [ ! -f "${CHECK_DB_FILE}" ] && touch "${CHECK_DB_FILE}"
	# 清空所有文件
	> "${CHECK_RESULT_JSON_FILE}"
	> "${CHECK_RESULT_EXCEL_FILE}"
	> "${CHECK_INFO_FILE}"
	> "${CHECK_LOG_FILE}"
}

#########获取芯片ip映射#######
##$1 板卡列表信息列表
Get_Chip_Ip()
{
	#遍历板卡列表
	for engien_ssh_info_line in "$@"
	do
		local ssh_ip=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $1}')
		local ssh_port=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $2}')
		local ssh_user_name=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $3}')
		local ssh_passwd=$(echo ${engien_ssh_info_line} | awk -F "|" '{print $4}')
		local engine_name=$(Get_Engine_Name "$ssh_ip")
		LOG 1 "engine_name is ${engine_name}"
		if [ "${engine_name}" == "昆仑2" ];then
			# 获取昆仑2板卡芯片数
			local prototype_name=` SshCmd $ssh_ip $ssh_port $ssh_user_name $ssh_passwd " cat /tmp/.manuinfo | grep PROTOTYPE_NAME" 2>/dev/null `
			if [ -z "${prototype_name}" ];then
				LOG 0 "昆仑2板卡IP【${ssh_ip}】获取板卡类型${prototype_name}为空，昆仑2板卡【${ssh_ip}】无法正常巡检"
				return 1
			fi
			RecordCommndInfo "$ssh_ip" "cat /tmp/.manuinfo | grep PROTOTYPE_NAME" "${prototype_name}"
			LOG 1 "昆仑2 【$ssh_ip】prototype_name的值是${prototype_name}"
			if [ "$prototype_name" != "" ];then
				local prototypeRet=$(echo $prototype_name | grep "FB-IA12500-B")
				if [ "$prototypeRet" != "" ];then
					prototype_name="tx1_5"
				elif [ -n "$(echo $prototype_name | grep "FB-IA12500-S")" ];then
					prototype_name="tx1_2"
				else
					prototype_name="tx1_10"
				fi
				LOG 1 "昆仑2板卡IP【${ssh_ip}】获取板卡类型为${prototype_name}"
			else
				LOG 0 "昆仑2板卡IP【${ssh_ip}】获取板卡类型${prototype_name}为空，昆仑2板卡【${ssh_ip}】无法正常巡检"
			fi
			
			if [ "${prototype_name}" == "tx1_5" ]
			then    # 5芯片昆仑2，第一颗芯片ip：192.168.255.11
				BOARD_CHIP_IP[$ssh_ip-0]="192.168.255.11"
				BOARD_CHIP_IP[$ssh_ip-1]="192.168.255.13"
				BOARD_CHIP_IP[$ssh_ip-2]="192.168.255.15"
				BOARD_CHIP_IP[$ssh_ip-3]="192.168.255.17"
				BOARD_CHIP_IP[$ssh_ip-4]="192.168.255.19"
			elif [ "${prototype_name}" == "tx1_2" ]
			then	# 2芯片昆仑2，第一颗芯片ip：192.168.255.17
				BOARD_CHIP_IP[$ssh_ip-0]="192.168.255.17"
				BOARD_CHIP_IP[$ssh_ip-1]="192.168.255.19"
			else
				BOARD_CHIP_IP[$ssh_ip-0]="192.168.255.1"
				BOARD_CHIP_IP[$ssh_ip-1]="192.168.255.3"
				BOARD_CHIP_IP[$ssh_ip-2]="192.168.255.5"
				BOARD_CHIP_IP[$ssh_ip-3]="192.168.255.7"
				BOARD_CHIP_IP[$ssh_ip-4]="192.168.255.9"
				BOARD_CHIP_IP[$ssh_ip-5]="192.168.255.11"
				BOARD_CHIP_IP[$ssh_ip-6]="192.168.255.13"
				BOARD_CHIP_IP[$ssh_ip-7]="192.168.255.15"
				BOARD_CHIP_IP[$ssh_ip-8]="192.168.255.17"
				BOARD_CHIP_IP[$ssh_ip-9]="192.168.255.19"
			fi
		fi	
	done
}

### 格式化python脚本巡检结果
### $1 巡检状态，0：INFO，1：ERROR，3：WARNING
### $2 描述信息
FormatPyhtonCheckReuslt()
{
    check_result=$(echo -e "$2" | tr '\n' ',')
    check_result="${check_result%,}"
    if [ $1 -eq 0 ]
    then
        ResultTxtOutput "INFO" "${check_result}"
    elif [ $1 -eq 1 ]
    then
        ResultTxtOutput "WARNING" "${check_result}"
    elif [ $1 -eq 2 ]
    then
        ResultTxtOutput "ERROR" "${check_result}"
    else
        ResultTxtOutput "UNKNOWN: $1" "检查失败: ${check_result}"   
    fi
}
