#!/bin/bash

#缓存检查项的结果，并输出下面的json报告格式
#[
#    {
#		"IP": "IAM",
#		"BoardCardType": "IAM", 
#    	"CheckItem": [
#        	{
#            	"ItemName": "Item1",
#            	"Status": "some_status",
#          		"Message": {
#   			 "ERROR": ["message2_part1","message2_part5"],
#    			 "INFO": ["message2_part2"],
#   			 "WARNING": ["message2_part3"]}
#        	},
#        	{
#           	"ItemName": "Item2",
#            	"Status": "another_status",
#            	"Message": {
#   			 "ERROR": ["message2_part1"],
#    			 "INFO": ["message2_part2"],
#   			 "WARNING": ["message2_part3"]}
#        	}
#    	]
#	},
#   {
#		"IP": "206.206.124.89",
#		"BoardCardType": "GPU",   #昆仑2、昆仑3、GPU、寒武纪
#    	"CheckItem": [
#        	{
#            	"ItemName": "Item1",
#            	"Status": "some_status",
#            	"Message": {
#   			 "ERROR": ["message2_part1"],
#    			 "INFO": ["message2_part2"],
#   			 "WARNING": ["message2_part3"]}
#        	},
#        	{
#           	"ItemName": "Item2",
#            	"Status": "another_status",
#            	"Message": {
#   			 "ERROR": ["message2_part1"],
#    			 "INFO": ["message2_part2"],
#   			 "WARNING": ["message2_part3"]}
#        	}
#    	]
#	}
#]

# 声明一个关联数组来缓存 error的message 信息
declare -a error_message_cache
# 声明一个关联数组来缓存 info的message 信息
declare -a info_message_cache
# 声明一个关联数组来缓存 warning的message 信息
declare -a warning_message_cache
#声明检查项，动态修改
declare check_item_name=""
#声明检查项的状态
declare current_item_status=""

# 函数：JSON开头字段，输出下面的文本到文件中
#
#[
#
StartJsonOutput() 
{
    echo -n "[" >> "${CHECK_RESULT_JSON_FILE}"
}


# 函数：JSON文件的IP段，输出下面的文本到文件中
#
# {
#	"IP": "IAM",
#   "CheckItem": [
#
# $1 IP信息
StartIpJsonOutput()
{
    local ip="$1"
	if [ "${ip}" == "IAM" ];then  #IAM的处理逻辑
		echo -n "{\"IP\": \"$ip\", \"BoardCardType\": \"IAM\", \"CheckItem\": [" >> "${CHECK_RESULT_JSON_FILE}"
    elif [ "${ip}" == "MQ" ];then  #IAM的处理逻辑
		echo -n "{\"IP\": \"${MD_BALANCE_IP}\", \"BoardCardType\": \"MQ\", \"CheckItem\": [" >> "${CHECK_RESULT_JSON_FILE}"
	else #增加板卡类型
		local engine_name=$(Get_Engine_Name $1)
		echo -n "{\"IP\": \"$ip\", \"BoardCardType\": \"$engine_name\", \"CheckItem\": [" >> "${CHECK_RESULT_JSON_FILE}"
	fi
}

# 函数：JSON的单个ItemName数据项接口，接收ItemName信息，输出下面的信息到文件中
#
# {
#    "ItemName": "Item1",
#
# $1 ItemName的信息
StartCheckItemJsonOutput() 
{
    local item_name="$1"
	check_item_name="$1"
    echo -n "{" >> "${CHECK_RESULT_JSON_FILE}"
    echo -n "\"ItemName\": \"$item_name\"," >> "${CHECK_RESULT_JSON_FILE}"
}

# 函数：设置JSON数据项状态接口，接收 status 信息,动态更新，如果传入的状态值高，则更新对应的状态
# $1 status的描述 1：正常 2：异常 3：错误
SetStatus() 
{
    local new_value="$1"
	 # 检查current_item_status已经存在
    if [ -z "${current_item_status}" ]; then
        # 如果键不存在，则直接赋值
        current_item_status="${new_value}"
    else
		# 如果新值比现有值大，则替换
        if (( new_value > "${current_item_status}" )); then
            current_item_status="${new_value}"
        fi
    fi
}

# 函数：将数值转换为对应的字符串
# $1 status的描述 1：正常 2：异常 3：错误
# 函数：将数值转换为对应的字符串
JsonStatusToString() {
    local status="$1"
    case $status in
        $STATUS_NORMAL) echo "OK" ;;
        $STATUS_EXCEPTION) echo "WARNING" ;;
        $STATUS_ERROR) echo "ERROR" ;;
        *) echo "UNKNOWN" ;;
    esac
}

# 函数：追加 message 信息到缓存中
# $1 status的描述
# $2 message的描述
AppendMessage() 
{
	local status="$1"
    case $status in
        $STATUS_NORMAL)
			info_message_cache+=("\"$2\"")
		;;
        $STATUS_EXCEPTION) 
			warning_message_cache+=("\"$2\"")	
		;;
        $STATUS_ERROR)
			error_message_cache+=("\"$2\"")
		;;
        *)
		LOG 0 "AppendMessage获取的日志等级${status}不支持"
		;;
    esac
}

# 函数：追加JSON的message和status
# $1 status的描述
# $2 message的描述
AppendMessageAndStatus() 
{
	SetStatus "$1"
	local message=$(echo "$2"| sed "s/\"/'/g") #去除""符号，使用单引号替换
	AppendMessage "$1" "${message}"
}

# 将数组元素用逗号分隔并输出
JoninCommoncomma() 
{
    local IFS=','  # 设置内部字段分隔符为逗号
    echo "$*"  #将传入的所有参数输出
}

# 函数：结束 JSON 数据项的 message 追加接口
# 将message的描述打印，并清空历史信息。，输出下面的信息到文件中
#
#"Status": "some_status",
#"Message": ["message1_part1", "message1_part2", "message1_part3"]
#
EndJsonMessageAndStatus() 
{
	local item_name="${check_item_name}"
	#输出状态描述到json中
	local status_value="${current_item_status}"
    local status_string=$(JsonStatusToString "${status_value}")
	echo -n "\"Status\": \"${status_string}\"," >> "${CHECK_RESULT_JSON_FILE}"
	#输出message描述到json中
	echo -n "\"Message\":{"  >> "${CHECK_RESULT_JSON_FILE}"
	if [ ${#error_message_cache[@]} -gt 0 ]; then #不为空
		echo -n "\"ERROR\": [$(JoninCommoncomma "${error_message_cache[@]}")]," >> "${CHECK_RESULT_JSON_FILE}"
	fi
	if [ ${#info_message_cache[@]} -gt 0 ]; then #不为空
		echo -n "\"INFO\": [$(JoninCommoncomma "${info_message_cache[@]}")]," >> "${CHECK_RESULT_JSON_FILE}"
	fi
	if [ ${#warning_message_cache[@]} -gt 0 ]; then #不为空
		echo -n "\"WARNING\": [$(JoninCommoncomma "${warning_message_cache[@]}")]," >> "${CHECK_RESULT_JSON_FILE}"
	fi
	# 移除最后一个逗号
    sed -i '$ s/,$//' "${CHECK_RESULT_JSON_FILE}"
	echo -n "}"  >> "${CHECK_RESULT_JSON_FILE}"
	#清空相关数组
    unset error_message_cache
	unset info_message_cache
	unset warning_message_cache
	current_item_status=""

}

# 函数：结束 JSON 数据项单项接口，输出下面的内容到文本中
#
# },
#
EndCheckItemJsonOutput()
{
	EndJsonMessageAndStatus
	check_item_name=""
    echo -n "}," >> "${CHECK_RESULT_JSON_FILE}"
}


# 函数：结束IP段的数据接口，将checkItem的上一个，删除然后输出下面的信息到文本中
#
#    	]},
#
EndIpJsonOutput() 
{
	# 移除最后一个逗号
    sed -i '$ s/,$//' "${CHECK_RESULT_JSON_FILE}"
    echo -n "]}," >> "${CHECK_RESULT_JSON_FILE}"
}

# 函数：结束JSON数据项全局接口，将上一个IP段的数据的“，”删除，删除然后输出下面的信息到文本中
#
#    	]
#
EndJsonOutput() 
{
	# 移除最后一个逗号
    sed -i '$ s/,$//' "${CHECK_RESULT_JSON_FILE}"
    echo -n "]" >> "${CHECK_RESULT_JSON_FILE}"
}