#!/bin/bash
###############################################################
#GPU在线状态检查
##1、检测芯片温度,阈值温度90
##2、检测芯片上线个数和iam记录gpu数是否一致
###############################################################

### 获取GPU芯片信息
GetGpuChipInfo()
{
    local gpu_info=""
    # 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "GetGpuChipInfo入参1检查为空: Expected length: $1"
	elif [ -z "$2" ]; then
		LOG 1 "GetGpuChipInfo入参2检查为空: Expected length: $2"
	elif [ -z "$3" ]; then
		LOG 1 "GetGpuChipInfo入参3检查为空: Expected length: $3"
	elif [ -z "$4" ]; then
		LOG 1 "GetGpuChipInfo入参4检查为空: Expected length: $4"
    else
        #获取GPU的驱动的下标（0开始）name 温度 用，分割
        gpu_info=` SshCmd $1 $2 $3 $4 " nvidia-smi --query-gpu=index,name,temperature.gpu --format=csv,noheader " 2>/dev/null ` 
        RecordCommndInfo "$1" "nvidia-smi --query-gpu=index,name,temperature.gpu --format=csv,noheader" "${gpu_info}"
	fi
    echo "${gpu_info}"    
}

### GPU ng状态检查
### $1 gpu 物理在线芯片信息
### $2 板卡IP
CheckGpuNGState()
{
     # 判断变量是否为空
	if [ -z "$1" ]; then
		ResultTxtOutput "WARNING" "GPU板卡IP[$2]:--query-gpu=index,name,temperature.gpu --format=csv,noheader返回为空，检查失败"
		return 1
	fi
    
	##获取iam记录的有分析能力的的GPU个数
	local engine_num_have_abality=$(sqlite3 /opt/wise/ia12k/db/ias.db "SELECT COUNT(*) FROM ia_engine WHERE address='$1' AND business_type!='1' AND port!='10245';")
	LOG 1 "CheckGpuState获取IAM记录GPU板卡IP[$1]的GPU芯片个数：${engine_num_have_abality}"	
	local gpu_total_num=` echo "$1"|wc -l  2>/dev/null ` #驱动获取到的芯片数量
	LOG 1 "CheckGpuState：GPU板卡IP[$2]的驱动实际芯片个数：$gpu_total_num"
	
	#判断是否存在数量不一致情况
	if [ "$gpu_total_num" -ne "${engine_num_have_abality}" ]; then
		ResultTxtOutput "ERROR" "GPU板卡IP[$2]：实际芯片个数[${gpu_total_num}]与iam上芯片在线个数[${engine_num_have_abality}]不一致"
	else
		ResultTxtOutput "INFO" "GPU板卡IP[$2]：实际芯片个数[${gpu_total_num}]与iam上芯片在线个数[${engine_num_have_abality}]一致"
	fi
}

### GPU温度检查
### $1 gpu芯片信息
### $2 板卡ip
CheckGpuTempetature()
{
    local temperature_threshold=93
	#获取当前温度
	local temperature_seq=$(echo "$1"|awk 'BEGIN {FS=","}{print $3}')
	LOG 1 "CheckGpuState：获取GPU板卡IP[$2]的芯片温度序列：\"$temperature_seq\" "
	# 将提取的温度序列转换为数组
	IFS=$'\n' read -r -d '' -a temperatures <<< "$(echo "$temperature_seq" | awk '{gsub(/^[ \t]+|[ \t]+$/, ""); print}')"
	# 打印数组中的每个温度
	# 遍历温度序列，检查每个温度值是否大于额定数值
	for ((i=0; i<${#temperatures[@]}; i++)); do
		if [ "${temperatures[i]}" -gt "$temperature_threshold" ]; then
			# 输出超过额定数值的温度及其完整的 IP 地址
			ResultTxtOutput "ERROR" "GPU板卡IP[$2]芯片序号[${i}]芯片温度${temperatures[i]}，超过阈值${temperature_threshold}，温度异常"
		else
			ResultTxtOutput "INFO" "GPU板卡IP[$2]芯片序号[${i}]芯片温度${temperatures[i]}，未超过阈值${temperature_threshold}"
		fi
		
	done
}

### GPU环境检查封装
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
CheckGpuEnvironment()
{
    local gpu_chip_info=` GetGpuChipInfo $1 $2 $3 $4 `
    # GPU 物理卡在线状态检查
    StartCheckItemJsonOutput "${CHECK_ENV_ITEM_NG}" #开始检查项JSON输出
    CheckGpuNGState "${gpu_chip_info}" $1
    IAEUpdateProgress
	EndCheckItemJsonOutput #结束检查项JSON输出
    
    # GPU温度检查
    StartCheckItemJsonOutput "${CHECK_ENV_ITEM_TEMPERATURE}" #开始检查项JSON输出
    CheckGpuTempetature "${gpu_chip_info}" $1
    IAEUpdateProgress
    EndCheckItemJsonOutput #结束检查项JSON输出
}