#!/bin/bash
###############################################################
#昆仑3的板卡驱动检查，校验驱动版本是否最新
###############################################################

### hisi驱动版本检查
##$1 hisi驱动信息
##$2 hisi最低驱动版本限制
##$3 板卡ip
CheckHisiDriver()
{
	local driver_version=$(echo $1 | awk '/Hisi3559 DRV Version: K-R020-/ {print $NF}' | awk 'BEGIN {FS="-"} {print $3}' | sed 's/[[:space:]]*$//') 
	# 比较版本号
	if [[ "$driver_version" == "$2" || "$driver_version" > "$2" ]]; then
		ResultTxtOutput "INFO" "昆仑3板卡IP[$3]:当前驱动版本${driver_version}, 最低驱动版本$2"
	else
		ResultTxtOutput "WARNING" "昆仑3板卡IP[$3]:当前驱动版本${driver_version}，最低驱动版本$2,需升级驱动版本"
	fi
}

GetHisiDriverVerLimit()
{
    local hisi_driver_limit=""
    # 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "GetHisiDriverVerLimit入参1检查为空: Expected length: $1"
        hisi_driver_limit=""
	elif [ -z "$2" ]; then
		LOG 1 "GetHisiDriverVerLimit入参2检查为空: Expected length: $2"	
        hisi_driver_limit=""
	elif [ -z "$3" ]; then
		LOG 1 "GetHisiDriverVerLimit入参3检查为空: Expected length: $3"
        hisi_driver_limit=""
	elif [ -z "$4" ]; then
		LOG 1 "GetHisiDriverVerLimit入参4检查为空: Expected length: $4"
        hisi_driver_limit=""
	else
        local sys_info=$( SshCmd "$1" "$2" "$3" "$4" "uname -r" 2>/dev/null ) #获取系统信息
        RecordCommndInfo "$1" "uname -r" "${sys_info}"
        if [ -z "$sys_info" ]; then
            ResultTxtOutput "WARNING" "HISI板卡IP[$1]:uname -r返回为空，版本检查失败"
        else
            if [ "$sys_info" != "4.19" ]; then
                #不是KL3-A版本
                local chip_num=$( SshCmd "$1" "$2" "$3" "$4" "lspci | grep 3559 | wc -l" 2>/dev/null ) #获取芯片数量
                RecordCommndInfo "$1" "lspci | grep 3559 | wc -l" "${chip_num}"
                if [ -z "$chip_num" ]; then
                    ResultTxtOutput "WARNING" "HISI板卡IP[$1]:【lspci | grep 3559 | wc -l】返回为空，驱动版本检查失败"
                    hisi_driver_limit=""
                else
                    if [ "$chip_num" -gt 3 ]; then
                        hisi_driver_limit="D015"	 #普通的版本
                    else
                        hisi_driver_limit="D014"  #超融合定制
                    fi
                fi
            else
                hisi_driver_limit="D006"
            fi
        fi
	fi
    echo "${hisi_driver_limit}"
}

GetHisiDirverInfo()
{
    local driver_info=""
    # 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "GetHisiDirverInfo入参1检查为空: Expected length: $1"
        driver_info=""
	elif [ -z "$2" ]; then
		LOG 1 "GetHisiDirverInfo入参2检查为空: Expected length: $2"	
        driver_info=""
	elif [ -z "$3" ]; then
		LOG 1 "GetHisiDirverInfo入参3检查为空: Expected length: $3"
        driver_info=""
	elif [ -z "$4" ]; then
		LOG 1 "GetHisiDirverInfo入参4检查为空: Expected length: $4"
        driver_info=""
	else
        local driver_info=$( SshCmd "$1" "$2" "$3" "$4" "update.sh -v" 2>/dev/null ) 
        RecordCommndInfo "$1" "update.sh -v" "${driver_info}"
        if [ -z "$driver_info" ]; then
            ResultTxtOutput "WARNING" "HISI板卡IP[$1]:update.sh -v返回为空，驱动版本检查失败"
            return 1
        fi
    fi
    echo "${driver_info}"
}

### HISI环境检查
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码 
CheckHisiEnvironment()
{
    local hisi_dirver_ver_limit=` GetHisiDriverVerLimit $1 $2 $3 $4 `
    local hisi_dirver_info=` GetHisiDirverInfo $1 $2 $3 $4 `

    StartCheckItemJsonOutput "${CHECK_ENV_ITEM_DRIVER}" #开始检查项JSON输出
    CheckHisiDriver "${hisi_dirver_info}" "${hisi_dirver_ver_limit}" "$1"
    IAEUpdateProgress
    EndCheckItemJsonOutput #结束检查项JSON输出
}