#!/bin/bash
###############################################################
##IAM检查项，校验PG数据库压力和端口是否存在阻塞
#PG数据库显示当前的连接数
###############################################################

#校验PG数据库压力
CheckIamPgPressure()
{
	# 配置阈值
	local pg_presure=1000  # 压力阈值（活动连接数）

	# 查询活动连接数
	local active_connections=` SelectDB "pg" "SELECT COUNT(*) FROM pg_stat_activity WHERE state = 'active'" | awk 'NR==3 {print $1}' `
	RecordCommndInfo "IAM" "SELECT COUNT(*) FROM pg_stat_activity WHERE state = 'active' | awk 'NR==3 {print $1}' " "${active_connections}"
	# 输出结果
	LOG 1 "CheckIamPgPressure 获取PG数据库活动连接数: ${active_connections}"

	# 检查压力是否超过阈值
	if [ ${active_connections} -gt ${pg_presure} ]; then
		ResultTxtOutput "ERROR" "IAM的PG数据库连接数 ${active_connections} 超出阈值 ${pg_presure}"
	else
		ResultTxtOutput "INFO" "IAM的PG数据库连接数 ${active_connections}正常"
	fi
}


# 检查端口接收缓冲区
CheckPortBuffers()
{
	#设置阈值
	local rcv_que_threshold=129
	# 遍历每个端口
	local ports=(10240 5832 5347 30320)
	for port in "${ports[@]}"; do
		# 使用 netstat 检查指定端口的连接
		local netstat_output=$(netstat -anp | grep ":${port}")
		# 检查输出是否为空
		if [ -z "$netstat_output" ]; then
		  ResultTxtOutput "WARNING" "IAM未获取到${port}信息"
		  continue
		fi
        local port_license_ok="True"
		# 解析 netstat 输出，检查接收缓冲区
		while read -r line; do
			# 提取 PID 和程序名
			local program=$(echo "$line" | awk '{print $7}' | cut -d'/' -f2)
			if [ "${program}" != "iam" ] && [ "${program}"  != "IAIM" ] ;then
				continue
			fi
			local pid=$(echo "$line" | awk '{print $7}' | cut -d'/' -f1)
			#获取本地IP和远方ip
			local local_ip=$(echo "$line" | awk '{print $4}')
			local foreign_ip=$(echo "$line" | awk '{print $5}')
			# 获取Recv-Q的队列数
			local rcv_buffer=$(echo "$line" | awk '{print $2}'| bc -l)
			# 检查接收缓冲区是否超过阈值
			if (( $(echo "${rcv_buffer} > ${rcv_que_threshold}" | bc -l) )); then
                port_license_ok="False"
				LOG 1 "IAM服务器(PID:${pid}, Program:${program})接收队列超出阈值 ${rcv_que_threshold}：本地IP[${local_ip}],远方IP[${foreign_ip}],接收队列数[${rcv_buffer}]"
			else
                port_license_ok="True"
				LOG 1 "IAM服务器(PID:${pid}, Program:${program})接收队列正常。本地IP[${local_ip}],远方IP[${foreign_ip}],接收队列数[${rcv_buffer}]不超过阈值【${rcv_que_threshold}】"
			fi
		done <<< "$netstat_output"
		
        if [ "True"=="${port_license_ok}" ]
        then
            ResultTxtOutput "INFO" "IAM服务器端口：${port}，监听正常"
        else
            ResultTxtOutput "ERROR" "IAM服务器端口：${port}，监听异常"
        fi
	done
}


