#!/bin/bash
###############################################################
##mlu：mlu状态检查、驱动版本检测
###############################################################

#########获取板卡驱动版本#######
##$1 寒武纪驱动信息
##$2 板卡IP
CheckMluDriver()
{	
	if [ -z "${1}" ]; then
		ResultTxtOutput "WARNING" "寒武纪板卡IP[$1]:c20e_info.sh -v返回为空，版本检查失败"
		return 1
	fi
	
	local driver_version=$(echo "$1" | awk '/C20e Info Tools Version:/ {print $NF}') #版本号
	if [ -z "${driver_version}" ]; then
		LOG 1 "CheckMluDriver获取驱动版本号为空"
		return 1
	fi
	
	# 定义要比较的版本号
	local current_version="D014"

	# 比较版本号
	if [[ "${driver_version}" == "${current_version}" || "${driver_version}" > "${current_version}" ]]; then
		ResultTxtOutput "INFO" "寒武纪板卡IP[$2]驱动版本号为[${driver_version}]，当前最低版本号为${current_version}，驱动满足"
	else
		ResultTxtOutput "WARNING" "寒武纪板卡IP[$3]驱动版本号为[${driver_version}]，当前最低版本号为${current_version}，请升级驱动"
	fi
}

# 解析信息，获取IP列，并检查IP的第4位是否按指定序列累加，判断缺失的IP板卡
##$1 芯片数量 
##$2 c20e_info.sh的输出
##$3 板卡IP
CheckMluIpSequence()
{
	# 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "CheckMluIpSequence入参1检查为空: Expected length: $1"
		return 1
	fi
	
	if [ -z "$2" ]; then
		LOG 1 "CheckMluIpSequence入参2检查为空：Input data: $2"
		return 1
	fi
    if [ -z "$3" ]; then
		LOG 1 "CheckMluIpSequence入参3检查为空：Input data: $3"
		return 1
	fi
		
	#解析c20e_info.sh信息获取芯片数量
	local tx1_total_num=` echo "$2"| grep -E " [0-9]+ 220 Reign|[0-9]+ GPU Dropped" | sed -E 's/.* ([0-9]+) 220.* ([0-9]+) GPU Dropped.*/\1 \2/'  2>/dev/null ` #驱动获取到的芯片数量 Reign和Dropped的数量
	LOG 1 "解析c20e_info.sh获取的在线的芯片数量为：${tx1_total_num}"
	local tx1_region_num=$(echo "$tx1_total_num" | awk '{print $1}') #region数量
	LOG 1 "解析c20e_info.sh获取的region芯片数量为：${tx1_region_num}"
	local tx1_dropped_num=$(echo "$tx1_total_num" | awk '{print $2}') #dropped数量
	LOG 1 "解析c20e_info.sh获取的dropped芯片数量为：${tx1_dropped_num}"
	
	#判断是否存在数量不一致情况
	# 计算 region 和 dropped 的总和
	local tx1_region_dropped_sum=$((tx1_region_num + tx1_dropped_num)) #算数运算符
	LOG 1 "解析c20e_info.sh获取的region和dropped的芯片总数量为：${tx1_region_dropped_sum}"
	# 比较 tx1_pci_num 和 tx1_region_dropped_sum
	if [ "$1" -ne "$tx1_region_dropped_sum" ]; then
		ResultTxtOutput "ERROR" "寒武纪板卡IP[$3]：硬件接口的芯片数量[${1}]与实际个数[${tx1_region_dropped_sum}]不一致，请联系驱动进行排查"
	else
		ResultTxtOutput "INFO" "寒武纪板卡IP[$3]：硬件接口的芯片数量[${1}]与实际个数[${tx1_region_dropped_sum}]一致，无异常"
	fi
    
	#获取期望的IP数列
	local expected_sequence=()
	for ((i=0; i<$1; i++)); do
		expected_sequence[i]=$((1 + 2 * i))
	done
	
	#提取IP第四位的序列
    local ip_sequence=$(echo "$2" | grep -E "IP:192.168.255.[0-9]+" | awk '{print $5}' | awk 'BEGIN {FS=":"} {print $2}' | awk 'BEGIN {FS="."} {print $4}')
	
	# 将提取的序列转换为数组
	IFS=$'\n' read -r -d '' -a actual_sequence <<< "$ip_sequence"
	LOG 1 "CheckMluIpSequence:获取的实际数列个数${actual_sequence[@]}"
	
	# 遍历 expected_sequence，检查每个数字是否在 actual_sequence 中
	local found=""
	for ((i=0; i<${#expected_sequence[@]}; i++)); do
		found="false"
		for actual in "${actual_sequence[@]}"; do
			if [ "${expected_sequence[i]}" -eq "${actual}" ]; then
				found="true"
				LOG 1 "寒武纪板卡IP[$3]芯片序号[${i}]的芯片IP[192.168.255.${expected_sequence[i]}] 存在"
				break
			fi
		done
		if [ "${found}" = "false" ]; then
			# 输出缺失的数字及其完整的 IP 地址
			ResultTxtOutput "ERROR" "寒武纪板卡IP[$3]缺失芯片序号[${i}]的芯片IP[192.168.255.${expected_sequence[i]}]"
		fi
	done
}

### 芯片温度检查
##$1 寒武纪驱动信息
##$2 板卡ip
CheckMluChipsTemperature() 
{ 
	# 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "CheckMluTemperature入参1检查为空: Expected length: $1"
		return 1
	fi
	if [ -z "$2" ]; then
		LOG 1 "CheckMluTemperature入参2检查为空: Expected length: $2"
		return 1
	fi
    local temperature_threshold=95
    # 运行命令并提取温度序列
	temperature_sequence=$(echo "$1" | grep -E "IP:192.168.255.[0-9]+" | awk '{print $6}' | awk 'BEGIN {FS=":"} {print $2}')
	# 将提取的温度序列转换为数组
	IFS=$'\n' read -r -d '' -a temperatures <<< "$temperature_sequence"
	# 提取完整的 IP 地址序列
	ip_addresses=$(echo "$1" | grep -E "IP:192.168.255.[0-9]+" | awk '{print $5}' | awk 'BEGIN {FS=":"} {print $2}')
	# 将提取的 IP 地址序列转换为数组
	IFS=$'\n' read -r -d '' -a ips <<< "$ip_addresses"
	# 遍历温度序列，检查每个温度值是否大于额定数值
	for ((i=0; i<${#temperatures[@]}; i++)); do
		if [ "${temperatures[i]}" -gt "${temperature_threshold}" ]; then
			# 输出超过额定数值的温度及其完整的 IP 地址
			ResultTxtOutput "ERROR" "寒武纪板卡IP[$2]芯片序号[${i}]的芯片温度为[${temperatures[i]}]超过温度阈值[$temperature_threshold]"
		else
			ResultTxtOutput "INFO" "寒武纪板卡IP[$2]芯片序号[${i}]的芯片温度为[${temperatures[i]}]"
		fi
	done
}

# 获取最后一列，检查是否为[ OK ]
#检验芯片在线状态
## $1 c20e_info.sh的输出，获取最后一列，检查是否为[ OK ]
## $2 板卡ip
CheckMluChipsOnline() 
{
	# 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "CheckMluChipsOnline入参1检查为空: Expected length: $1"
		return 1
	fi
	if [ -z "$2" ]; then
		LOG 1 "CheckMluChipsOnline入参2检查为空: Expected length: $1"
		return 1
	fi
	
    # 运行命令并提取温度序列
	state_sequence=$(echo "$1" | grep -E "IP:192.168.255.[0-9]+" | awk '{print $10}')
	
	# 将提取的温度序列转换为数组
	IFS=$'\n' read -r -d '' -a states <<< "$state_sequence"

	# 提取完整的 IP 地址序列
	ip_addresses=$(echo "$1" | grep -E "IP:192.168.255.[0-9]+" | awk '{print $5}' | awk 'BEGIN {FS=":"} {print $2}')
	
	# 将提取的 IP 地址序列转换为数组
	IFS=$'\n' read -r -d '' -a ips <<< "$ip_addresses"
	
	# 遍历状态序列，检查每个状态是否为 "OK"
	for ((i=0; i<${#states[@]}; i++)); do
		if [ "${states[i]}" != "OK" ]; then
			ResultTxtOutput "ERROR" "寒武纪板卡IP[$2]芯片序号[${i}]的芯片状态为[${states[i]}]"
		else
			ResultTxtOutput "INFO" "寒武纪板卡IP[$2]芯片序号[${i}]的芯片状态为[${states[i]}]"
		fi
	done
}


### 获取MLU驱动版本信息
GetMluDriverVerInfo()
{
    local driver_ver_info=""
    # 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "CheckAxStatus入参1检查为空: Expected length: $1"
        driver_ver_info=""
	elif [ -z "$2" ]; then
		LOG 1 "CheckAxStatus入参2检查为空: Expected length: $2"
		driver_ver_info=""
	elif [ -z "$3" ]; then
		LOG 1 "CheckAxStatus入参3检查为空: Expected length: $3"
		driver_ver_info=""
	elif [ -z "$4" ]; then
		LOG 1 "CheckAxStatus入参4检查为空: Expected length: $4"
		driver_ver_info=""
    else
        driver_ver_info=` SshCmd $1 $2 $3 $4 " c20e_info.sh -v " 2>/dev/null `
	fi
	RecordCommndInfo "$1" "c20e_info.sh -v" "${driver_ver_info}"
	echo "${driver_ver_info}"
}

### 获取MLU驱动信息
GetMluDriverInfo()
{
    local driver_info=""
    # 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "CheckAxStatus入参1检查为空: Expected length: $1"
        driver_info=""
	elif [ -z "$2" ]; then
		LOG 1 "CheckAxStatus入参2检查为空: Expected length: $2"
		driver_info=""
	elif [ -z "$3" ]; then
		LOG 1 "CheckAxStatus入参3检查为空: Expected length: $3"
		driver_info=""
	elif [ -z "$4" ]; then
		LOG 1 "CheckAxStatus入参4检查为空: Expected length: $4"
		driver_info=""
    else
        driver_info=` SshCmd $1 $2 $3 $4 " c20e_info.sh" 2>/dev/null `
	fi
	RecordCommndInfo "$1" "c20e_info.sh" "${driver_info}"
	echo "${driver_info}"
}

GetMluPciNum()
{
    local tx1_pci_num=""
    # 输入参数检查
	if [ -z "$1" ]; then
		LOG 1 "CheckAxStatus入参1检查为空: Expected length: $1"
        tx1_pci_num=""
	elif [ -z "$2" ]; then
		LOG 1 "CheckAxStatus入参2检查为空: Expected length: $2"
		tx1_pci_num=""
	elif [ -z "$3" ]; then
		LOG 1 "CheckAxStatus入参3检查为空: Expected length: $3"
		tx1_pci_num=""
	elif [ -z "$4" ]; then
		LOG 1 "CheckAxStatus入参4检查为空: Expected length: $4"
		tx1_pci_num=""
    else
        tx1_pci_num=` SshCmd $1 $2 $3 $4 " lspci -tv|grep 0220|wc -l " 2>/dev/null ` #硬件接口的芯片数量
	fi
	RecordCommndInfo "$1" "lspci -tv|grep 0220|wc -l" "${tx1_pci_num}"
	echo "${tx1_pci_num}"
}

### 寒武纪环境检查
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码 
CheckMluEnvironment()
{
    local mlu_driver_ver_info=` GetMluDriverVerInfo $1 $2 $3 $4 `
    local mlu_driver_info=` GetMluDriverInfo $1 $2 $3 $4 `
    local mlu_pci_num=` GetMluPciNum $1 $2 $3 $4 `
    
    #检查版本驱动
	StartCheckItemJsonOutput "${CHECK_ENV_ITEM_DRIVER}" #开始检查项JSON输出
	CheckMluDriver "${mlu_driver_ver_info}" "$1"
    IAEUpdateProgress
	EndCheckItemJsonOutput #结束检查项JSON输出  
    
    #检测温度情况
	StartCheckItemJsonOutput "${CHECK_ENV_ITEM_TEMPERATURE}" #开始检查项JSON输出
	CheckMluChipsTemperature "${mlu_driver_info}" "$1"
    IAEUpdateProgress
	EndCheckItemJsonOutput #结束检查项JSON输出
    
    #检测运行状态
	StartCheckItemJsonOutput "${CHECK_ENV_ITEM_CHIPONLINE}" #开始检查项JSON输出
	CheckMluChipsOnline "$mlu_driver_info" "$1"
    IAEUpdateProgress
	EndCheckItemJsonOutput #结束检查项JSON输出
    
    #检查芯片NG情况
    StartCheckItemJsonOutput "${CHECK_ENV_ITEM_NG}" #开始检查项JSON输出
	CheckMluIpSequence "${mlu_pci_num}" "${mlu_driver_info}" "$1"
    IAEUpdateProgress
	EndCheckItemJsonOutput #结束检查项JSON输出
}
