#!/bin/bash
########## 芯 片 免 密 登 录 检 查 ###########
### 获取芯片免密检查的命令
### $1板卡类型
### $2芯片编号
### $3板卡IP
GetPasswordFreeCheckCommand()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "AX")
        command=" ssh 192.168.254.$ipLastIndex \"  \" "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex \"  \" "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} \"  \" "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex \"  \" "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
   esac
   echo "${command}"
}

### 芯片免密登录检查
###$1 板卡ip
###$2 板卡ssh端口
###$3 板卡ssh用户名
###$4 板卡ssh密码 
PasswordFreeCheck()
{
    LOG 1 "开始检查IAE【$1】芯片免密登录检查"
    #根据IP获取板卡名称
    local engine_name=$(Get_Engine_Name $1)
	if [ -z "${engine_name}" ]; then
		ResultTxtOutput "WARNING" "PasswordFreeCheck获取板卡IP【$1】对应的板卡类型为空，检验失败"
		return 1
	fi

    # 获取板卡名称和芯片个数
    local chip_num=$(Get_Chip_Num $1)
	if [ ${chip_num} == 0 ]; then
		ResultTxtOutput "WARNING" "PasswordFreeCheck获取板卡IP【$1】对应的芯片个数为空，检验失败"
		return 1
	fi

	LOG 1 "IAE【$1】获取的板卡类型【${engine_name}】,芯片个数为【${chip_num}】"
	#遍历芯片单元
    local index=0
    for ((; index < chip_num; index++))
    do
		local command=$(GetPasswordFreeCheckCommand "$engine_name" "$index" "$1")
		if [ -z "${command}" ]; then
			ResultTxtOutput "WARNING" "获取板卡IP【$1】芯片【${index}】GetPasswordFreeCheckCommand为空，检验失败"
			continue
		fi
		chip_ssh_login_result=$(timeout 40s ${SSHPASS} -p $4 ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p $2 $3@$1 "$command" 2>&1)
        check_status=$?
        case ${check_status} in
            0)
                ResultTxtOutput "INFO" "免密登录成功"
                ;;
            5)
                ResultTxtOutput "WARNING" "$1 ssh登录密码错误"
                ;;
            127)
                ResultTxtOutput "WARNING" "【$1】【${index}】号芯片远程命令执行异常，远程命令：$command"
                ;;
            255)
                if [[ "${chip_ssh_login_result}" == *"publickey"* ]]
                then
                    ResultTxtOutput "ERROR" "【$1】【${index}】号芯片免密登录失效"
                else
                    ResultTxtOutput "ERROR" "【$1】【${index}】号芯片执行命令：${command} 发生未知异常，状态码: ${check_status}，异常信息：${chip_ssh_login_result}"
                fi
                ;;
            *) 
                ResultTxtOutput "ERROR" "【$1】【${index}】号芯片执行命令：${command} 发生未知异常，异常状态码：: ${check_status}，异常信息：${chip_ssh_login_result}"
                ;;
        esac
    done
}
