#!/bin/bash

### 检查IAM的coredump是否开启
Check_IAM_Coredump()
{
    LOG 1 "开始检查iam是否开启coredump"
	# 查看IAM的core是否开启
	local cmd="ulimit -c | grep \"unlimited\" "
	local result=$(ulimit -c | grep "unlimited")
	RecordCommndInfo "iam" "${cmd}" "${result}"
	if [ "$result" == "unlimited" ];then
		ResultTxtOutput "WARNING" "IAM已开启coredump" 
		# core开启检查有没有core产生
		local cmd_core_file_all="ls -al /root | grep \"core*\" 2>/dev/null"
		local core_file_all=` ls -al /root | grep "core*" 2>/dev/null `
		RecordCommndInfo "iam" "${cmd_core_file_all}" "${core_file_all}"
		if [ "${core_file_all}" != "" ]
		then
			ResultTxtOutput "ERROR" "IAM进程出现过崩溃" 
		else
			ResultTxtOutput "INFO" "IAM进程没有出现过崩溃" 
		fi
	else
		ResultTxtOutput "INFO" "IAM未开启coredump" 
	fi
	LOG 1 "结束检查iam是否开启coredump"
}

### 检查BW的coredump是否开启
###$1 板卡ip
###$2 板卡ssh端口
###$3 板卡ssh用户名
###$4 板卡ssh密码 
Check_BW_Coredump()
{
    LOG 1 "开始检查服务器【$1】BW是否开启coredump"
	# 查看BW的core是否开启
	local cmd="ulimit -c | grep \"unlimited\" "
	local result=$(SshCmd $1 $2 $3 $4 "$cmd")
	RecordCommndInfo "$1" "${cmd}" "${result}"
	if [ "$result" == "unlimited" ];then
		ResultTxtOutput "WARNING" "BW已开启coredump" 
		# core开启检查有没有core产生
		local cmd_core_file_all="ls -al /var/log/proxylog/corefiles | grep \"core*\" 2>/dev/null"
		local core_file_all=$(SshCmd $1 $2 $3 $4 "$cmd_core_file_all")
		RecordCommndInfo "$1" "${cmd_core_file_all}" "${core_file_all}"
		if [ "${core_file_all}" != "" ]
		then
			ResultTxtOutput "ERROR" "BW进程出现过崩溃" 
		else
			ResultTxtOutput "INFO" "BW进程没有出现过崩溃" 
		fi
	else
		ResultTxtOutput "INFO" "BW未开启coredump" 
	fi
	LOG 1 "结束检查服务器【$1】BW是否开启coredump"
}


### 检查IAE的coredump是否开启
###$1 板卡ip
###$2 板卡ssh端口
###$3 板卡ssh用户名
###$4 板卡ssh密码 
Check_IAE_Coredump()
{
    LOG 1 "开始检查IAE【$1】是否开启coredump"
    #根据IP获取板卡名称
    local engine_name=$(Get_Engine_Name $1)
	if [ -z "${engine_name}" ]; then
		LOG 1 "Check_IAE_Coredump获取板卡IP【$1】对应的板卡类型为空，检验失败"
		return 1
	fi

    # 获取板卡名称和芯片个数
    local chip_num=$(Get_Chip_Num $1)
	if [ ${chip_num} == 0 ]; then
		LOG 1 "Check_IAE_Coredump获取板卡IP【$1】对应的芯片个数为空，检验失败"
		return 1
	fi

	LOG 1 "IAE【$1】获取的板卡类型【${engine_name}】,芯片个数为【${chip_num}】"
	#遍历芯片单元
    local index=0
    for ((; index < chip_num; index++))
    do
		local command=$(Get_Core_Command_For_Pid "$engine_name" "$index" "$1")
		if [ -z "${command}" ]; then
			ResultTxtOutput "WARNING" "获取板卡IP【$1】芯片【${index}】Get_Core_Command_For_Pid为空，检验失败"
			continue
		fi
		local iae_pid=$(SshCmd "$1" "$2" "$3" "$4" "$command" 2>/dev/null)
		RecordCommndInfo "$1" "${command}" "${iae_pid}"
		if [ -z "${iae_pid}" ]; then
			ResultTxtOutput "WARNING" "使用命令【${command}】获取板卡IP【$1】的PID为空，芯片【${index}】检验失败"
			continue
		fi
		
		# GPU特殊处理
		local Check_CoreSize_Command=$(Get_Check_CoreSize_Command "$engine_name" "$index" "$1" "$iae_pid")
		if [ -z "${Check_CoreSize_Command}" ]; then
			ResultTxtOutput "WARNING" "${engine_name}:$1:${index}号芯片，core检查失败"
			continue
		fi
		local core_size_info=$(SshCmd "$1" "$2" "$3" "$4" "$Check_CoreSize_Command" 2>/dev/null)
		RecordCommndInfo "$1" "${Check_CoreSize_Command}" "${core_size_info}"
		if [ -z "${core_size_info}" ]; then
			ResultTxtOutput "WARNING" "${engine_name}:$1:${index}号芯片，core检查失败，相关信息获取失败"
			continue
		fi
		# 空格分割信息为数组
		local core_size=$(echo "$core_size_info"| awk '/Max core file size/ {print $5}')
		if [ "${core_size}" == "unlimited" ];then
			ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片，已开启core"  #增加报告输出结果
			# core开启检查有没有core产生
			local Check_CoreFile_Command=$(Get_Check_CoreFile_Command "$engine_name" "$index" "$1")
			local core_file_all=` SshCmd $1 $2 $3 $4 "$Check_CoreFile_Command" 2>/dev/null `
			RecordCommndInfo "$1" "${Check_CoreFile_Command}" "${core_file_all}"
			if [ "${core_file_all}" != "" ];then
				ResultTxtOutput "ERROR" "${engine_name}:$1:${index}号芯片，存在core文件"
			else
				ResultTxtOutput "INFO" "${engine_name}:$1:${index}号芯片，不存在core文件"
			fi
		else
			ResultTxtOutput "INFO" "${engine_name}:$1:${index}号芯片, 未开启core"  #增加报告输出结果
		fi
    done
	LOG 1 "结束检查IAE【$1】是否开启coredump"
}

### 获取iae进程号
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Core_Command_For_Pid()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="cat /opt/wise_iae/ia12k/data$2/run/iae.pid"
        ;;
    "AX")
        command=" ssh 192.168.254.$ipLastIndex \"cat /opt/wise_iae/ia12k/data$2/run/iae.pid\" "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex \"cat /opt/wise_iae/ia12k/data$2/run/iae.pid\" "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} \"cat /opt/wise/ia12k/run/iae.pid\" "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex \"cat /opt/wise_iae/ia12k/data$2/run/iae.pid\" "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
   esac
   echo "${command}"
}

### 获取芯片core大小限制
### $1板卡名称
### $2芯片编号
### $3板卡IP
### $4进程PID号
Get_Check_CoreSize_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="cat /proc/$4/limits | grep \"core file size\" "
        ;;
    "AX")
        command=" ssh 192.168.254.$ipLastIndex  cat /proc/$4/limits | grep \"core file size\" "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex  cat /proc/$4/limits | grep \"core file size\" "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]}  cat /proc/$4/limits | grep \"core file size\"  "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex  cat /proc/$4/limits | grep \"core file size\"  "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
   esac
   echo "${command}"
 }
 
### 获取查看DEBUG日志的命令 分平台
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Check_CoreFile_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command=" ls -al /root | grep \"core*\" "
        ;;
    "AX")
        command=" ssh 192.168.254.$ipLastIndex  ls -al /opt/wise_iae/ia12k/data$2/corefiles | grep \"core*\"  "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex ls -al /opt/wise_iae/ia12k/data$2/corefiles | grep \"core*\"  "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]}  ls -al /program/corefiles | grep \"core*\" "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex  ls -al /opt/wise_iae/ia12k/data$2/corefiles | grep \"core*\"  "
        ;;
     *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
    esac
	echo "${command}"
 }
 

 