#!/bin/bash
cpu_usage_threshold=90
### 获取总的 CPU 使用率
### $1 TOP 信息
get_cpu_usage() 
{
    # 获取 top 命令的输出
    local top_output="$1"
	# 提取 CPU 使用率相关的行
    local cpu_line=$(echo "$top_output" | grep -i -E "^cpu:|^%cpu")
	LOG 1 "获取CPU使用率的内容：【${cpu_line}】"
    # 提取各个百分比值
    local user=$(echo "$cpu_line" | awk 'BEGIN { FS = "[, \t]+" } {print $2}' | sed 's/%//')
    local sys=$(echo "$cpu_line" | awk 'BEGIN { FS = "[, \t]+" } {print $4}' | sed 's/%//')
    local nice=$(echo "$cpu_line" | awk 'BEGIN { FS = "[, \t]+" } {print $6}' | sed 's/%//')
    local idle=$(echo "$cpu_line" | awk 'BEGIN { FS = "[, \t]+" } {print $8}' | sed 's/%//')
    local iowait=$(echo "$cpu_line" | awk 'BEGIN { FS = "[, \t]+" } {print $10}' | sed 's/%//')
    local irq=$(echo "$cpu_line" | awk 'BEGIN { FS = "[, \t]+" } {print $12}' | sed 's/%//')
    local softirq=$(echo "$cpu_line" | awk 'BEGIN { FS = "[, \t]+" } {print $14}' | sed 's/%//')

    # 计算总的 CPU 使用率（排除 idle）
    local total_usage=$(echo "scale=2; 100 - $idle" | bc)

    # 返回总的 CPU 使用率
    echo "$total_usage"
}

### 获取CPU使用率高的前三个进程名称
### $1 TOP 信息
get_top_cpu_process()
{
	# 获取行号
	local file="$1"
	local pid_row=$(echo "$file" | awk '/^[[:space:]]*PID/{print NR}')
	local end_row=$((pid_row + 3))
	LOG 1 "pid_row【$pid_row】,start_row【$pid_row】，end_row【$end_row】"

	# 获取 %CPU 列数
	local cpu_col=$(echo "$file" | awk -v pid_row="$pid_row" '
		NR == pid_row {
			for (i=1; i<=NF; i++) if ($i == "%CPU") {cpu_col=i; break}
		}
		END {print cpu_col}
	')

	# 获取 COMMAND 列数
	local command_col=$(echo "$file" | awk -v pid_row="$pid_row" '
		NR == pid_row {
			for (i=1; i<=NF; i++) if ($i == "COMMAND") {command_col=i; break}
		}
		END {print command_col}
	')

	LOG 1 "TOP命令获取的PID所在行为【$pid_row】,CPU%所在列为【$cpu_col】，COMMAND所在列为【$command_col】"
	# 获取 CPU 使用率高的三个进程
	local cpu_info=$(echo "$file" | awk -v start_row="$pid_row" -v end_row="$end_row" -v cpu_col="$cpu_col" -v command_col="$command_col" '
	NR > start_row && NR <= end_row {
		print $command_col, $cpu_col
	}')
	LOG 1 "TOP命令获取的cpu_info为【$cpu_info】"
	local cpu_line=$(echo "$cpu_info" | awk 'BEGIN{ORS="";FS=" ";OFS=""}{if (NR>1) printf "|"; printf "%s（%s）", $1, $2} END{print ""}')
	# 输出结果
    echo "$cpu_line"
}

#########获取IAM运行状态#######
Check_IAM_Cpu_Usage()
{
    LOG 1 "开始检查【iam】的CPU使用率"
	# 检查IAM CPU使用率
	local cmd="top -b -n1 2>/dev/null"
	local top_iam_info=$(top -b -n1 2>/dev/null)
	RecordCommndInfo "iam" "${cmd}" "${top_iam_info}"
	if [ -z "$top_iam_info" ]; then
		ResultTxtOutput "WARNING" "【IAM】${cmd}执行结果为空" 
		return 1
	fi
	cpu_usage=$(get_cpu_usage "$top_iam_info")
	#比对
	local compare=$(echo "$cpu_usage >= ${cpu_usage_threshold}" | bc)
	LOG 1 "【IAM】的CPU使用率【$cpu_usage】，阈值【${cpu_usage_threshold}】，是否大于的比对结果【$compare】"
    if [ "$compare" -eq 1 ]; then
		local top_process=$(get_top_cpu_process "$top_iam_info")
		ResultTxtOutput "WARNING" "IAM的CPU使用率过高，当前使用率:$cpu_usage，cpu使用率Top3:${top_process}"
	else
		ResultTxtOutput "INFO" "IAM的CPU使用率正常，当前使用率:$cpu_usage"
    fi

}

#########获取BW运行状态#######
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码 
Check_BW_Cpu_Usage()
{
	# 检查bw CPU使用率
	local cmd="top -b -n1 2>/dev/null"
	local top_bw_info=$(SshCmd $1 $2 $3 $4 "$cmd" 2>/dev/null)
	RecordCommndInfo "$1" "${cmd}" "${top_bw_info}"
	if [ -z "$top_bw_info" ]; then
		ResultTxtOutput "WARNING" "【$1】BW执行${cmd}执行结果为空" 
		return 1
	fi
	cpu_usage=$(get_cpu_usage "$top_bw_info")
	#计算出阈值
	local compare=$(echo "$cpu_usage >= ${cpu_usage_threshold}" | bc)
	LOG 1 "【$1】bw的CPU使用率【$cpu_usage】，阈值【${cpu_usage_threshold}】，是否大于的比对结果【$compare】"
    if [ "$compare" -eq 1 ]; then
		local top_process=$(get_top_cpu_process "$top_bw_info")
		ResultTxtOutput "WARNING" "【$1】bw的CPU使用率过高，当前使用率:$cpu_usage，cpu使用率Top3:${top_process}"
	else
		ResultTxtOutput "INFO" "【$1】bw的CPU使用率正常，当前使用率:$cpu_usage"
    fi
}

#########获取板卡运行状态#######
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码 
Check_IAE_Cpu_Usage()
{
    LOG 1 "开始检查【$1】的CPU使用率"
  
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)

	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	local engine_name=$(Get_Engine_Name $1)

	local index=0
	for ((; index < chip_num; index++))
	do
		# GPU只需要检查一次
		if [ "$engine_name" == "GPU" ]&& [ "$index" -ne 0 ];then
		  return 0
		fi
		local command=$(Get_CPU_Command "$engine_name" "$index" "$1")
		local iae_info=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
		RecordCommndInfo "$1" "${command}" "${iae_info}"
		#返回值的后续处理
		if [ -z "$iae_info" ]; then
			ResultTxtOutput "WARNING" "Check_IAE_Cpu_Usage：${engine_name}:${1}:${index}号芯片,${command}命令返回为空"
			continue
		fi
	
		cpu_usage=$(get_cpu_usage "$iae_info")
		if [ -z "$cpu_usage" ]; then
			ResultTxtOutput "WARNING" "Check_IAE_Cpu_Usage：${engine_name}:${1}:${index}号芯片,get_cpu_usage $iae_info命令返回为空"
			continue
		fi
		#比对
		local compare=$(echo "$cpu_usage >= ${cpu_usage_threshold}" | bc)
		LOG 1 "【${engine_name}:${1}:${index}号芯片的CPU使用率【$cpu_usage】，阈值【${cpu_usage_threshold}】，是否大于的比对结果【$compare】"
		if [ "$compare" -eq 1 ]; then
			local top_process=$(get_top_cpu_process "$iae_info")
			if [ "$engine_name" == "GPU" ];then
				ResultTxtOutput "WARNING" "${engine_name}:${1}:CPU使用率过高，当前使用率:${cpu_usage}，cpu使用率Top3:${top_process}"
			else
				ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片, CPU使用率过高，当前使用率:${cpu_usage}，cpu使用率Top3:${top_process}"
			fi
		else
			if [ "$engine_name" == "GPU" ];then
				ResultTxtOutput "INFO" "${engine_name}:${1}:CPU使用率正常，当前使用率:${cpu_usage}"
			else
				ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片, CPU使用率正常，当前使用率:${cpu_usage}"
			fi
		fi
	done
}

### 获取CPU使用率查询命令
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_CPU_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="top -b -n1"
        ;;
    "AX")
        command=" ssh 192.168.254.$ipLastIndex \"top -b -n1\" "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex \"top -b -n1\" "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} \"top -b -n1\" "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex \"top -b -n1\" "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的新类型：$1"
	    ;;
   esac
   echo ${command}
   
 }