#!/bin/bash

THRESHOLD=85 #磁盘容量检查阈值

### IAM磁盘剩余容量检查
Check_IAM_Disk()
{
    LOG 1 "开始检查IAM磁盘剩余容量"
    local cmd="df -h | grep -v -e \"tmpfs\" -e \"文件系统\" -e \"Filesystem\""
    local iam_info=$(df -h | grep -v -e "tmpfs" -e "文件系统" -e "Filesystem")
    RecordCommndInfo "iam" "${cmd}" "${iam_info}"
    #检查IAM磁盘使用率
    while read -r line
	do
	    # 处理单行数据
	    local FILESYSTEM=$(echo "$line" | awk '{print $1}') #文件系统名
	    local TOTAL=$(echo "$line" | awk '{print $2}') #磁盘总量
	    local USAGE=$(echo "$line" | awk '{print $5}' | sed 's/%//') #使用率
		local mount_on=$(echo "$line" | awk '{print $6}' | sed 's/%//') #挂载点

	    # 超过阈值会输出提示信息
		if [ $USAGE -ge 100 ]
		then
			ResultTxtOutput "WARNING" "IAM的${mount_on}使用率以达100，请尽快清理"
	    elif [ $USAGE -ge $THRESHOLD ]
		then
		    ResultTxtOutput "WARNING" "IAM的${mount_on}使用率过高，总容量：${TOTAL}，当前使用率:${USAGE}%"
	    else
		    ResultTxtOutput "INFO" "IAM的${mount_on}使用率正常，总容量：${TOTAL}，当前使用:${USAGE}%"
	    fi
    done <<< "${iam_info}"
}


### BW磁盘剩余容量检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码
Check_BW_Disk()
{
    LOG 1 "开始检查bw磁盘剩余容量"
    local command="df -h | grep -v -e \"Filesystem\" -e \"tmpfs\" -e \"文件系统\" "
	local bw_info=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
	RecordCommndInfo "$1" "${command}" "${bw_info}"
	if [ -z "${bw_info}" ]; then
		ResultTxtOutput "WARNING" "服务器${1}，磁盘校验失败，${command}命令返回为空"
		continue
	fi
	
	
	#检查BW磁盘使用率
	while read -r line; do
		# 处理单行数据
		local FILESYSTEM=$(echo $line | awk '{print $1}') #文件系统名
		local TOTAL=$(echo $line | awk '{print $2}') #磁盘总量
		local USAGE=$(echo $line | awk '{print $5}' | sed 's/%//') #使用率
		local mount_on=$(echo "$line" | awk '{print $6}' | sed 's/%//') #挂载点

		# 超过阈值会输出提示信息 
		if [ $USAGE -ge 100 ];then
			ResultTxtOutput "ERROR" "${1},${mount_on}的磁盘总量已达到100%，请尽快清理"
		elif [ $USAGE -ge $THRESHOLD ];then
			ResultTxtOutput "WARNING" "${1},${mount_on}的磁盘总量：${TOTAL}，磁盘使用率过高，当前磁盘使用率:${USAGE}%"
		else
			ResultTxtOutput "INFO" "${1},${mount_on}的磁盘总量：${TOTAL}，磁盘使用率正常，当前磁盘使用率:${USAGE}%"
		fi
	done <<< "$bw_info"
	LOG 1 "结束检查bw磁盘剩余容量"
}





### IAE磁盘剩余容量检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_IAE_Disk()
{
    LOG 1 "开始检查IAE【$1】磁盘剩余容量"

	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)
	
	# GPU、爱芯、寒武纪、昆仑3只需要检查服务器的磁盘即可
	if [ "$engine_name" == "GPU" ] || [ "$engine_name" == "AX" ] || [ "$engine_name" == "昆仑3" ] || [ "$engine_name" == "寒武纪" ];then
		return 0
	fi
		
	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	local engine_name=$(Get_Engine_Name $1)
	
	local index=0
	for ((; index < chip_num; index++))
	do
		local command=$(Get_CheckDisk_Command "$engine_name" "$index" "$1")
		local iae_info=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
		RecordCommndInfo "$1" "${command}" "${iae_info}"
		if [ -z "${iae_info}" ]; then
			ResultTxtOutput "WARNING" "Check_IAE_Disk：${engine_name}:${1}:${index}号芯片:${command}命令返回为空"
			continue
		fi
		
		
		#检查IAE磁盘使用率
		while read -r line; do
			# 处理单行数据
			local FILESYSTEM=$(echo $line | awk '{print $1}') #文件系统名
            local TOTAL=$(echo $line | awk '{print $2}') #磁盘总量
            local USAGE=$(echo $line | awk '{print $5}' | sed 's/%//') #使用率
			local mount_on=$(echo "$line" | awk '{print $6}' | sed 's/%//') #挂载点
			
			# 超过阈值会输出提示信息
			if [ "$engine_name" == "昆仑2" ];then
				if [ $USAGE -ge 100 ];then
					ResultTxtOutput "ERROR" "${1},${mount_on}的磁盘总量已达到100%，请尽快清理"
				elif [ $USAGE -ge $THRESHOLD ];then
					ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片,${mount_on}的磁盘总量：${TOTAL}，磁盘使用率过高，当前磁盘使用率:${USAGE}%"
				else
					ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片,${mount_on}的磁盘总量：${TOTAL}，磁盘使用率正常，当前磁盘使用率:${USAGE}%"
				fi
			else
				if [ $USAGE -ge 100 ];then
					ResultTxtOutput "ERROR" "${1},${mount_on}的磁盘总量已达到100%，请尽快清理"
				elif [ $USAGE -ge $THRESHOLD ];then
					ResultTxtOutput "WARNING" "${engine_name}:${1}:${mount_on}的磁盘总量：${TOTAL}，磁盘使用率过高，当前磁盘使用率:${USAGE}%"
				else
					ResultTxtOutput "INFO" "${engine_name}:${1}:${mount_on}的磁盘总量：${TOTAL}，磁盘使用率正常，当前磁盘使用率:${USAGE}%"
				fi
			fi
		done <<< "$iae_info"
	done
}


### 获取磁盘容量检查命令
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_CheckDisk_Command(){
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="df -h | grep -v -e \"Filesystem\" -e \"tmpfs\" -e \"文件系统\" "
        ;;
    "AX")
        command="df -h | grep -v -e \"Filesystem\" -e \"tmpfs\" -e \"文件系统\" "
        ;;
    "寒武纪")
        command="df -h | grep -v -e \"Filesystem\" -e \"tmpfs\" -e \"文件系统\" "
        ;;
    "昆仑3")
        command="df -h | grep -v -e \"Filesystem\" -e \"tmpfs\" -e \"文件系统\" "
        ;;
    "昆仑2")
	    command=" ssh ${BOARD_CHIP_IP[$3-$2]}  df -h | grep -v -e \"192.168.255\" -e \"tmpfs\" -e \"Filesystem\" -e \"none\" "
	    ;;
     *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
    esac
    echo "$command"
 }
 
 
 