#!/bin/bash

#系统日志检查，检查是否存在智能相关的进程信息

### IAM的Dmesg校验
Check_IAM_Dmesg()
{
	#获取dmesg的日志
	local dmesg_info=$(dmesg | grep -E -i "iam" |grep -E -i "kill|segfault|error|fail|warn" 2>/dev/null)
	RecordCommndInfo "IAM" " dmesg | grep -E -i \"iam\" |grep -E -i \"kill|segfault|error|fail|warn\" "  "${dmesg_info}"
	#校验日志
	if [ -z "${dmesg_info}" ];then
		ResultTxtOutput "INFO" "IAM的Dmesg系统日志无异常"
		return 1
	fi
	#获取日志信息，并进行打印
	IFS=$'\n' read -d '' -ra dmesg_output_info <<< "${dmesg_info}"
	# 初始化一个空变量来缓存日志信息
	local cached_string=""
	local last_cached_string=""  #最后一条数据
	local num=0
	local last_actual_time="" #最后一次异常时间
	# 处理每一行 dmesg 输出
	for line in "${dmesg_output_info[@]}"
	do
		num=$((num + 1))
		last_cached_string="$line"
		 # 提取相对时间戳（假设时间戳在行首）
		local timestamp=$(echo "$line" | sed -n 's/\[ *\([0-9.]*\)\].*/\1/p')
		# 计算实际时间戳
		local unix_time=` echo "$(date +%s) - $(cat /proc/uptime | cut -f 1 -d' ') + ${timestamp} " | bc `  
		local actual_time=$(date -d "@${unix_time}" '+%Y-%m-%d %H:%M:%S')
		if [ -z "${actual_time}" ];then
			LOG 1 "iam执行【date -d \"@${unix_time}\" '+%Y-%m-%d %H:%M:%S'】返回为空"
			continue
		fi
		# 输出实际时间戳和剩余部分
		last_cached_string=$(echo "$line" | sed "s/^\[\s*\($timestamp\)\]/[\1]/" | sed "s/$timestamp/$actual_time/g")
		last_actual_time="$actual_time"
		cached_string+="${last_cached_string}\n"
	done
	#防止时间戳转换失败
	if [ -n "${cached_string}" ];then
		RecordCommndInfo "IAM" " dmesg | grep \"iam\" 转换时间戳之后的数据 "  "${cached_string}"
	fi
	
	#防止最后一次系统日志的时间戳转换失败
	if [ -z "${last_actual_time}" ];then
		local log_message=$(echo -e "Dmesg系统日志存在异常信息,总条数:${num}，转换日志时间失败")
		ResultTxtOutput "ERROR" "${log_message}"
		return
	fi
	
	#增加判断，判断最后一次时间增加不同等级的日志提示
	# 获取当前时间
	local current_time=$(date '+%Y-%m-%d %H:%M:%S')

	# 计算时间差（秒数）
	local actual_time_seconds=$(date -d "${last_actual_time}" +%s)
	local current_time_seconds=$(date -d "${current_time}" +%s)

	# 计算秒数差值
	local time_difference_seconds=$((current_time_seconds - actual_time_seconds))

	# 处理负数情况
	if [ $time_difference_seconds -lt 0 ]; then
		time_difference_seconds=$(( -time_difference_seconds ))  # 将负数转换为正数
	fi
	
	# 将秒数差值转换为天数
	time_difference_days=$((time_difference_seconds / 86400))
	LOG 1 "【IAM】服务器计算的时间间隔为【$time_difference_days】day"
	# 检查时间差是否大于7天
	local log_message=$(echo -e "【IAM】Dmesg系统日志存在异常信息,总条数:${num}，最后一次异常时间:【${last_actual_time}】")
	# 检查时间差
	if [ $time_difference_days -lt 7 ]; then  #小于7天
		ResultTxtOutput "ERROR" "${log_message}"
	elif [ $time_difference_days -ge 7 ] && [ $time_difference_days -le 30 ]; then
		ResultTxtOutput "WARNING" "${log_message}"
	elif [ $time_difference_days -gt 30 ]; then
		ResultTxtOutput "INFO" "${log_message}"
	fi
	
	
}

### BW的Dmesg校验
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_BW_Dmesg()
{
	#获取dmesg的日志
	local dmesg_info=$(SshCmd $1 $2 $3 $4 "dmesg | grep -E -i \"braswell|bw\" |grep -E -i \"kill|segfault|error|fail|warn\" " 2>/dev/null)
	RecordCommndInfo "$1" "dmesg | grep -E -i \"braswell|bw\" |grep -E -i \"kill|segfault|error|fail|warn\" "  "${dmesg_info}"
	
	#校验日志
	if [ -z "${dmesg_info}" ];then
		ResultTxtOutput "INFO" "【$1】BW的Dmesg系统日志无异常"
		return 1
	fi
	#获取日志信息，并进行打印
	IFS=$'\n' read -d '' -ra dmesg_output_info <<< "${dmesg_info}"
	# 初始化一个空变量来缓存日志信息
	local cached_string=""
	local last_cached_string=""  #最后一条数据
	local last_actual_time=""  #最后一条数据的时间
	local num=0
	# 处理每一行 dmesg 输出
	for line in "${dmesg_output_info[@]}"
	do
		num=$((num + 1))
		last_cached_string="$line"
		# 提取相对时间戳（假设时间戳在行首）
		local timestamp=$(echo "$line" | sed -n 's/\[ *\([0-9.]*\)\].*/\1/p')
		# 计算实际时间戳
		local cmd=" echo \"\$(date +%s) - \$(cat /proc/uptime | cut -f 1 -d' ') + ${timestamp}\" | awk '{ sum=\$1-\$3+${timestamp}; printf \"%.5f\", sum}'"
		local actual_second_diff=$(SshCmd "$1" "$2" "$3" "$4" "$cmd" 2>/dev/null)
		RecordCommndInfo "$1" "$cmd"  "${actual_second_diff}"
		if [ -z "${actual_second_diff}" ];then
			LOG 1 "服务器【$1】执行【$cmd】返回为空"
			continue
		fi
		# 根据偏移的秒数计算出实际日期时间
		local actual_time=$(date -d "@${actual_second_diff}" '+%Y-%m-%d %H:%M:%S')
		# 输出实际时间戳和剩余部分
		last_cached_string=$(echo "$line" | sed "s/^\[\s*\($timestamp\)\]/[\1]/" | sed "s/$timestamp/$actual_time/g")
		last_actual_time="$actual_time"
		cached_string+="${last_cached_string}\n"
	done
	#记录转换时间戳后的日志
	if [ -n "${cached_string}" ];then
		RecordCommndInfo "$1" " dmesg | grep \"bw\" 转换时间戳后的日志："  "${cached_string}"
	fi
	
	#防止最后一次系统日志的时间戳转换失败
	if [ -z "${last_actual_time}" ];then
		local log_message=$(echo -e "Dmesg系统日志存在异常信息,总条数:${num}，转换日志时间失败")
		ResultTxtOutput "ERROR" "${log_message}"
		return
	fi
	
	#增加判断，判断最后一次时间增加不同等级的日志提示
	# 获取当前时间
	local current_time=$(date '+%Y-%m-%d %H:%M:%S')

	# 计算时间差（秒数）
	local actual_time_seconds=$(date -d "${last_actual_time}" +%s)
	local current_time_seconds=$(date -d "${current_time}" +%s)

	# 计算秒数差值
	local time_difference_seconds=$((current_time_seconds - actual_time_seconds))

	# 处理负数情况
	if [ $time_difference_seconds -lt 0 ]; then
		time_difference_seconds=$(( -time_difference_seconds ))  # 将负数转换为正数
	fi

	# 将秒数差值转换为天数
	time_difference_days=$((time_difference_seconds / 86400))
	LOG 1 "【$1】【BW】服务器计算的时间间隔为【$time_difference_days】day"
	# 检查时间差是否大于7天
	local log_message=$(echo -e "【$1】【BW】Dmesg系统日志存在异常信息,总条数:${num}，最后一次异常时间:【${last_actual_time}】")
	# 检查时间差
	if [ $time_difference_days -lt 7 ]; then  #小于7天
		ResultTxtOutput "ERROR" "${log_message}"
	elif [ $time_difference_days -ge 7 ] && [ $time_difference_days -le 30 ]; then
		ResultTxtOutput "WARNING" "${log_message}"
	elif [ $time_difference_days -gt 30 ]; then
		ResultTxtOutput "INFO" "${log_message}"
	fi
	
}



### IAE的Dmesg的校验
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_IAE_Dmesg()
{
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)

	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	
	#获取对应的文件
	local index=0
	for ((; index < chip_num; index++))
	do
		# GPU只需要检查一次
		if [ "$engine_name" == "GPU" ];then
			if [ "$index" -ne 0 ];then
				return 0
			fi
		fi
		#获取Dmesg信息
		local get_dmesg_cmd=$(Get_Dmesg_Command "$engine_name" "$index" "$1")
		local cmd_ret=` SshCmd $1 $2 $3 $4 "$get_dmesg_cmd" 2>/dev/null `
		RecordCommndInfo "$1" "${get_dmesg_cmd}" "${cmd_ret}"
		#校验日志
		if [ -z "${cmd_ret}" ]; then
			if [ "$engine_name" == "GPU" ];then
				ResultTxtOutput "INFO" "${engine_name}:${1}:Dmesg系统日志不存在异常"
			else
				ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片:Dmesg系统日志不存在异常"
			fi
			continue
		fi
		#获取日志信息，并进行打印
		IFS=$'\n' read -d '' -ra dmesg_output_info <<< "${cmd_ret}"
	
		#存在内容需要进行日志校验
		# 初始化一个空变量来缓存日志信息
		local cached_string=""
		local last_cached_string=""  #最后一条数据
		local last_actual_time=""  #最后一条数据的时间
		local num=0
		# 处理每一行 dmesg 输出
		for line in "${dmesg_output_info[@]}"
		do
			num=$((num + 1))
			# 提取相对时间戳（假设时间戳在行首）
			local timestamp=$(echo "$line" | sed -n 's/\[ *\([0-9.]*\)\].*/\1/p')
			# 计算实际时间戳
			local get_timespn_cmd=$(Get_Dmesg_TimeSpan_Command "$engine_name" "$index" "$1" "${timestamp}")
			local timespn_cmd_ret=` SshCmd $1 $2 $3 $4 "$get_timespn_cmd" 2>/dev/null `
			RecordCommndInfo "$1" "$get_timespn_cmd"  "${timespn_cmd_ret}"
			if [ -z "${timespn_cmd_ret}" ];then
				LOG 1 "服务器【$1】执行【$get_timespn_cmd】返回为空"
				continue
			fi
			# 根据偏移的秒数计算出实际日期时间
			local actual_time=$(date -d "@${timespn_cmd_ret}" '+%Y-%m-%d %H:%M:%S')
			local last_actual_time="$actual_time"  #最后一条数据的时间
			# 输出实际时间戳和剩余部分
			last_cached_string=$(echo "$line" | sed "s/^\[\s*\($timestamp\)\]/[\1]/" | sed "s/$timestamp/$actual_time/g")
			cached_string+="${last_cached_string}\n"
		done
		#转换后的时间日志
		if [ -n "${cached_string}" ];then
			RecordCommndInfo "$1" "${get_dmesg_cmd}转换后的日志："  "${cached_string}"
		fi
		#防止最后一次系统日志的时间戳转换失败
		if [ -z "${last_actual_time}" ];then
			local log_message=$(echo -e "Dmesg系统日志存在异常信息,总条数:${num}，转换日志时间失败")
			ResultTxtOutput "ERROR" "${log_message}"
			return
		fi
		
		#增加判断，判断最后一次时间增加不同等级的日志提示
		# 获取当前时间
		local current_time=$(date '+%Y-%m-%d %H:%M:%S')

		# 计算时间差（秒数）
		local actual_time_seconds=$(date -d "${last_actual_time}" +%s)
		local current_time_seconds=$(date -d "${current_time}" +%s)

		# 计算秒数差值
		local time_difference_seconds=$((current_time_seconds - actual_time_seconds))

		# 处理负数情况
		if [ $time_difference_seconds -lt 0 ]; then
			time_difference_seconds=$(( -time_difference_seconds ))  # 将负数转换为正数
		fi

		# 将秒数差值转换为天数
		time_difference_days=$((time_difference_seconds / 86400))
		if [ "$engine_name" == "GPU" ];then
			LOG 1 "${engine_name}:${1}:服务器计算的时间间隔为【$time_difference_days】day"
		else
			LOG 1 "${engine_name}:${1}:${index}号芯片:服务器计算的时间间隔为【$time_difference_days】day"
		fi
		
		# 日志
		local log_message=""
		# 检查时间差是否大于7天
		if [ "$engine_name" == "GPU" ];then
			log_message=$(echo -e "${engine_name}:${1}:Dmesg系统日志存在异常信息,总条数:${num}，最后一次异常时间:【${last_actual_time}】")
		else
			log_message=$(echo -e "${engine_name}:${1}:${index}号芯片:Dmesg系统日志存在异常信息,总条数:${num}，最后一次异常时间:【${last_actual_time}】")
		fi
		# 检查时间差
		if [ $time_difference_days -lt 7 ]; then  #小于7天
			ResultTxtOutput "ERROR" "${log_message}"
		elif [ $time_difference_days -ge 7 ] && [ $time_difference_days -le 30 ]; then
			ResultTxtOutput "WARNING" "${log_message}"
		elif [ $time_difference_days -gt 30 ]; then
			ResultTxtOutput "INFO" "${log_message}"
		fi
	done
}


### 获取Dmesg信息
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Dmesg_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="dmesg | grep -E -i \"iae\" |grep -E -i \"kill|segfault|error|fail|warn\" "
        ;;
	"AX")
		command=" ssh 192.168.254.$ipLastIndex dmesg | grep -E -i \"iae\" |grep -E -i \"kill|segfault|error|fail|warn\" "
	;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex dmesg | grep -E -i \"iae\" |grep -E -i \"kill|segfault|error|fail|warn\"  "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} dmesg | grep -E -i \"iae\" |grep -E -i \"kill|segfault|error|fail|warn\"  "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex dmesg | grep -E -i \"iae\" |grep -E -i \"kill|segfault|error|fail|warn\" "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的新类型：$1"
	    ;;
   esac
   echo ${command}
 }
 
 
 
### 获取Dmesg的偏移时间
### $1 板卡名称
### $2 芯片编号
### $3 板卡IP
### $4 dmesg的时间戳
Get_Dmesg_TimeSpan_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command=" echo \"\$(date +%s) - \$(cat /proc/uptime | cut -f 1 -d' ') + $4\" | awk '{sum=\$1-\$3+$4;; printf \"%.5f\", sum}' "
        ;;
    "AX")
        command=" ssh 192.168.254.$ipLastIndex  echo \"\$(date +%s) - \$(cat /proc/uptime | cut -f 1 -d' ') + $4\" |awk '{sum=\$1-\$3+$4;; printf \"%.5f\", sum}' "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex  echo \"\$(date +%s) - \$(cat /proc/uptime | cut -f 1 -d' ') + $4\" |awk '{ sum=\$1-\$3+$4;; printf \"%.5f\", sum}' "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} echo \"\$(date +%s) - \$(cat /proc/uptime | cut -f 1 -d' ') + $4\" |awk '{ sum=\$1-\$3+$4;; printf \"%.5f\", sum}' "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex echo \"\$(date +%s) - \$(cat /proc/uptime | cut -f 1 -d' ') + $4\" |awk '{ sum=\$1-\$3+$4;; printf \"%.5f\", sum}' "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的新类型：$1"
	    ;;
   esac
   echo ${command}
 }
 
 