#!/bin/bash

#使用vmstat命令获取wa值，也就是I/O等待的CPU的百分比。
IO_WA_THRESHOLD=10 #wa值的阈值

### IAM的IO检查
Check_IAM_IO()
{
    local cmd="vmstat 1 2"
    local cmd_ret=$(eval "$cmd")
    RecordCommndInfo "iam" "${cmd}" "${cmd_ret}"
	if [ -z "${cmd_ret}" ]; then
		ResultTxtOutput "WARNING" "【IAM】服务器执行${cmd}为空，巡检磁盘IO错误"
		return 1
    fi
	# 提取第一次输出的 wa 值
	local wa_value1=$(echo "${cmd_ret}" | awk 'NR==3 {print $16}')

	# 提取第二次输出的 wa 值
	local wa_value2=$(echo "${cmd_ret}" | awk 'NR==4 {print $16}')

	# 检查两次 wa 值是否都大于阈值
	compare_value1=$(echo "${wa_value1} > ${IO_WA_THRESHOLD}" | bc -l)
	compare_value2=$(echo "${wa_value2} > ${IO_WA_THRESHOLD}" | bc -l)
	LOG 1 "IAM服务器2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，与最低阈值【${IO_WA_THRESHOLD}】的比较结果为分别为【${compare_value1}】【${compare_value2}】"
	if [ "$compare_value1" -eq 1 ] && [ "$compare_value2" -eq 1 ]; then
		ResultTxtOutput "ERROR" "IAM服务器2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，都大于最低阈值【${IO_WA_THRESHOLD}】"
	else
		if [ "$compare_value1" -eq 1 ]; then
			ResultTxtOutput "WARNING" "IAM服务器2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，第1s的IO占用高于最低阈值【${IO_WA_THRESHOLD}】"
		elif [ "$compare_value2" -eq 1 ]; then
			ResultTxtOutput "WARNING" "IAM服务器2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，第2s的IO占用高于最低阈值【${IO_WA_THRESHOLD}】"
		else
			ResultTxtOutput "INFO" "IAM服务器2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，低于最低阈值【${IO_WA_THRESHOLD}】"
		fi
	fi
}

### BW的IO检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_BW_IO()
{
	local cmd="vmstat 1 2"
	local cmd_ret=$(SshCmd $1 $2 $3 $4 "${cmd}" 2>/dev/null)
    RecordCommndInfo "$1" "${cmd}" "${cmd_ret}"
	if [ -z "${cmd_ret}" ]; then
        ResultTxtOutput "WARNING" "【$1】的BW服务器执行${cmd}为空，巡检磁盘IO错误"
        return 1
    fi
	
	# 提取第一次输出的 wa 值
	local wa_value1=$(echo "${cmd_ret}" | awk 'NR==3 {print $16}')

	# 提取第二次输出的 wa 值
	local wa_value2=$(echo "${cmd_ret}" | awk 'NR==4 {print $16}')

	# 检查两次 wa 值是否都大于阈值
	compare_value1=$(echo "${wa_value1} > ${IO_WA_THRESHOLD}" | bc -l)
	compare_value2=$(echo "${wa_value2} > ${IO_WA_THRESHOLD}" | bc -l)
	LOG 1 "【$1】的BW服务器2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，与最低阈值【${IO_WA_THRESHOLD}】的比较结果为分别为【${compare_value1}】【${compare_value2}】"
	if [ "$compare_value1" -eq 1 ] && [ "$compare_value2" -eq 1 ]; then
		ResultTxtOutput "ERROR" "BW服务器【$1】2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，都大于最低阈值【${IO_WA_THRESHOLD}】"
	else
		
		if [ "$compare_value1" -eq 1 ]; then
			ResultTxtOutput "WARNING" "BW服务器【$1】2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，第1s的IO占用高于最低阈值【${IO_WA_THRESHOLD}】"
		elif [ "$compare_value2" -eq 1 ]; then
			ResultTxtOutput "WARNING" "BW服务器【$1】2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，第2s的IO占用高于最低阈值【${IO_WA_THRESHOLD}】"
		else
			ResultTxtOutput "INFO" "BW服务器【$1】2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，低于最低阈值【${IO_WA_THRESHOLD}】"
		fi
	fi
}



### IAE的IO检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_IAE_IO()
{
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)

	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	
	# GPU只需要检查BW即可
	if [ "$engine_name" == "GPU" ];then
		return 0
	fi
		
	local index=0
	for ((; index < chip_num; index++))
	do
		
		local command=$(Get_IO_Command "$engine_name" "$index" "$1")
		local cmd_ret=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
		RecordCommndInfo "$1" "${command}" "${cmd_ret}"
		if [ -z "${cmd_ret}" ]; then
			ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片:${command}命令返回为空"
			continue
		fi
		
		#检查IAE的IO使用率
		# 提取第一次输出的 wa 值
		local wa_value1=$(echo "${cmd_ret}" | awk 'NR==3 {print $16}')

		# 提取第二次输出的 wa 值
		local wa_value2=$(echo "${cmd_ret}" | awk 'NR==4 {print $16}')

		# 检查两次 wa 值是否都大于阈值
		compare_value1=$(echo "${wa_value1} > ${IO_WA_THRESHOLD}" | bc -l)
		compare_value2=$(echo "${wa_value2} > ${IO_WA_THRESHOLD}" | bc -l)
		LOG 1 "${engine_name}:${1}:${index}号芯片:2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，与最低阈值【${IO_WA_THRESHOLD}】的比较结果为分别为【${compare_value1}】【${compare_value2}】"
		if [ "$compare_value1" -eq 1 ] && [ "$compare_value2" -eq 1 ]; then
			ResultTxtOutput "ERROR" "${engine_name}:${1}:${index}号芯片,2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，都大于最低阈值【${IO_WA_THRESHOLD}】"
		else
			if [ "$compare_value1" -eq 1 ]; then
				ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片,2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，第1s的IO占用高于最低阈值【${IO_WA_THRESHOLD}】"
			elif [ "$compare_value2" -eq 1 ]; then
				ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片,2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，第2s的IO占用高于最低阈值【${IO_WA_THRESHOLD}】"
			else
				ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片,2s内的IO占用分别为【${wa_value1}】和【${wa_value2}】，低于最低阈值【${IO_WA_THRESHOLD}】"
			fi
		fi
	done
}


### 获取IO使用率查询命令
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_IO_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "AX")
        command=" ssh 192.168.254.$ipLastIndex \"vmstat 1 2\" "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex \"vmstat 1 2\" "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} \"vmstat 1 2\" "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex \"vmstat 1 2\" "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的新类型：$1"
	    ;;
   esac
   echo ${command}
 }
 
 