#!/bin/bash
#校验日志等级是否开启debug

### IAM的日志等级检查
Check_IAM_Log_Level()
{
	# 提取日志等级
   	local cmd="cat /opt/wise/ia12k/etc/ias.conf"
	local cmd_ret=$(eval "$cmd" 2>/dev/null)
    RecordCommndInfo "iam" "${cmd}" "${cmd_ret}"
	if [ -z "${cmd_ret}" ]; then
        ResultTxtOutput "WARNING" "IAM获取日志等级内容为空"
        return 1
    fi
	local log_level=$(echo "${cmd_ret}" | awk 'BEGIN {FS="="} {print $2}' 2>/dev/null)
	if [ -z "${log_level}" ]; then
        ResultTxtOutput "WARNING" "IAM获取日志等级为空"
        return 1
    fi
	# 判断日志等级
	if [ "${log_level}" -gt 0 ]; then
		ResultTxtOutput "WARNING" "IAM当前的日志等级为【${log_level}】，已开启debug日志"
	else
		ResultTxtOutput "INFO" "IAM当前的日志等级为【${log_level}】，未开启debug日志"
	fi
}

### BW的日志等级检查
### 58P03之后新增/opt/wise_proxy/ia12k/etc/bw_log.conf控制
### $1 服务器IP
### $2 /opt/wise_proxy/ia12k/etc/bw_log.conf的文件内容
Check_BW_Log_Level_NEW()
{
	local cmd_ret="$2"
	# 使用 awk 提取每个日志的控制项
	local output=$(echo "$cmd_ret" | awk '
	BEGIN { debug_found = 0; }
	{
		if ($0 ~ /^--\s+/) {
			print "ProcessName:" $2;
		}
		if ($1 ~ /DEBUG/) {
			debug_found = 1;
		}
		if (debug_found == 1 && $1 ~ /ENABLED/) {
			print "ENABLED:" $3;
		}
		if ($1 ~ /FATAL/) {
			debug_found = 0;
		}
	}
	' 2>/dev/null)

	#遍历一下确定是否存在
	local process_name=""
	# 解析输入
	echo "${output}" | while read -r line; do
		local key_name=$(echo "$line" | awk 'BEGIN {FS=":"} {print $1}' 2>/dev/null)
		local value=$(echo "$line" | awk 'BEGIN {FS=":"} {print $2}' | sed 's/[[:space:]]*$//'  2>/dev/null)
		if [ "${key_name}" == "ProcessName" ]; then
			process_name="${value}"
			LOG 1 "process_name: ${process_name}"
		elif [ "${key_name}" == "ENABLED" ]; then
			if [ "${value}" == "true" ]; then
				ResultTxtOutput "WARNING" "【$1】BW服务器进程$process_name已开启debug日志"
			else
				ResultTxtOutput "INFO" "【$1】BW服务器进程$process_name未开启debug日志"
			fi
		fi
	done
}


### BW的日志等级检查
### 58P03之前使用/opt/wise_proxy/ia12k/etc/iae.conf   58P03之后新增/opt/wise_proxy/ia12k/etc/bw_log.conf控制
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_BW_Log_Level()
{
	# 定义58P03特有的文件
	local cmd="cat /opt/wise_proxy/ia12k/etc/bw_log.conf"
	local cmd_ret=` SshCmd $1 $2 $3 $4 "$cmd" 2>/dev/null `
	RecordCommndInfo "$1" "${cmd}" "${cmd_ret}"
	if [ -n "${cmd_ret}" ]; then  #58P03之后的版本
		LOG 1 "存在/opt/wise_proxy/ia12k/etc/bw_log.conf文件，按照新版本进行解析"
		Check_BW_Log_Level_NEW "$1" "${cmd_ret}"
	else #58P03之前的版本
		LOG 1 "不存在/opt/wise_proxy/ia12k/etc/bw_log.conf文件，按照旧版本进行解析"
		# 提取日志等级
		local cmd="cat /opt/wise_proxy/ia12k/etc/iae.conf"
		local cmd_ret=$(SshCmd "$1" "$2" "$3" "$4" "${cmd}" 2>/dev/null)
		RecordCommndInfo "$1" "${cmd}" "${cmd_ret}"
		if [ -z "${cmd_ret}" ]; then
			ResultTxtOutput "WARNING" "【$1】获取BW日志等级内容为空"
			return 1
		fi
		local log_level=$(echo "${cmd_ret}" | awk 'BEGIN {FS="="} {print $2}' 2>/dev/null)
		if [ -z "${log_level}" ]; then
			ResultTxtOutput "WARNING" "【$1】获取BW日志等级为空"
			return 1
		fi
		# 判断日志等级
		if [ "${log_level}" -gt 0 ]; then
			ResultTxtOutput "WARNING" "【$1】BW当前的日志等级为【${log_level}】，已开启debug日志"
		else
			ResultTxtOutput "INFO" "【$1】BW当前的日志等级为【${log_level}】，未开启debug日志"
		fi
	fi
}


### IAE的日志等级检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_IAE_Log_Level()
{
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)

	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
		
	local index=0
	for ((; index < chip_num; index++))
	do
		#获取日志等级
		local command=$(Get_Log_Level_Command "$engine_name" "$index" "$1")
		local cmd_ret=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
		RecordCommndInfo "$1" "${command}" "${cmd_ret}"
		if [ -z "${cmd_ret}" ]; then
			ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片:${command}命令返回为空"
			continue
		fi
		local log_level=$(echo "${cmd_ret}" | awk 'BEGIN {FS="="} {print $2}' 2>/dev/null)
		if [ -z "${log_level}" ]; then
			ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片:获取日志等级为空"
			return 1
		fi
		# 判断日志等级
		if [ "${log_level}" -gt 0 ]; then
			ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片:当前的日志等级为【${log_level}】，已开启debug日志"
		else
			ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片:当前的日志等级为【${log_level}】，未开启debug日志"
		fi
	done
}


### 日志等级获取
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Log_Level_Command()
{
    local command=""
    case $1 in
	"GPU")
        command="cat /opt/wise_iae/ia12k/data$2/etc/iae.conf"
        ;;
    "AX")
        command="cat /opt/wise_iae/ia12k/data$2/etc/iae.conf"
        ;;
    "寒武纪")
        command="cat /opt/wise_iae/ia12k/data$2/etc/iae.conf"
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} \"cat /opt/wise/ia12k/etc/iae.conf\" "
        ;;
    "昆仑3")
        command="cat /opt/wise_iae/ia12k/data$2/etc/iae.conf"
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的新类型：$1"
	    ;;
   esac
   echo ${command}
 }
 