#!/bin/bash
memory_threshold=85
#获取mem的前三的进程和内存容量
top3_mem_cmd="awk '
BEGIN {
    name = \"\";
    mem = \"\";
    found = 0;
}
{
    if (\$1 == \"Name:\") {
        name = \$2;
    }
    if (\$1 == \"VmRSS:\") {
        mem = \$2;
        found = 1;
    }
    if (found == 1) {
        if(name != \"\") {
            print mem \" \" name;
            found = 0;
        }
    }
}
' /proc/[0-9]*/status | sort -nr | head -n 3"
get_top3_mem_cmd=$(echo "${top3_mem_cmd}"| sed 's/^[ \t]*//' | tr -d '\n') #将上面的字符串的行前空白字符和行尾的换行符删除
get_top3_mem_iae_cmd="awk 'BEGIN {name = \\\"\\\"; mem = \\\"\\\"; found = 0;} {if (\\\$1 == \\\"Name:\\\") {name = \\\$2;} if (\\\$1 == \\\"VmRSS:\\\") {mem = \\\$2; found = 1;} if (found == 1) {if (name != \\\"\\\") {print mem \\\" \\\" name; found = 0;}}}' /proc/[0-9]*/status | sort -nr | head -n 3" #包含转义字符

### 内存使用率检查
### $1 free 显示的内存信息
Check_Free()
{
	# 使用 awk 提取 Mem 行的总量和使用量
    local total=$(echo "$1" | grep "Mem" | awk '{print $2}')
    local used=$(echo "$1" | grep "Mem" | awk '{print $3}')
	
    # 检查 total 和 used 是否为数字
    if ! [[ "$total" =~ ^[0-9]+$ ]] || ! [[ "$used" =~ ^[0-9]+$ ]]; then
        ResultTxtOutput "WARNING" "获取到的内存使用情况或者内存总容量有异常"
        return 1
    fi

	# 计算使用率
	local use_rate=$(echo "scale=2; $used / $total * 100" | bc -l)
    echo $use_rate
}


### 获取内存使用率高的前三个进程名称
### $1 TOP 信息 
#示例： 内存大小kb 进程名称
#1005168 iam
#907004 java
#134888 java
get_top_mem_process()
{
	if [ -z "$1" ]; then
		echo "获取为空"
		return 1
	fi
	# 获取 top 命令的输出
    local top_output="$1"
	#获取cpu使用率高的三个进程
	local mem_line=$(echo "$top_output"| awk 'BEGIN{ORS="";FS=" ";OFS=""}{if (NR>1) printf "|"; printf "%s（%sKB）", $2, $1} END{print ""}' )
	# 返回总的 CPU 使用率
    echo "$mem_line"
}

### IAM内存检查
Check_IAM_Mem(){
    LOG 1 "开始检查IAM内存使用情况"
	#检查IAM内存使用率
	local cmd="free 2>/dev/null"
	local check_mem_ret=$(free 2>/dev/null)
	RecordCommndInfo "iam" "${cmd}" "${check_mem_ret}"
	if [ -z "${check_mem_ret}" ]; then
		ResultTxtOutput "WARNING" "${cmd}执行结果为空" 
		return 1
	fi
	local use_rate=$(Check_Free "$check_mem_ret")	# 检查内存，使用超过95%的分区需要给出提示
	# 检查变量是否为空
	if [ -z "$use_rate" ]; then
		ResultTxtOutput "WARNING" "Check_Free计算内存失败"
		return 1
	fi
	if (( $(echo "$use_rate > $memory_threshold" | bc -l) )); then
		#获取top的内存信息
		cmd="${get_top3_mem_cmd}"
		top_mem_ret=$(2>/dev/null eval "$cmd")
		RecordCommndInfo "iam" "${cmd}" "${top_mem_ret}"
		if [ -z "$top_mem_ret" ]; then
			ResultTxtOutput "WARNING" "【IAM】执行$cmd返回为空，无法获取内存前三的进程"
			return 1
		fi
		#获取内存使用率Top3进程信息
		local top_process=$(get_top_mem_process "$top_mem_ret")
		ResultTxtOutput "ERROR" "IAM的内存使用率: ${use_rate}%,top3进程内存信息：${top_process}"
	else
		ResultTxtOutput "INFO" "IAM的内存使用率: ${use_rate}%"
	fi
}

#########获取BW运行状态#######
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码 
Check_BW_mem()
{
	# 检查bw 内存使用率
	local cmd="free 2>/dev/null"
	local check_mem_ret=$(2>/dev/null SshCmd $1 $2 $3 $4 "$cmd")
	RecordCommndInfo "$1" "${cmd}" "${check_mem_ret}"
	if [ -z "$check_mem_ret" ]; then
		ResultTxtOutput "WARNING" "【$1】BW服务器${cmd}执行结果为空" 
		return 1
	fi
	local use_rate=$(Check_Free "$check_mem_ret")	# 检查内存，使用超过95%的分区需要给出提示
	# 检查变量是否为空
	if [ -z "$use_rate" ]; then
		ResultTxtOutput "WARNING" "Check_Free计算内存失败"
		return 1
	fi
	if (( $(echo "$use_rate > $memory_threshold" | bc -l) )); then
		#获取top的内存信息
		cmd="${get_top3_mem_cmd}"
		top_mem_ret=$(SshCmd $1 $2 $3 $4 "$cmd" 2>/dev/null)
		RecordCommndInfo "$1" "${cmd}" "${top_mem_ret}"
		if [ -z "$top_mem_ret" ]; then
			ResultTxtOutput "WARNING" "【$1】【BW】执行$cmd返回为空，无法获取内存前三的进程"
		fi
		#获取内存使用率Top3进程信息
		local top_process=$(get_top_mem_process "$top_mem_ret")
		ResultTxtOutput "ERROR" "【$1】【BW】的内存使用率: ${use_rate}%,top3进程内存信息：${top_process}"
	else
		ResultTxtOutput "INFO" "【$1】【BW】的内存使用率: ${use_rate}%"
	fi
}


###IAE内存检查
###$1 板卡ip
###$2 板卡ssh端口
###$3 板卡ssh用户名
###$4 板卡ssh密码 
Check_IAE_Mem()
{
    LOG 1 "开始检查IAE【$1】内存使用情况"
	
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)

	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	local engine_name=$(Get_Engine_Name $1)

	local index=0
	for ((; index < chip_num; index++))
	do
		# GPU只需要检查一次
		if [ "$engine_name" == "GPU" ]&& [ "$index" -ne 0 ];then
		  continue
		fi
		local command=$(Get_CheckMemory_Command "$engine_name" "$index" "$1")
		local iae_info=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
		RecordCommndInfo "$1" "${command}" "${iae_info}"
		if [ -z "$iae_info" ]; then
			ResultTxtOutput "WARNING" "Check_IAE_Mem：${engine_name}:${1}:${index}号芯片,${command}命令返回为空"
			continue
		fi
		local use_rate=$(Check_Free "$iae_info")	# 检查内存，使用超过95%的分区需要给出提示
		# 检查变量是否为空
		if [ -z "$use_rate" ]; then
			ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片Check_Free计算内存失败"
			continue
		fi
		if (( $(echo "$use_rate > $memory_threshold" | bc -l) )); then
			local top_mem_ret=""
			#获取top的内存信息
			if [ "$engine_name" == "GPU" ];then
				top_mem_ret=$(SshCmd $1 $2 $3 $4 "${get_top3_mem_cmd}" 2>/dev/null)
				RecordCommndInfo "$1" "$command" "${top_mem_ret}"
			else
				command=$(Get_Top_Memory_Command "$engine_name" "$index" "$1")
				top_mem_ret=$(SshCmd $1 $2 $3 $4 "${command} \"$get_top3_mem_iae_cmd\"" 2>/dev/null)
				RecordCommndInfo "$1" "${command} \"$get_top3_mem_iae_cmd\"" "${top_mem_ret}"
			fi
			
			if [ -z "$top_mem_ret" ]; then
				ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片:执行${command}返回为空，无法获取内存前三的进程"
			fi
			#获取内存使用率Top3进程信息
			local top_process=$(get_top_mem_process "$top_mem_ret")
			
			if [ "$engine_name" == "GPU" ];then
				ResultTxtOutput "ERROR" "${engine_name}:${1}:内存使用率过高，当前内存使用率:${use_rate}%,top3进程内存信息：${top_process}"
			else
				ResultTxtOutput "ERROR" "${engine_name}:${1}:${index}号芯片, 内存使用率过高，当前内存使用率:${use_rate}%,top3进程内存信息：${top_process}"
			fi
			
		else
			if [ "$engine_name" == "GPU" ];then
				ResultTxtOutput "INFO" "${engine_name}:${1}, 内存使用率正常，当前内存使用率:${use_rate}%"
			else
				ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片, 内存使用率正常，当前内存使用率:${use_rate}%"
			fi
		fi
	done
}

### 获取内存使用情况的查询命令
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_CheckMemory_Command(){
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="free"
        ;;
    "AX")
        command=" ssh 192.168.254.$ipLastIndex free"
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex free"
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} free"
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex free"
        ;;
     *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
    esac
    echo "$command"
 }
 
 ### 获取CPU使用率查询命令
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Top_Memory_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "AX")
        command=" ssh 192.168.254.$ipLastIndex "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex  "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]}"
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex"
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的新类型：$1"
	    ;;
   esac
   echo "${command}"
   
 }
