#!/bin/bash

### 网络状态检查 - 主控端口映射 - 网络连通性
### net映射检查
###$1 板卡ip
###$2 板卡ssh端口
###$3 板卡ssh用户名
###$4 板卡ssh密码 
Check_IAE_NetTable()
{
    LOG 1 "开始检查网络IAE【$1】网络连通性"

	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	local engine_name=$(Get_Engine_Name $1)
	
	# GPU不做主控映射检查
	if [ "$engine_name" == "GPU" ];then
	    LOG 1 "GPU不做主控监听检查"
		return 1
	fi
    local cmd="iptables -t nat -L -n | grep -e 5296 -e 10245"
	local net_info=$(SshCmd "$1" "$2" "$3" "$4" "iptables -t nat -L -n | grep -e 5296 -e 10245" 2>/dev/null)
	RecordCommndInfo "$1" "${cmd}" "${net_info}"
	# 判断变量是否为空
	if [ -z "$net_info" ]; then
		ResultTxtOutput "WARNING" "IP[$1]执行${cmd}返回为空，Check_IAE_NetTable检查失败"
		return 1
	fi
	# ssh连接正常但目标服务器权限不允许 直接跳过
	local test=$(echo "$net_info" | grep -e 5296 -e 10245)
	if [ $? -ne 0 ];then
	    ResultTxtOutput "WARNING" "IAE【$1】网络检查失败"
		return 1
	fi

	local index=0
	for ((; index < chip_num; index++))
	do
		local ip_number=$((2 * index + 1))
		# 检查10245映射
		local nat_info=$(echo "$net_info" | grep ".${ip_number}:10245")
		if [ $? -ne 0 ] && [ -z "$nat_info" ];then  #不为空且执行无错误
			ResultTxtOutput "ERROR" "${engine_name}:${1}:${index}号芯片, 10245端口未映射"
		else
			ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片, 10245端口映射成功"
		fi


		# 检查5296映射
		nat_info=$(echo "$net_info" | grep ".${ip_number}:5296")
		if [ $? -ne 0 ]&& [ -z "$nat_info" ];then
			ResultTxtOutput ""ERROR "${engine_name}:${1}:${index}号芯片, 5296端口未映射"
		else
			ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片, 5296端口映射成功"
		fi
	done
}

###$1 板卡ip
###$2 板卡ssh端口
###$3 板卡ssh用户名
###$4 板卡ssh密码 
Check_IAE_BoardNetTable()
{
    LOG 1 "开始检查网络IAE【$1】端口映射"
	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	local engine_name=$(Get_Engine_Name $1)
	# GPU不做映射检查
	if [ "$engine_name" == "GPU" ];then
	    LOG 1 "GPU不做主控映射检查"
		return 1
	fi
	#获取ortMap.conf的的端口内容
	local port_map=$(SshCmd "$1" "$2" "$3" "$4" "cat /opt/wise_proxy/ia12k/etc/PortMap.conf" 2>/dev/null)
	RecordCommndInfo "$1" "cat /opt/wise_proxy/ia12k/etc/PortMap.conf" "${port_map}"
	# 判断变量是否为空
	if [ -z "$port_map" ]; then
		ResultTxtOutput "WARNING" "IP[$1]执行cat /opt/wise_proxy/ia12k/etc/PortMap.conf 返回为空，Check_IAE_BoardNetTable检查失败"
		return 1
	fi
	local arr_port_map=$(echo "${port_map}"|awk 'BEGIN {FS=":"} {print $2}')
	#遍历端口，查看是否存在映射关系
	IFS=$'\n' read -d '' -ra arr_port_info <<< "$arr_port_map"
	local index=0
    for arr_port_info_line in "${arr_port_info[@]}"
	do
		local cmd="iptables -t nat -L -n | grep -e \"${arr_port_info_line}\" "
		local net_info=$(SshCmd "$1" "$2" "$3" "$4" "${cmd}" 2>/dev/null)
		RecordCommndInfo "$1" "${cmd}" "${net_info}"
		if [ -z "${net_info}" ];then
			ResultTxtOutput "ERROR" "${engine_name}:${1}:${index}号芯片, ${arr_port_info_line}端口映射为空"
			continue
		fi
		#DNAT       tcp  --  0.0.0.0/0            206.206.124.247      tcp dpt:10246 to:192.168.255.1:10245
		#dpt:10246分割出10246
		local current_port=$(echo "${net_info}" |head -n 1 | awk '{print $7}'| awk 'BEGIN {FS=":"} {print $2}')
		if [ "${current_port}" !=  "${arr_port_info_line}" ];then
			ResultTxtOutput "ERROR" "${engine_name}:${1}:${index}号芯片, 端口映射错误，应转发端口${arr_port_info_line}，实际转发端口${current_port}，不匹配"
		fi
		#取出to:192.168.255.1:10245,获取192.168.255.1是否正确
		local current_ip=$(echo "${net_info}"|head -n 1|awk '{print $8}'| awk 'BEGIN {FS=":"} {print $2":"$3 }')
		local compare_ip=$(Get_chip_Ip "$engine_name" "$index" "$1")
		
		if [ "${current_ip}" !=  "${compare_ip}" ];then
			ResultTxtOutput "ERROR" "${engine_name}:${1}:${index}号芯片, 端口【${current_port}】映射错误，应该映射${compare_ip}，实际映射${current_ip}"
		fi
		ResultTxtOutput "INFO" "${engine_name}:${1}:${index}号芯片, 端口【${current_port}】映射正确，应该映射${compare_ip}，实际映射${current_ip}"
		index=$((index + 1))
	done
	
	LOG 1 "完成检查网络IAE【$1】端口映射"
}

 
### 获取芯片IP
### $1板卡名称
### $2芯片编号
### $3 板卡ip
Get_chip_Ip()
{
    local ip=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "AX")
        ip="192.168.254.$ipLastIndex:10245"
        ;;
    "寒武纪")
        ip="192.168.255.$ipLastIndex:10245"
        ;;
    "昆仑2")
        ip="${BOARD_CHIP_IP[$3-$2]}:10245"
        ;;
    "昆仑3")
        ip="192.168.255.$ipLastIndex:10245"
        ;;
     *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
    esac
    echo "${ip}"
 }



