#!/bin/bash
##校验SSD寿命。默认只支持昆仑3和昆仑5，后续需要扩展
ssd_write_threshold=90
### 获取系统盘符
### $1 lsblk信息输出
get_system_ssd() 
{
    local input="$1"
    local output=$(echo "$input" | grep "/$")
    local ssd_info=$(echo "$output" | grep -o "sd[a-z]*")
    
    if [ -z "$ssd_info" ]; then
        output=$(echo "$input" | grep "/boot$")
        ssd_info=$(echo "$output" | grep -o "sd[a-z]*")
    fi
    
    if [ -z "$ssd_info" ]; then
        LOG  1 "$input 获取lsblk信息失败!"
        return 1  # 返回错误状态码
    fi
    
    echo "$ssd_info"  # 返回字符串值
    return 0  # 返回成功状态码
}

### 分析SSD的写入量
### $1 smartctl的信息输出
### $2 日志的前缀信息
analyse_ssd() 
{
	output="$1"
    device_model=$(echo "$output" | grep "Device Model:" |awk ' BEGIN {FS=":"} {print $2} '| sed 's/^[ \t]*//;s/[ \t]*$//')
    if [ -z "${device_model}" ]; then
		ResultTxtOutput "WARNING" "$2：SSD的设备名称获取为空，SSD磁盘分析异常!"
        return 1
    fi

    total_lbas_written=$(echo "$output" | grep -E "Total_LBAs_Written|Host_Writes_32MiB" | awk '{print $10}')
    if [ -z "$total_lbas_written" ]; then
        total_lbas_written=0
    fi
	
	local ssd_find="true"
    case "${device_model}" in
        "ADATA_IM2S3334-128GD")
            written_gib=$(echo "$total_lbas_written * 65536 * 512 / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * 65536 * 512" | bc)
            written_total=160
            ;;
        "TS128GMTS400")
            written_gib=$(echo "$total_lbas_written * 65536 * 512 / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * 65536 * 512" | bc)
            written_total=300
            ;;
        "ADATA_IM2S3134N-128GM")
            written_gib=$(echo "$total_lbas_written * 512 / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * 512" | bc)
            written_total=43
            ;;
		"ADATA_IMSS332-064GM"|"IMSS332-064GM")
            written_gib=$(echo "$total_lbas_written * 512 / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * 512" | bc)
            written_total=60
            ;;
		"TRANSCEND_TS64GMSA370"|"TS64GMSA370"|"TRANSCEND")
            written_gib=$(echo "$total_lbas_written * 65536 * 512 / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * 65536 * 512" | bc)
            written_total=90
            ;;
		"Cactus_CactusFlashCard"|"CactusFlashCard"|"Cactus")
            written_gib=$(echo "$total_lbas_written * 65536 * 512 / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * 65536 * 512" | bc)
            written_total=128
            ;;
        "M.2 (S42) 3ME4")
            written_gib=$(echo "$total_lbas_written * 65536 * 512 / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * 65536 * 512" | bc)
            written_total=208
            ;;
        "TS128GMTS410M"|"ADATA_IM2S31C4-128GCTB5"|"ADATA_IM2S31C4-256GCTB5"|"ADATA_IM2S31C4-256GD"|"MFG0625C00TB")
            ssd_flag=true
            if (( $(echo "$total_lbas_written >= 150000000" | bc -l) )); then
                written_unit=512
            else
                written_unit=34359738368
            fi
            written_gib=$(echo "$total_lbas_written * $written_unit / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * $written_unit" | bc)
            case "$device_model" in
                "TS128GMTS410M") written_total=360 ;;
                "ADATA_IM2S31C4-128GCTB5") written_total=270 ;;
                "ADATA_IM2S31C4-256GCTB5") written_total=540 ;;
                "ADATA_IM2S31C4-256GD") written_total=320 ;;
                "MFG0625C00TB") written_total=514 ;;
            esac
            ;;
        "TS128GMTS570T")
            written_gib=$(echo "$total_lbas_written * 65536 * 512 / 1000 / 1000 / 1000 / 1000" | bc -l)
            written_byte=$(echo "$total_lbas_written * 65536 * 512" | bc)
            written_total=259
            ;;
        *)
            ssd_flag=true
			ssd_find="false"
            ;;
    esac
	if [ "${ssd_find}" == "false" ];then
		ResultTxtOutput "WARNING" "$2：SSD设备信息为：${device_model}，不支持SSD磁盘写入分析!"
		return 1
	fi
    written_used=$(echo "scale=2; $written_gib / $written_total *100" | bc -l)
	total_written_byte=$(echo "$written_total * 1000 * 1000 * 1000 * 1000" | bc -l)
	if (($(echo "$written_used >= 100" | bc -l) )); then
		ResultTxtOutput "ERROR" "$2：SSD使用寿命已超过100%，建议更换SSD"
	elif (( $(echo "$written_used > $ssd_write_threshold" | bc -l) )); then
		ResultTxtOutput "WARNING" "$2：SSD总写入$total_written_byte,当前已使用$written_used%,超过阈值 $ssd_write_threshold%，SSD寿命已超过阈值${ssd_write_threshold}"
	else
		ResultTxtOutput "INFO" "$2：SSD总写入$total_written_byte,当前已使用$written_used%,未超过阈值 $ssd_write_threshold%"
	fi
}


#校验IAM的SSD寿命
Check_IAM_SSD()
{
    LOG 1 "开始检查SSD寿命信息"
	#获取ssd的盘符
	local lsblk_info=$(lsblk 2>/dev/null)
	if [ -z "${lsblk_info}" ]; then
        ResultTxtOutput "INFO" "IAM，lsblk命令执行为空：SSD寿命检查失败"
        return 1
    fi
	RecordCommndInfo "iam" "lsblk" "${lsblk_info}"
	local ssd_name=$(get_system_ssd "${lsblk_info}")
	if [ -z "${ssd_name}" ]; then
        ResultTxtOutput "INFO" "IAM 获取盘符为空：SSD寿命检查失败"
        return 1
    fi
	LOG 1 "IAM获取的盘符:${ssd_name}"
	#获取SSD检查信息
	local smartctl_info=$(smartctl -a /dev/$ssd_name 2>/dev/null)
	if [ -z "${smartctl_info}" ]; then
        ResultTxtOutput "INFO" "IAM smartctl执行为空，SSD寿命检查失败"
        return 1
    fi
	RecordCommndInfo "iam" "smartctl -a /dev/$ssd_name" "${smartctl_info}"
	#校验SSD的检测数据
	analyse_ssd "$smartctl_info" "【IAM】"
}

### net映射检查
###$1 板卡ip
###$2 板卡ssh端口
###$3 板卡ssh用户名
###$4 板卡ssh密码 
Check_IAE_SSD()
{
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)
	local lsblk_info=""
	local ssd_name=""
	if [ "$engine_name" == "昆仑2" ];then
		#当前仅支持BW检查，内部暂不实现
		#kunlun2中没有lsblk命令，当前采用df -h命令，获取/program对应的磁盘
		#获取ssd的盘符
		lsblk_info=$(SshCmd "$1" "$2" "$3" "$4" "df -h" 2>/dev/null)
		RecordCommndInfo "$1" "df -h" "${lsblk_info}"
		if [ -z "${lsblk_info}" ]; then
			ResultTxtOutput "WARNING" "【$engine_name】【$1】df -h命令执行为空，SSD寿命检查失败"
			return 1
		fi
		
		ssd_name=$(echo "$lsblk_info"| grep '/program' | cut -d' ' -f1 |cut -d'/' -f3|sed 's/[0-9]*$//')
		if [ -z "${ssd_name}" ]; then
			ResultTxtOutput "WARNING" "【$engine_name】【$1】获取盘符为空，SSD寿命检查失败"
			return 1
		fi
		LOG 1 "【$engine_name】【$1】获取的盘符:$ssd_name"
	else
		# GPU、爱芯、寒武纪、昆仑3只需要检查服务器的磁盘即可
		#获取ssd的盘符
		lsblk_info=$(SshCmd "$1" "$2" "$3" "$4" "lsblk" 2>/dev/null)
		RecordCommndInfo "$1" "lsblk" "${lsblk_info}"
		if [ -z "${lsblk_info}" ]; then
			ResultTxtOutput "WARNING" "【$engine_name】【$1】lsblk命令执行为空，SSD寿命检查失败"
			return 1
		fi
		
		#Result_Detail_output "$1" "lsblk" "$lsblk_info"
		ssd_name=$(get_system_ssd "$lsblk_info")
		if [ -z "${ssd_name}" ]; then
			ResultTxtOutput "WARNING" "【$engine_name】【$1】获取盘符为空，SSD寿命检查失败"
			return 1
		fi
		LOG 1 "【$engine_name】【$1】获取的盘符:$ssd_name"
	fi
	
	#获取SSD检查信息
	local smartctl_info=$(SshCmd "$1" "$2" "$3" "$4" "smartctl -a /dev/$ssd_name" 2>/dev/null)
	RecordCommndInfo "$1" "smartctl -a /dev/${ssd_name}" "${smartctl_info}"
	if [ -z "${smartctl_info}" ]; then
        ResultTxtOutput "WARNING" "【$engine_name】【$1】smartctl执行结果为空，SSD寿命检查失败"
        return 1
    fi
	#校验SSD的检测数据
	analyse_ssd "$smartctl_info" "【$engine_name】【$1】"
}




