#!/bin/bash

#时间同步校验
TIME_THRESHOLD=30 #时间同步的秒数

### IAM的时间同步校验检查
Check_IAM_Time()
{
	#涉及vm，不实现
	LOG 0 "IAM不进行时间同步校验"
}

### BW的IO检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_BW_Time()
{
	# 获取本地服务器的时间
	local local_time=$(date +%s)

	# 获取远程服务器的时间
	local remote_time=$(SshCmd $1 $2 $3 $4 "date +%s")
	RecordCommndInfo "$1" "date +%s" "${remote_time}"

	# 计算时间差
	local time_diff=$((remote_time - local_time)) #秒级
	LOG 1 "${engine_name}:${1}:${index}号芯片的时间戳为【$remote_time】和iam的时间戳【$local_time】的差值为【$time_diff】"
	# 计算绝对值
	local abs_time_diff=$(echo $time_diff | awk '{print ($1 < 0) ? -$1 : $1}')
	# 检查时间是否同步
	if [ $abs_time_diff -lt $TIME_THRESHOLD ]; then
		ResultTxtOutput "INFO" "BW服务器【$1】和IAM的服务器时间差【${abs_time_diff}】，小于${TIME_THRESHOLD}s"
	else
		ResultTxtOutput "ERROR" "BW服务器【$1】和IAM的服务器时间差【${abs_time_diff}】，大于${TIME_THRESHOLD}s"
	fi
}



### IAE的IO检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_IAE_Time()
{
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)

	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	
	# GPU只需要检查BW即可
	if [ "$engine_name" == "GPU" ];then
		return 0
	fi
		
	local index=0
	for ((; index < chip_num; index++))
	do
		#获取远方时间
		local command=$(Get_Time_Command "$engine_name" "$index" "$1")
		local cmd_ret=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
		RecordCommndInfo "$1" "${command}" "${cmd_ret}"
		if [ -z "${cmd_ret}" ]; then
			ResultTxtOutput "WARNING" "${engine_name}:$1:${index}号芯片:${command}命令返回为空"
			continue
		fi
		# 获取本地服务器的时间
		local local_time=$(date +%s)
		
		# 计算时间差
		local time_diff=$((cmd_ret - local_time)) #秒级
		LOG 1 "${engine_name}:${1}:${index}号芯片的时间戳为【$cmd_ret】和iam的时间戳【$local_time】的差值为【$time_diff】"
		# 计算绝对值
		local abs_time_diff=$(echo $time_diff | awk '{print ($1 < 0) ? -$1 : $1}')

		# 检查时间是否同步
		if [ $abs_time_diff -lt $TIME_THRESHOLD ]; then
			ResultTxtOutput "INFO" "${engine_name}:$1:${index}号芯片和IAM的服务器时间差【${abs_time_diff}】，小于${TIME_THRESHOLD}s"
		else
			ResultTxtOutput "ERROR" "${engine_name}:$1:${index}号芯片和IAM的服务器时间差【${abs_time_diff}】，大于${TIME_THRESHOLD}s"
		fi
	done
}


### 获取IO使用率查询命令
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Time_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "AX")
        command=" ssh 192.168.254.$ipLastIndex \"date +%s\" "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex \"date +%s\" "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} \"date +%s\" "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex \"date +%s\" "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的新类型：$1"
	    ;;
   esac
   echo ${command}
 }
 