#!/bin/bash

### iam僵尸进程检查
Check_IAM_Zombie_Processes()
{
	# 检查IAM上的僵尸进程
	local cmd="ps aux 2>/dev/null"
	local ps_iam_info=$(ps aux 2>/dev/null)
	RecordCommndInfo "iam" "${cmd}" "${ps_iam_info}"
    local ps_info=$(echo "$ps_iam_info" |grep -v -e "grep --color=auto" -e "VSZ" | grep -e "Z")
	LOG 1 "【IAM】的服务器僵尸进程：【$ps_info】"
	if [ -n "$ps_info" ];then #非空
		local row_num=$(echo "$ps_info"|wc -l)
		local ia_zombie_process=$(echo "$ps_info" | grep -e "ia" -e "IA")
		LOG 1 "【IAM】服务器业务过滤的僵尸进程：【$ia_zombie_process】"
		if [ -n "${ia_zombie_process}" ];then
			local ia_row_num=$(echo "$ia_zombie_process"|wc -l)
			ResultTxtOutput "ERROR" "【IAM】服务器存在僵尸进程总个数【$row_num】,业务相关僵尸进程总个数【$ia_row_num】"
		else
			ResultTxtOutput "WARNING" "【IAM】服务器存在僵尸进程总个数【$row_num】,业务相关僵尸进程总个数【0】"
		fi
		
	else
		ResultTxtOutput "INFO" "【IAM】服务器不存在僵尸进程"
    fi
}

### BW僵尸进程检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_BW_Zombie_Processes()
{
	# 检查bw上的僵尸进程
	local cmd="ps aux"
	local ps_bw_info=$(SshCmd $1 $2 $3 $4 "$cmd" 2>/dev/null)
	RecordCommndInfo "$1" "${cmd}" "${ps_bw_info}"
	if [ -z "$ps_bw_info" ]; then
		ResultTxtOutput "WARNING" "${cmd}执行结果为空" 
		return 1
	fi
    local ps_info=$(echo "$ps_bw_info" |grep -v -e "grep --color=auto" -e "VSZ" | grep -e "Z")
	LOG 1 "【$1】bw的服务器僵尸进程：【$ps_info】"
	if [ -n "$ps_info" ];then #非空
		local row_num=$(echo "$ps_info"|wc -l)
		local bw_zombie_process=$(echo "$ps_info"| grep -e "ia" -e "bw" -e "braswell")
		LOG 1 "【$1】bw的服务器业务过滤的僵尸进程：【$bw_zombie_process】"
		if [ -n "${bw_zombie_process}" ];then
			local bw_row_num=$(echo "$bw_zombie_process"|wc -l)
			ResultTxtOutput "ERROR" "【$1】bw的服务器存在僵尸进程总个数【$row_num】,业务相关僵尸进程总个数【$bw_row_num】"
		else
			ResultTxtOutput "WARNING" "【$1】bw的服务器存在僵尸进程总个数【$row_num】,业务相关僵尸进程总个数【0】"
		fi
		
	else
		ResultTxtOutput "INFO" "【$1】bw的不存在僵尸进程"
    fi
	
}

### 检查IAE是否存在僵尸进程
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_IAE_Zombie_Processes()
{
    LOG 1 "开始检查板卡主控服务器上的僵尸进程"

    #根据IP获取板卡名称
    local engine_name=$(Get_Engine_Name $1)

    # 获取板卡名称和芯片个数
    local chip_num=$(Get_Chip_Num $1)
    local engine_name=$(Get_Engine_Name $1)

    local index=0
    for ((; index < chip_num; index++))
    do
		# GPU只需要检查一次
		if [ "$engine_name" == "GPU" ]&& [ "$index" -ne 0 ];then
		  continue
		fi
		local command=$(Get_Command_For_Pid "$engine_name" "$index" "$1")
		local ps_iae_info=` SshCmd $1 $2 $3 $4 "$command" 2>/dev/null `
		RecordCommndInfo "$1" "${command}" "${ps_iae_info}"
		# 判断变量是否为空
		if [ -z "$ps_iae_info" ]; then
			ResultTxtOutput "WARNING" "IP[$1]执行${command}返回为空，Check_IAE_Zombie_Processes检查失败"
			continue
		fi
		local ps_info=$(echo "$ps_iae_info" |grep -v -e "grep --color=auto" -e "VSZ" | grep -e "Z")
		LOG 1 "${engine_name}:IP[$1]:${index}号芯片的僵尸进程：【$ps_info】"
		if [ -n "$ps_info" ];then #非空
			local row_num=$(echo "$ps_info"|wc -l)
			local ia_zombie_process=$(echo "$ps_info" | grep -e "ia" -e "IA")
			LOG 1 "${engine_name}:IP[$1]:${index}号芯片业务过滤的僵尸进程：【$ia_zombie_process】"
			if [ -n "${ia_zombie_process}" ];then
				local ia_row_num=$(echo "$ia_zombie_process"|wc -l)
				ResultTxtOutput "ERROR" "${engine_name}:IP[$1]:${index}号芯片, 存在僵尸进程总个数【$row_num】,业务相关僵尸进程总个数【$ia_row_num】"
			else
				ResultTxtOutput "WARNING" "${engine_name}:IP[$1]:${index}号芯片, 存在僵尸进程总个数【$row_num】,业务相关僵尸进程总个数【0】"
			fi
			
		else
			ResultTxtOutput "INFO" "${engine_name}:IP[$1]:${index}号芯片不存在僵尸进程"
		fi
    done
}


### 获取SSD寿命检查命令
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Command_For_Pid()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="ps aux"
        ;;
    "AX")
        command=" ssh 192.168.254.$ipLastIndex ps aux "
        ;;
    "寒武纪")
        command=" ssh 192.168.255.$ipLastIndex ps aux "
        ;;
    "昆仑2")
        command=" ssh ${BOARD_CHIP_IP[$3-$2]} ps aux "
        ;;
    "昆仑3")
        command=" ssh 192.168.255.$ipLastIndex ps aux "
        ;;
    *)
        ResultTxtOutput "WARNING" "不支持的芯片类型"
        ;;
    esac
   echo "$command"
 }
 
