﻿#资源分配校验巡检模块
import subprocess
import sys
import common
import time
import platform

# 定义业务类型及其子业务类型的字典结构
# "结构化业务": {
#     "结构化视频流": 0,
#     "结构化图片流": 0
# },
# "全结构化业务": {
#     "全结构化视频流": 0,
#     "全结构化图片流": 0,
#     "图片流全量解析": 0,
# },
# "人脸业务": {
#     "人脸视频流": 0,
#     "人脸图片流": 0,
#     "人脸特征清洗": 0,
#     "人脸图搜": 0
# },
# "地铁行为分析业务": {
#     "监管类": 0,
#     "人群分析类": 0,
#     "动作模型类": 0,
#     "人数统计类": 0,
#     "安全类": 0
# },
# "行为分析业务": {
#     "监管类": 0,
#     "人群分析类": 0,
#     "动作模型类": 0,
#     "人数统计类": 0,
#     "视频流": 0  # 合一版本，只有该类型
# },
# "未知类型":{
#     "未知模式": 0  未分配资源的芯片
# }
#
# 定义业务类型常量
BussinesType_Unknown_V3 = 0                    #未知
BussinesType_VAS_Video_V3 = 100           #结构化视频流
BussinesType_Vas_Pic_V3 = 103             #结构化图片流
BussinesType_FS_Video_V3 = 400            #人脸视频流
BussinesType_FS_Pic_V3 = 300              #人脸图片流
BussinesType_FS_FeatureRextract_V3 = 304  #人脸特征清洗
BussinesType_FS_PicSearch_V3 = 305        #人脸图搜
BussinesType_BA_Supervise_V3 = 200           #行为分析监管类型
BussinesType_BA_PersonAnalysis_V3 = 203     #行为分析人群分析
BussinesType_BA_Action_V3 = 202             #行为分析动作类型
BussinesType_BA_PersonNumStatic_V3 = 201    #行为分析人数统计
BussinesType_BA_Safety_V3 = 208            #行为分析安全
BussinesType_BA_CommonVideo_V5 = 210             #昆仑5、GPU、爱芯行为分析通用
BussinesType_AS_Video_V3 = 902              #全结构化视频流
BussinesType_AS_PicSingle_V3 = 903          #全结构化二次解析
BussinesType_AS_PicAll_V3 = 901             #全结构化全量解析

#定义tasktype枚举
TaskType_Unknown = 0 #未知类型
TaskType_vas = 1    #结构化业务
TaskType_fS = 2     #人脸业务
TaskType_ba = 3     #行为分析业务
TaskType_bas = 7    #地铁行为分析
TaskType_as = 9     #全结构化业务

#引擎类型
EngineType_k2 = 1 #昆仑2
EngineType_k3 = 2 #昆仑3
EngineType_gpu = 3 #GPU
EngineType_MLU = 4 #寒武纪
EngineType_aixin = 7 #爱芯


# 定义数值枚举（数值在前）
global_bussines_value_enums = {
    BussinesType_VAS_Video_V3: "结构化视频流",
    BussinesType_FS_Video_V3: "人脸视频流",
    BussinesType_Vas_Pic_V3: "结构化图片流",
    BussinesType_FS_Pic_V3: "人脸图片流",
    BussinesType_FS_FeatureRextract_V3: "人脸特征清洗",
    BussinesType_FS_PicSearch_V3: "人脸图搜",
    BussinesType_BA_Supervise_V3: "监管类",
    BussinesType_BA_PersonAnalysis_V3: "人群分析类",
    BussinesType_BA_Action_V3: "动作模型类",
    BussinesType_BA_PersonNumStatic_V3: "人数统计类",
    BussinesType_BA_Safety_V3: "安全类",
    BussinesType_BA_CommonVideo_V5: "视频流",  #行为分析视频流，合一版本业务类型
    BussinesType_AS_Video_V3: "全结构化视频流",
    BussinesType_AS_PicSingle_V3: "全结构化图片流",
    BussinesType_AS_PicAll_V3: "图片流全量解析",
    BussinesType_Unknown_V3: "未知模式"
}

# 定义数值枚举（数值在前）
global_task_type_value_enums = {
    TaskType_Unknown: "未知类型",
    TaskType_vas: "结构化业务",
    TaskType_fS: "人脸业务",
    TaskType_ba: "行为分析业务",
    TaskType_bas: "地铁行为分析业务",
    TaskType_as: "全结构化业务"
}

# 定义数值枚举（数值在前）
global_engine_type_value_enums = {
    EngineType_k2: "昆仑2",
    EngineType_k3: "昆仑3",
    EngineType_gpu: "GPU",
    EngineType_MLU: "寒武纪",
    EngineType_aixin: "爱芯"
}
#汇总数据
global_data_desc={}    #汇总描述数据，直接用中文描述的结构体
global_statics_data={} #汇总资源使用情况，用枚举类型描述的结构体
global_static_tasktype_data={} #按照任务类型统计的数量结果

def getSqllite3Data(str_sql):
    bool_ret=False
    str_ret=""
    #获取单行sql
    sql_query_single_line = str_sql.replace('\n', ' ').strip()
    #调用sql命令
    os_type = platform.system()
    if os_type == 'Windows':
        sqlite_command = 'sqlite3.exe {} "{}"'.format("ias.db", sql_query_single_line)
    elif os_type == 'Linux':
        sqlite_command = 'sqlite3 {} "{}"'.format("/opt/wise/ia12k/db/ias.db", sql_query_single_line)
    info_message = "组织的sql命令为：%s" % (sql_query_single_line)
    common.Log(info_message)
    #获取结果
    try:
        # 使用 subprocess.Popen 执行命令
        process = subprocess.Popen(sqlite_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True,universal_newlines=True)
        # 获取输出和错误信息
        stdout, stderr = process.communicate()
        if process.returncode != 0:
            info_message = "Error executing SQLite command: %s" % (common.GetUtf8Str(stderr))
            common.Log(info_message)
            print "读取sqllie数据库异常【%s】,巡检异常" % (common.GetUtf8Str(stderr))
            sys.exit(common.EXIT_STATUS_WARNING)
        else:
            # 解析输出
            bool_ret = True
            str_ret = stdout
    except Exception as e:
        info_message = "Error executing SQLite command，An error occurred: %s" % (common.GetUtf8Str(e))
        common.Log(info_message)
        print "读取sqllie数据库异常【%s】,巡检异常" % (common.GetUtf8Str(e))
        sys.exit(common.EXIT_STATUS_WARNING)
    return bool_ret,str_ret

#处理描述数据
def handleDescData(engine_version, task_type, business_type, count):
    global global_data_desc
    # 如果该engine_version不存在，则初始化
    str_engine_type = global_engine_type_value_enums[int(engine_version)]
    if str_engine_type not in global_data_desc:
        global_data_desc[str_engine_type] = {}
    # 如果task_type不存在，则初始化
    str_task_type = global_task_type_value_enums[int(task_type)]
    str_business_type = global_bussines_value_enums[int(business_type)]
    if str_task_type not in global_data_desc[str_engine_type]:
        global_data_desc[str_engine_type][str_task_type] = {}
    if str_business_type not in global_data_desc[str_engine_type][str_task_type]:
        global_data_desc[str_engine_type][str_task_type][str_business_type] = 0
    # 赋值
    global_data_desc[str_engine_type][str_task_type][str_business_type] = int(count)


#处理汇总数据
def handelStaticData(engine_version, task_type, business_type, count):
    global global_statics_data
    # 如果该engine_version不存在，则初始化
    if int(engine_version) not in global_statics_data:
        global_statics_data[int(engine_version)] = {}
    # 如果task_type不存在，则初始化
    if int(task_type) not in global_statics_data[int(engine_version)]:
        global_statics_data[int(engine_version)][int(task_type)] = {}
    if int(business_type) not in global_statics_data[int(engine_version)][int(task_type)]:
        global_statics_data[int(engine_version)][int(task_type)][int(business_type)] = 0
    # 赋值
    global_statics_data[int(engine_version)][int(task_type)][int(business_type)] = int(count)

#处理数据
def handleSqlLiteData(str_lines):
    for line in str_lines:
        # 去除行首和行尾的空白字符
        line = line.strip()
        # 检查行是否为空
        if not line:
            continue
        # 尝试拆分行
        parts = line.split('|')
        # 确保拆分后的部分数量正确
        if len(parts) != 4:
            info_message = "获取的数据，按照|分割，不足4个字段长度: %s" % (line)
            common.Log(info_message)
            continue
        # 处理数据
        engine_version, task_type, business_type, count = parts
        try:
            #汇总描述数据
            handleDescData(engine_version, task_type, business_type, count)

            #汇总统计数据，累加使用的
            handelStaticData(engine_version, task_type, business_type, count)
        except KeyError as e:
            print("KeyError: The key does not exist in the globals dictionary.{}".format(e))
        except IndexError as e:
            print("IndexError: The index is out of range of the global_task_type_value_enums list.{}".format(e))
        except ValueError as e:
            print("ValueError: The task_type cannot be converted to an integer.{}".format(e))
        except Exception as e:
            print("An unexpected error occurred: {}".format(e))

#的芯片授权一次申请全部，只看资源分配
def getAllAssignResourceInfo():
    #组织sql
    # 编写 SQL 查询
    sql_query = """
SELECT 
    engine_version,
    task_type, 
    business_type, 
    COUNT(*) AS count
FROM 
    ia_engine
WHERE 
    id LIKE '%#%'
    AND business_type != 1
GROUP BY 
    engine_version,
    task_type,
    business_type;
    """
    #获取数据
    bool_ret,str_ret = getSqllite3Data(sql_query)
    #组织数据
    # 解析输出
    output_lines = str_ret.strip().split('\n')
    info_message = "获取的数据 %s" % (output_lines)
    common.Log(info_message)
    handleSqlLiteData(output_lines)

#统计板卡类型分配的资源所用的授权
def staticEngineResourceData(data):
    # 定义数值枚举（数值在前）
    struct_static_tasktype = {
        "int_total_vas_pic_num": 0,  # 结构化业务单元总个数，包括结构化相关的所有业务类型
        "int_total_fS_pic_num": 0,  # 人脸业务单元总个数，包括人脸业务相关的所有业务类型
        "int_total_vas_video_num": 0,  # 结构化业务单元总个数，包括结构化相关的所有业务类型
        "int_total_fS_video_num": 0,  # 人脸业务单元总个数，包括人脸业务相关的所有业务类型
        "int_total_ba_num": 0,  # 行为分析业务单元总个数，包括行为分析业务相关的所有业务类型
        "int_total_bas_num": 0,  # 地铁行为分析业务单元总个数，包括地铁行为分析业务相关的所有业务类型
        "int_total_as_num": 0,  # 全结构化业务单元总个数，包括全结构化业务相关的所有业务类型
        "int_total_unknown_num": 0,  # 未分配的单元数
    }
    #单个引擎类型资源分配的统计
    for key, value in data.items():
        #key表示tasktype
        if key == TaskType_vas:
            #处理Vas的业务
            struct_static_tasktype["int_total_vas_pic_num"] = value.get(BussinesType_Vas_Pic_V3,0)
            struct_static_tasktype["int_total_vas_video_num"] = value.get(BussinesType_VAS_Video_V3, 0)

        elif key == TaskType_fS:
            #处理fs的业务
            struct_static_tasktype["int_total_fS_pic_num"] += value.get(BussinesType_FS_Pic_V3,0)
            struct_static_tasktype["int_total_fS_video_num"] += value.get(BussinesType_FS_Video_V3,0)
        elif key == TaskType_ba:
            #处理行为分析的业务
            struct_static_tasktype["int_total_ba_num"] += value.get(BussinesType_BA_Supervise_V3,0)
            struct_static_tasktype["int_total_ba_num"] += value.get(BussinesType_BA_PersonAnalysis_V3, 0)
            struct_static_tasktype["int_total_ba_num"] += value.get(BussinesType_BA_Action_V3, 0)
            struct_static_tasktype["int_total_ba_num"] += value.get(BussinesType_BA_PersonNumStatic_V3, 0)
            struct_static_tasktype["int_total_ba_num"] += value.get(BussinesType_BA_Safety_V3, 0)
            struct_static_tasktype["int_total_ba_num"] += value.get(BussinesType_BA_CommonVideo_V5, 0)   #老板本没有，新版本才有
        elif key == TaskType_bas:
            #处理地铁行为分析业务
            struct_static_tasktype["int_total_bas_num"] += value.get(BussinesType_BA_Supervise_V3, 0)
            struct_static_tasktype["int_total_bas_num"] += value.get(BussinesType_BA_PersonAnalysis_V3, 0)
            struct_static_tasktype["int_total_bas_num"] += value.get(BussinesType_BA_Action_V3, 0)
            struct_static_tasktype["int_total_bas_num"] += value.get(BussinesType_BA_PersonNumStatic_V3, 0)
            struct_static_tasktype["int_total_bas_num"] += value.get(BussinesType_BA_Safety_V3, 0)
            struct_static_tasktype["int_total_bas_num"] += value.get(BussinesType_BA_CommonVideo_V5, 0)  # 老板本没有，新版本才有
        elif key == TaskType_as:
            #处理as的业务
            struct_static_tasktype["int_total_as_num"] += value.get(BussinesType_AS_Video_V3,0)
            struct_static_tasktype["int_total_as_num"] += value.get(BussinesType_AS_PicAll_V3, 0)
            struct_static_tasktype["int_total_as_num"] += value.get(BussinesType_AS_PicSingle_V3, 0)
        elif key == TaskType_Unknown:
            #处理为分配的业务类型
            struct_static_tasktype["int_total_unknown_num"] += value.get(BussinesType_Unknown_V3, 0)
    #汇总
    return struct_static_tasktype

#统计数据返回结果并删除，计算各个芯片未分配的单元总数、业务流程
def staticsResource():
    global global_static_tasktype_data
    int_ret = common.EXIT_STATUS_INFO
    str_ret = ""
    #处理爱芯的数据
    if EngineType_aixin in global_statics_data:
        global_static_tasktype_data[EngineType_aixin] = staticEngineResourceData(global_statics_data[EngineType_aixin])
        if global_static_tasktype_data[EngineType_aixin]["int_total_unknown_num"] != 0:
            str_ret += "爱芯板卡存在未分配资源的芯片单元;"
            int_ret = common.EXIT_STATUS_WARNING
    #处理k2数据
    if EngineType_k2 in global_statics_data:
        global_static_tasktype_data[EngineType_k2] = staticEngineResourceData(global_statics_data[EngineType_k2])
        if global_static_tasktype_data[EngineType_k2]["int_total_unknown_num"] != 0:
            str_ret += "昆仑2板卡存在未分配资源的芯片单元;"
            int_ret = common.EXIT_STATUS_WARNING
    #处理k3数据
    if EngineType_k3 in global_statics_data:
        global_static_tasktype_data[EngineType_k3] = staticEngineResourceData(global_statics_data[EngineType_k3])
        if global_static_tasktype_data[EngineType_k3]["int_total_unknown_num"] != 0:
            str_ret += "昆仑3板卡存在未分配资源的芯片单元;"
            int_ret = common.EXIT_STATUS_WARNING
    #处理mlu数据
    if EngineType_MLU in global_statics_data:
        global_static_tasktype_data[EngineType_MLU] = staticEngineResourceData(global_statics_data[EngineType_MLU])
        if global_static_tasktype_data[EngineType_MLU]["int_total_unknown_num"] != 0:
            str_ret += "寒武纪板卡存在未分配资源的芯片单元;"
            int_ret = common.EXIT_STATUS_WARNING
    #处理GPU数据
    if EngineType_gpu in global_statics_data:
        global_static_tasktype_data[EngineType_gpu] = staticEngineResourceData(global_statics_data[EngineType_gpu])
        if global_static_tasktype_data[EngineType_gpu]["int_total_unknown_num"] != 0:
            str_ret += "GPU板卡存在未分配资源的芯片单元;"
            int_ret = common.EXIT_STATUS_WARNING
    return int_ret,str_ret

# 使用递归函数打印字典
def getAssignResourceDesc(data, result=None):
    if result is None:
        result = []
    for key, value in data.items():
        result.append(key)
        result.append("分配的资源:")
        for taskkey, taskvalue in value.items():
            if taskkey != global_task_type_value_enums[TaskType_Unknown]:
                result.append(taskkey)
                result.append(":")
            for bussinesskey, bussinessValue in taskvalue.items():
                result.append(bussinesskey)
                result.append(":")
                result.append(str(bussinessValue))
                result.append("单元 ")
            result.append("|")
        result.append(" ")
    return "".join(result)

def mainCheck():
    int_ret = common.EXIT_STATUS_INFO
    str_ret = ""
    # 获取资源分配的情况
    getAllAssignResourceInfo()
    # 统计数据,并输出是否存在未分配资源的芯片单元
    int_ret,str_ret = staticsResource()
    # 判断是否存在全部未分配的情况
    if len(global_static_tasktype_data) == 0 and len(global_statics_data) == 0:
        str_ret = "所有的芯片单元都未分配资源"
        int_ret = common.EXIT_STATUS_ERROR
    # 授权分配情况
    assign_resource_desc_string = getAssignResourceDesc(global_data_desc)
    common.Log(assign_resource_desc_string)
    # 汇总
    str_ret += "资源分配的总体情况如下："
    str_ret += assign_resource_desc_string
    # 将授权新返回
    print str_ret
    sys.exit(int_ret)

mainCheck()





