# coding=utf-8
#功能授权校验巡检模块：通用授权和授权分配情况
import subprocess
import sys
import common
import time
import urllib2
import urllib

#获取授权相关信息
global_licenseUsedInfo = {}  #缓存新版本授权信息,包括总授权和使用授权
#新版本接口
def getLicenseUsedInfo():
    global global_licenseUsedInfo
    # 获取IAM的授权到期情况
    int_time = int(time.time() * 1000)
    # URL
    url = "http://127.0.0.1:8084/iam3/licensemanage/getLicenseUsedInfo"
    # 参数
    params = {"t": int_time}
    # 构建查询字符串
    query_string = urllib.urlencode(params)
    # 完整的URL
    full_url = url + '?' + query_string
    # 获取请求数据
    json_result = common.GetDataFromUrl(full_url)
    if json_result == "":
        #返回的数据不是json格式
        info_message = "HTTP请求【%s】返回的数据信息为空，不支持该接口" % (full_url)
        common.Log(info_message)
        return False
    # 打印日志信息
    info_message = "HTTP请求【%s】返回的数据信息【%s】" % (full_url, json_result)
    common.Log(info_message)
    # 解析返回的结果
    json_object = common.ParseJsonData(json_result)
    #增加判断，如果不是json格式则提示
    if json_result != ""  and json_object == "":
        #返回的数据不是json格式
        info_message = "HTTP请求【%s】返回的数据信息不是json格式，不支持该接口" % (full_url)
        common.Log(info_message)
        return False
    # 检查 'IAMInfo' 是否存在且不为空
    if 'Config' in json_object and len(json_object['Config']) > 0:
        for config in json_object["Config"]:
            iAuthCode = config["iAuthCode"]
            dlTotalNum = config["dlTotalNum"]
            dlUsedNum = config["dlUsedNum"]
            # 使用 iAuthCode 作为键，将其他信息存储在一个嵌套字典中
            global_licenseUsedInfo[iAuthCode] = {
                "dlTotalNum": dlTotalNum,
                "dlUsedNum": dlUsedNum
            }
    else:
        int_total_ret = common.EXIT_STATUS_WARNING
        str_total_messgae = "获取授权情况为空，巡检授权使用信息异常退出"
        common.Log(str_total_messgae)
    # 返回数据
    return True

#老版本相关接口
global_authorize_info = {}   #缓存老版本的授权信息，只有业务授权的路数，没有使用量
#获取kunlun3的授权信息
def getAuthorizeInfo():
    global global_authorize_info
    # 获取IAM的授权到期情况
    int_time = int(time.time() * 1000)
    # URL
    url = "http://127.0.0.1:8084/iam3/licensemanage/getAuthorizeInfo"
    # 参数
    params = {"t": int_time,"condition":2}
    # 构建查询字符串
    query_string = urllib.urlencode(params)
    # 完整的URL
    full_url = url + '?' + query_string
    # 获取请求数据
    json_result = common.GetDataFromUrl(full_url)
    # 打印日志信息
    info_message = "HTTP请求【%s】返回的数据信息【%s】" % (full_url, json_result)
    common.Log(info_message)
    # 解析返回的结果
    json_object = common.ParseJsonData(json_result)
    #增加判断，如果不是json格式则提示
    if json_result != ""  and json_object == "":
        #返回的数据不是json格式
        info_message = "HTTP请求【%s】返回的数据信息不是json格式，不支持该接口" % (full_url)
        common.Log(info_message)
        return False
    # 检查 'AuthorizeInfo' 是否存在且不为空
    if 'AuthorizeInfo' in json_object and len(json_object['AuthorizeInfo']) > 0:
        for key, value in json_object["AuthorizeInfo"].items():
            global_authorize_info[key] = value
    else:
        str_total_messgae = "获取授权情况为空，巡检IAM授权使用信息异常退出"
        common.Log(str_total_messgae)
    # 返回数据
    return True

#获取GPU的授权信息
def getAuthorizeDetailInfoGPU():
    global global_authorize_info
    # 获取IAM的授权到期情况
    int_time = int(time.time() * 1000)
    # URL
    url = "http://127.0.0.1:8084/iam3/licensemanage/getAuthorizeDetailInfo"
    # 参数
    params = {"t": int_time,"condition":3}
    # 构建查询字符串
    query_string = urllib.urlencode(params)
    # 完整的URL
    full_url = url + '?' + query_string
    # 获取请求数据
    json_result = common.GetDataFromUrl(full_url)
    # 打印日志信息
    info_message = "HTTP请求【%s】返回的数据信息【%s】" % (full_url, json_result)
    common.Log(info_message)
    # 解析返回的结果
    json_object = common.ParseJsonData(json_result)
    #增加判断，如果不是json格式则提示
    if json_result != ""  and json_object == "":
        #返回的数据不是json格式
        info_message = "HTTP请求【%s】返回的数据信息不是json格式，不支持该接口" % (full_url)
        common.Log(info_message)
        return False
    # 检查 'AuthorizeDetailInfo' 是否存在且不为空
    if 'AuthorizeDetailInfo' in json_object and len(json_object['AuthorizeDetailInfo']) > 0:
        for key, value in json_object["AuthorizeDetailInfo"].items():
            global_authorize_info[key] = value
    else:
        str_total_messgae = "获取授权情况为空，巡检IAM授权使用信息异常退出"
        common.Log(str_total_messgae)
    # 返回数据
    return True

#获取mlu的授权信息
def getAuthorizeDetailInfoMLU():
    global global_authorize_info
    # 获取IAM的授权到期情况
    int_time = int(time.time() * 1000)
    # URL
    url = "http://127.0.0.1:8084/iam3/licensemanage/getAuthorizeDetailInfo"
    # 参数
    params = {"t": int_time,"condition":4}
    # 构建查询字符串
    query_string = urllib.urlencode(params)
    # 完整的URL
    full_url = url + '?' + query_string
    # 获取请求数据
    json_result = common.GetDataFromUrl(full_url)
    # 打印日志信息
    info_message = "HTTP请求【%s】返回的数据信息【%s】" % (full_url, json_result)
    common.Log(info_message)
    # 解析返回的结果
    json_object = common.ParseJsonData(json_result)
    #增加判断，如果不是json格式则提示
    if json_result != ""  and json_object == "":
        #返回的数据不是json格式
        info_message = "HTTP请求【%s】返回的数据信息不是json格式，不支持该接口" % (full_url)
        common.Log(info_message)
        return False
    # 检查 'AuthorizeDetailInfo' 是否存在且不为空
    if 'AuthorizeDetailInfo' in json_object and len(json_object['AuthorizeDetailInfo']) > 0:
        for key, value in json_object["AuthorizeDetailInfo"].items():
            global_authorize_info[key] = value
    else:
        str_total_messgae = "获取授权情况为空，巡检IAM授权使用信息异常退出"
        common.Log(str_total_messgae)
    # 返回数据
    return True

#通用授权再分配，判断是否有授权未分配，分配后的资源都在上面的接口可以查询到
global_unvauthor_info = {}   #通用授权使用情况
def getUnvAuthorConfigs():
    global global_unvauthor_info
    # 获取IAM的授权到期情况
    int_time = int(time.time() * 1000)
    # URL
    url = "http://127.0.0.1:8084/iam3/config/getUnvAuthorConfigs"
    # 参数
    params = {"t": int_time}
    # 获取请求数据
    json_result = common.PostDataFromUrl(url,params)
    # 打印日志信息
    info_message = "HTTP请求【%s】返回的数据信息【%s】" % (url, json_result)
    common.Log(info_message)
    # 解析返回的结果
    json_object = common.ParseJsonData(json_result)
    #增加判断，如果不是json格式则提示
    if json_result != ""  and json_object == "":
        #返回的数据不是json格式
        info_message = "HTTP请求【%s】返回的数据信息不是json格式，不支持该接口" % (url)
        common.Log(info_message)
        return False
    # 检查 'AuthorizeDetailInfo' 是否存在且不为空
    if 'Config' in json_object and len(json_object['Config']) > 0:
        for config in json_object["Config"]:
            authorTotal = config["authorTotal"]
            restNum = config["restNum"]
            if int(authorTotal) == 0 or int(restNum) == 0:
                continue
            unvCode = config["unvCode"]
            restNum = config["restNum"]
            allocateAuthor = config["allocateAuthor"]
            # 将信息存储到缓存字典中
            global_unvauthor_info[unvCode] = {
                "authorTotal": authorTotal,
                "restNum": restNum,
            }
    else:
        str_total_messgae = "获取通用授权再分配内容为空"
        common.Log(str_total_messgae)
    # 返回数据
    return True

#汇总数据
#老版本的数据汇总
#人脸识别业务0个单元+80路/400路(图)
# 视频结构化业务0个单元+80路/1000万张
# 行为分析通用业务0个单元(昆仑3)+60路(GPU)+监管0路(昆仑5&Kunlun-G5)+综合40路(昆仑5&Kunlun-G5)
# 行为分析地铁业务0个单元(昆仑3)+60路(GPU)
# 交通违法业务0个单元
# 交通事件业务0个单元+0个GPU单元 ，您有298个通用授权待分配)
# K3:人脸识别业务【IaBusinessFA_V3】、视频结构化业务【IaBusinessVAS_V3】、行为分析通用业务【IaBusinessBA_V3】、行为分析地铁业务【IaBusinessBAS_V3】、交通违法业务【IaBusinessCEP_V3】、交通事件业务【IaBusinessTA_V3】
# k2:人脸识别业务、视频结构化业务、行为分析通用业务  昆仑2只有板卡授权即可
# GPU：人脸识别业务80路【10047】/400路(图)【10046】、视频结构化业务80路【10048】/1000万张【10049】、行为分析通用业务60路(GPU)【10043】、行为分析地铁业务60路(GPU)【20019】
# MLU：人脸图片【5000000】、人脸视频【5000001】、视频结构化【5000003】、图片结构化【5000002】、监管行为分析【20023】、综合行为分析【20025】
def summaryAuthorInfoOld():
    # 组织 K3 的信息
    k3_info = {
        "人脸识别业务(芯片个数)": global_authorize_info.get("IaBusinessFA_V3", 0),
        "结构化业务(芯片个数)": global_authorize_info.get("IaBusinessVAS_V3", 0),
        "行为分析通用业务(芯片个数)": global_authorize_info.get("IaBusinessBA_V3", 0),
        "行为分析地铁业务（路数）": global_authorize_info.get("IaBusinessBAS_V3", 0),
        "交通违法业务(芯片个数)": global_authorize_info.get("IaBusinessCEP_V3", 0),
        "交通事件业务(芯片个数)": global_authorize_info.get("IaBusinessTA_V3", 0)
    }

    # 组织 GPU 的信息
    gpu_info = {
        "人脸业务视频流（路数）": global_authorize_info.get("10047", 0),
        "人脸业务图片流（路数）": global_authorize_info.get("10046", 0),
        "结构化业务视频流（路数）": global_authorize_info.get("10048", 0),
        "结构化业务图片流（万张）": global_authorize_info.get("10049", 0),
        "行为分析通用业务（路数）": global_authorize_info.get("10043", 0),
        "行为分析地铁业务（路数）":global_authorize_info.get("20019", 0)
    }
    # 组织 MLU 的信息
    mlu_info = {
        "人脸业务图片流（路数）": global_authorize_info.get("5000000", 0),
        "人脸业务视频流（路数）": global_authorize_info.get("5000001", 0),
        "结构化业务视频流（路数）": global_authorize_info.get("5000003", 0),
        "结构化业务图片流（路数）": global_authorize_info.get("5000002", 0),
        "监管行为分析（路数）": global_authorize_info.get("20023", 0),
        "综合行为分析（路数）": global_authorize_info.get("20025", 0)
    }
    # 打印结果
    result_string = ""
    # 过滤掉值为0的项
    filtered_k3_info = {k: v for k, v in k3_info.items() if v != 0}
    # 过滤掉值为0的项
    filtered_gpu_info = {k: v for k, v in gpu_info.items() if v != 0}
    # 过滤掉值为0的项
    filtered_mlu_info = {k: v for k, v in mlu_info.items() if v != 0}
    # 将过滤后的字典项组合成一个字符串
    if len(filtered_k3_info) != 0:
        result_string += "kunlun3授权信息为："
        result_string += ', '.join("{}: {}".format(k, v) for k, v in filtered_k3_info.items())
        result_string += ";"
    if len(filtered_gpu_info) != 0:
        result_string += "GPU的授权信息为："
        result_string += ', '.join("{}: {}".format(k, v) for k, v in filtered_gpu_info.items())
        result_string += ";"
    if len(filtered_mlu_info) != 0:
        result_string += "MLU的授权信息为："
        result_string += ', '.join("{}: {}".format(k, v) for k, v in filtered_mlu_info.items())
        result_string += ";"
    return result_string


#新版本授权信息
def getAuthorInfoNewDesc(iAuthCode,boolTx1):  #boolTx1芯片授权标志，设置是路数授权还是单元授权
    str_ret = ""
    if iAuthCode in global_licenseUsedInfo:
        value = global_licenseUsedInfo.get(iAuthCode)
        totalNum = value.get("dlTotalNum",0)
        dlUsedNum = value.get("dlUsedNum",0)
        if iAuthCode == 10049:
            str_ret = "总授权数量：" + str(totalNum) + "张/天，" +  "已用授权数量：" + str(dlUsedNum) + "张/天，" + "剩余授权数量：" + str(totalNum-dlUsedNum) + "张/天；"
        else:
            if boolTx1 == True:  #芯片授权
                str_ret = "总授权数量：" + str(totalNum) + "个，" + "已用授权数量：" + str(
                    dlUsedNum) + "个，" + "剩余授权数量：" + str(totalNum - dlUsedNum) + "个；"
            else:  #路数授权
                str_ret = "总授权数量：" + str(totalNum) + "路，" + "已用授权数量：" + str(
                    dlUsedNum) + "路，" + "剩余授权数量：" + str(totalNum - dlUsedNum) + "路；"
    return str_ret

#获取两个授权码
def getTwoAuthorInfoNewDesc(iAuthCode1,iAuthCode2):
    str_ret = ""
    totalNum1 = 0
    dlUsedNum1 = 0
    totalNum2 = 0
    dlUsedNum2 = 0
    bAuthCode1Find = False
    bAuthCode2Find = False
    if iAuthCode1 in global_licenseUsedInfo:
        value1 = global_licenseUsedInfo.get(iAuthCode1)
        totalNum1 = value1.get("authorTotal",0)
        dlUsedNum1 = value1.get("dlUsedNum",0)
        bAuthCode1Find = True
    if iAuthCode2 in global_licenseUsedInfo:
        value2 = global_licenseUsedInfo.get(iAuthCode2)
        totalNum2 = value2.get("authorTotal", 0)
        dlUsedNum2 = value2.get("dlUsedNum", 0)
        bAuthCode2Find = True
    if bAuthCode1Find == True or bAuthCode2Find == True:
        str_ret = "总授权数量：" + str(totalNum1+totalNum2) + "路，" + "已用授权数量：" + str(dlUsedNum1+dlUsedNum2) + "路，" + "剩余授权数量：" + str(totalNum1+totalNum2-dlUsedNum1-dlUsedNum2) + "路；"
    return str_ret

def summaryAuthorInfoNew58p02():
    # 58P02到58P05版本的授权情况
    # K3:人脸业务单元授权【10038】、机非人业务单元授权【10039】、行为分析业务单元授权【10040】、地铁业务单元授权【20021】、交通违法业务单元授权【10044】、交通事件业务单元授权【10152】、全结构化昆仑3通用授权【10156】
    # k2:人脸识别业务、视频结构化业务、行为分析通用业务  昆仑2只有板卡授权即可
    # GPU：人脸视频流业务授权【10047】、人脸图片流业务授权【10046】、机非人视频流业务授权【10048】、机非人图片流业务授权【10049】、行为分析业务授权【10043】、地铁行为分析业务授权【20019】、全结构化GPU通用授权【10157】
    # MLU：全结构化授权【20022】  （行为分析实际不进行强校验，都是行为分析的授权加和）监管行为分析【20023】、综合行为分析【20025】

    # 组织 K3 的信息
    k3_info = {
        "人脸识别业务": getAuthorInfoNewDesc(10038,True),
        "结构化业务": getAuthorInfoNewDesc(10039,True),
        "行为分析通用业务": getAuthorInfoNewDesc(10040,True),
        "行为分析地铁业务": getAuthorInfoNewDesc(20021,True),
        "交通违法业务": getAuthorInfoNewDesc(10044,True),
        "交通事件业务": getAuthorInfoNewDesc(10152,True),
        "全结构化昆仑3通用授权": getAuthorInfoNewDesc(10156,False)
    }

    # 组织 GPU 的信息
    gpu_info = {
        "人脸业务视频流（路数）": getAuthorInfoNewDesc(10047,False),
        "人脸业务图片流（路数）": getAuthorInfoNewDesc(10046,False),
        "结构化业务视频流（路数）": getAuthorInfoNewDesc(10048,False),
        "结构化业务图片流（张）": getAuthorInfoNewDesc(10049,False),
        "行为分析通用业务（路数）": getAuthorInfoNewDesc(10043,False),
        "行为分析地铁业务（路数）": getAuthorInfoNewDesc(20019,False),
        "全结构化GPU通用授权": getAuthorInfoNewDesc(10157, False)
    }
    # 组织 MLU 的信息
    mlu_info = {
        "全结构化授权（路数）": getAuthorInfoNewDesc(20022,False),
        "行为分析（路数）": getTwoAuthorInfoNewDesc(20023, 20025),
    }
    # 打印结果
    result_string = ""
    # 过滤掉值为0的项
    filtered_k3_info = {k: v for k, v in k3_info.items() if v != ""}
    # 过滤掉值为0的项
    filtered_gpu_info = {k: v for k, v in gpu_info.items() if v != ""}
    # 过滤掉值为0的项
    filtered_mlu_info = {k: v for k, v in mlu_info.items() if v != ""}
    # 将过滤后的字典项组合成一个字符串
    if len(filtered_k3_info) != 0:
        result_string += "kunlun3授权信息为："
        result_string += ', '.join("{}: {}".format(k, v) for k, v in filtered_k3_info.items())
        result_string += ";"
    if len(filtered_gpu_info) != 0:
        result_string += "GPU的授权信息为："
        result_string += ', '.join("{}: {}".format(k, v) for k, v in filtered_gpu_info.items())
        result_string += ";"
    if len(filtered_mlu_info) != 0:
        result_string += "MLU的授权信息为："
        result_string += ', '.join("{}: {}".format(k, v) for k, v in filtered_mlu_info.items())
        result_string += ";"
    return result_string

def summaryAuthorInfoNew58p06():
    # 58P05版本之后使用的下面的授权格式：
    # K3:交通违法业务【10044】
    # 智能通用授权：
    # 交通事件分析授权【20029】、通用行为分析业务授权【20025】、通用全结构化授权【20022】
    k3_info = {
        "交通违法业务（芯片个数）": getAuthorInfoNewDesc(10044,True)
    }

    # 组织 GPU 的信息
    common_info = {
        "交通事件分析业务（路数）": getAuthorInfoNewDesc(20029,False),
        "通用行为分析业务（路数）": getAuthorInfoNewDesc(20025,False),
        "通用全结构化业务（路数）": getAuthorInfoNewDesc(20022,False)
    }
    # 打印结果
    result_string = ""
    # 过滤掉值为0的项
    filtered_k3_info = {k: v for k, v in k3_info.items() if v != ""}
    # 过滤掉值为0的项
    filtered_common_info = {k: v for k, v in common_info.items() if v != ""}
    # 将过滤后的字典项组合成一个字符串
    if len(filtered_k3_info) != 0:
        result_string += "K3授权信息为："
        result_string += ', '.join("{}: {}".format(k, v) for k, v in filtered_k3_info.items())
        result_string += ";"
    if len(filtered_common_info) != 0:
        result_string += "智能通用授权为："
        result_string += ', '.join("{}: {}".format(k, v) for k, v in filtered_common_info.items())
        result_string += ";"
    return result_string

#版本校验,判断是否是58P05之后的版本
def isNewVersions(version_string):
    # 解析版本信息
    major_version, minor_version = parseVersionString(version_string)
    # 比较大版本号和小版本号
    if major_version > 158:
        return True
    elif major_version == 158 and minor_version > 5:
        return True
    else:
        return False

def summaryAuthorInfoNew(bool_new):
    # 58P02到58P05版本的授权情况
    # K3:人脸业务单元授权【10038】、机非人业务单元授权【10039】、行为分析业务单元授权【10040】、地铁业务单元授权【20021】、交通违法业务单元授权【10044】、交通事件业务单元授权【10152】、全结构化昆仑3通用授权【10156】
    # k2:人脸识别业务、视频结构化业务、行为分析通用业务  昆仑2只有板卡授权即可
    # GPU：人脸视频流业务授权【10047】、人脸图片流业务授权【10046】、机非人视频流业务授权【10048】、机非人图片流业务授权【10049】、行为分析业务授权【10043】、地铁行为分析业务授权【20019】、全结构化GPU通用授权【10157】
    # MLU：全结构化授权【20022】  （行为分析实际不进行强校验，都是行为分析的授权加和）监管行为分析【20023】、综合行为分析【20025】
    #58P05版本之后使用的下面的授权格式：
    # K3:交通违法业务【10044】
    # 智能通用授权：
    # 交通事件分析授权【20029】、通用行为分析业务授权【20025】、通用全结构化授权【20022】
    str_ret = ""
    # 判断不同的版本
    if bool_new == True:
        str_ret = summaryAuthorInfoNew58p06()
    else:
        str_ret = summaryAuthorInfoNew58p02()
    return str_ret

#版本号解析，解析出大版本号和小版本号
#HCVERO = "IAM5.0-B3351.151.1.210412"   输出：151 1
def parseVersionString(version_string):
    # 提取版本号部分
    version_part = version_string.split('-')[1]

    # 分割版本号部分
    version_parts = version_part.split('.')

    # 提取大版本号和小版本号
    major_version = int(version_parts[1])
    minor_version = int(version_parts[2])

    return major_version, minor_version

#判断昆仑3的图片流是否存在规格限制，在58P06版本以后，就进行了合一处理，存在限制,占时不计算图片数量
def isK3PicNumberLimit(version_string):
    # 解析版本信息
    major_version, minor_version = parseVersionString(version_string)
    # 比较大版本号和小版本号
    if major_version > 158:
        return True
    elif major_version == 158 and minor_version > 6:
        return True
    else:
        return False

#获取当前的iam的版本信息
def getCurrentVersion():
    str_version=""  #版本信息
    try:
        # 定义要执行的命令
        command1 = ['iamcfgtool.sh', '-v']
        command2 = ['grep HCVERO']
        # 获取配置项
        proc1 = subprocess.Popen(command1, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        proc2 = subprocess.Popen(command2, stdin=proc1.stdout, stdout=subprocess.PIPE, stderr=subprocess.PIPE,shell=True)
        proc1.stdout.close()
        output2, error2 = proc2.communicate()  # 等待命令执行完成并获取输出
        info_message = "iamcfgtool.sh -v 获取的版本信息为[%s]" % (output2)
        common.Log(info_message)
        if output2: #不为空
            str_version = output2
        if error2:
            error_message = "iamcfgtool.sh -v 获取的版本信息失败，错误信息：【%s】" % (error2)
            common.Log(error_message)
    except Exception as e:
        error_message = "getCurrentVersion Exception occurred: %s" % str(e)
        common.Log(error_message)
    return str_version

#主函数
def mainCheck():
    str_ret = ""
    int_ret = common.EXIT_STATUS_INFO
    #通过发送老接口查看是否能正常返回，用来判断
    if getLicenseUsedInfo() == True:
        if len(global_licenseUsedInfo) == 0: #表示没有获取到授权信息
            int_ret = common.EXIT_STATUS_ERROR
            str_ret = "授权信息为空"
        else:
            # 获取版本信息
            str_version = getCurrentVersion()
            if str_version == "":
                str_ret = "获取iam的版本信息异常,巡检授权信息失败"
                int_ret = common.EXIT_STATUS_WARNING
            else:
                str_ret = summaryAuthorInfoNew(isNewVersions(str_version))
    else: #老版本
        if getAuthorizeInfo() == True and getAuthorizeDetailInfoGPU() == True and getAuthorizeDetailInfoMLU() == True:
            if len(global_authorize_info) == 0:  # 表示没有获取到授权信息
                int_ret = common.EXIT_STATUS_ERROR
                str_ret = "授权信息为空"
            else:
                str_ret = summaryAuthorInfoOld()
        else:
            # 将授权新返回
            str_ret="巡检授权信息出现问题,请排查日志"
            int_ret = common.EXIT_STATUS_WARNING
        #通用授权再分配校验
        if getUnvAuthorConfigs() == True:
            if len(global_unvauthor_info) != 0:
                str_ret += "；注意：通用授权再分配存在剩余资源，请重新分配"
                int_ret = common.EXIT_STATUS_ERROR
        else:
            # 将授权新返回
            str_ret = "；注意：通用授权再分配巡检出现问题,请排查日志"
            int_ret = common.EXIT_STATUS_WARNING
    #将授权新返回
    print str_ret
    sys.exit(int_ret)

mainCheck()