# coding=utf-8
#功能授权校验巡检模块
import subprocess
import sys
import common
import time
import urllib2
import urllib

#获取集群cluster-ID的信息
def getClusterIdInfo():
    ret_cluster_id = ""
    # 当前的毫秒时间戳
    int_time = int(time.time() * 1000)
    # URL
    url = "http://127.0.0.1:8084/iam3/devicemanage/clusterList"
    # 参数
    params = {"t": int_time}
    # 构建查询字符串
    query_string = urllib.urlencode(params)
    # 完整的URL
    full_url = url + '?' + query_string
    # 获取请求数据
    json_result = common.GetDataFromUrl(full_url)
    # 解析返回的结果
    json_object = common.ParseJsonData(json_result)
    #打印日志信息
    info_message = "HTTP请求【%s】返回的数据信息【%s】" % (full_url, json_result)
    common.Log(info_message)
    # 检查 'ClusterList' 是否存在且不为空
    if 'ClusterList' in json_object and len(json_object['ClusterList']) > 0:
        # 获取 ID 的数值
        ret_cluster_id = json_object['ClusterList'][0]['ID']
    else:
        error_message = "获取ClusterList为空，无法获取集群ID信息"
        common.Log(error_message)
    #返回数据
    return ret_cluster_id

#获取IAM的授权到期情况
def checkIamTimeExpires():
    int_ret = common.EXIT_STATUS_INFO  #巡检结果
    str_messgae = ""  #巡检返回信息
    #获取集群ID
    ret_cluster_id = getClusterIdInfo()
    if ret_cluster_id == "":
        int_ret = common.EXIT_STATUS_WARNING
        str_messgae = "获取集群ID异常，巡检IAM授权是否到期异常退出"
        return int_ret,str_messgae
    #获取IAM的授权到期情况
    int_time = int(time.time() * 1000)
    # URL
    url = "http://127.0.0.1:8084/iam3/licensemanage/getLicenseInfoIAM"
    # 参数
    params = {"condition": ret_cluster_id, "t": int_time}
    # 构建查询字符串
    query_string = urllib.urlencode(params)
    # 完整的URL
    full_url = url + '?' + query_string
    # 获取请求数据
    json_result = common.GetDataFromUrl(full_url)
    # 解析返回的结果
    json_object = common.ParseJsonData(json_result)
    # 打印日志信息
    info_message = "HTTP请求【%s】返回的数据信息【%s】" % (full_url, json_result)
    common.Log(info_message)
    # 检查 'IAMInfo' 是否存在且不为空
    if 'IAMInfo' in json_object and len(json_object['IAMInfo']) > 0:
        #获取授权申请日期
        str_createTime = json_object['IAMInfo']['LicenseTime']['createTime']
        # 获取总授权时间
        int_totalDay = json_object['IAMInfo']['LicenseTime']['totalDay']
        #获取剩余授权时间
        int_usedDay = json_object['IAMInfo']['LicenseTime']['usedDay']
        int_left_time = int_totalDay - int_usedDay
        if(int_totalDay == -1):
            int_ret = common.EXIT_STATUS_INFO
            str_messgae = "局点授权方式为永久授权"
        elif int_totalDay <= 31*3:  #小于3个月
            int_ret = common.EXIT_STATUS_ERROR
            str_messgae = "巡检IAM授权剩余时间【%d】天到期，授权申请时间【%s】,授权申请的总天数【%d】,已使用天数【%d】" % (
                int_left_time,str(str_createTime),int_totalDay,int_usedDay)
        else:
            int_ret = common.EXIT_STATUS_INFO
            str_messgae = "巡检IAM授权剩余时间【%d】天到期，授权申请时间【%s】,授权申请的总天数【%d】,已使用天数【%d】" % (
                int_left_time, str(str_createTime), int_totalDay, int_usedDay)
        common.Log(str_messgae)
    else:
        int_ret = common.EXIT_STATUS_WARNING
        str_messgae = "获取IAMInfo为空，无法获取IAM授权时间到期信息，巡检IAM授权是否到期异常退出"
        common.Log(str_messgae)
    # 返回数据
    return int_ret,str_messgae

#获取IAE的授权到期情况，只针对昆仑2进行巡检
def checkIaeTimeExpires():
    int_total_ret = common.EXIT_STATUS_INFO  #巡检结果
    str_total_messgae = ""  #巡检返回信息
    #获取集群ID
    ret_cluster_id = getClusterIdInfo()
    if ret_cluster_id == "":
        int_total_ret = common.EXIT_STATUS_WARNING
        str_total_messgae = "获取集群ID异常，巡检IAM授权是否到期异常退出"
        return int_total_ret,str_total_messgae
    #获取IAM的授权到期情况
    int_time = int(time.time() * 1000)
    # URL
    url = "http://127.0.0.1:8084/iam3/licensemanage/getEngineLicenseInfo"
    # 参数
    params = {"condition": ret_cluster_id, "t": int_time}
    # 构建查询字符串
    query_string = urllib.urlencode(params)
    # 完整的URL
    full_url = url + '?' + query_string
    # 获取请求数据
    json_result = common.GetDataFromUrl(full_url)
    # 解析返回的结果
    json_object = common.ParseJsonData(json_result)
    # 打印日志信息
    info_message = "HTTP请求【%s】返回的数据信息【%s】" % (full_url, json_result)
    common.Log(info_message)
    # 检查 'IAEngineInfo' 是否存在且不为空
    if 'IAEngineInfo' in json_object and len(json_object['IAEngineInfo']) > 0:
        for engine_info in json_object['IAEngineInfo']:
            str_messgae = ""
            int_ret = common.EXIT_STATUS_INFO
            engine_type = engine_info.get('EngineType')
            if engine_type != "ENGINE_VERSION_V2":  # 非昆仑2的直接返回
                continue
            #板卡IP
            address = engine_info.get('Addrress')
            name = engine_info.get('Name')
            # 获取授权申请日期
            license_time = engine_info.get('LicenseTime', {})
            str_createTime = license_time.get('createTime')
            # 获取总授权时间
            int_totalDay = license_time.get('totalDay')
            if int_totalDay == 0:
                int_ret = common.EXIT_STATUS_ERROR
                str_messgae = "昆仑2板卡IP【%s】未申请授权" % (common.GetUtf8Str(address))
            elif int_totalDay < 0:
                int_ret = common.EXIT_STATUS_INFO
                str_messgae = "昆仑2板卡IP【%s】申请的永久授权" % (common.GetUtf8Str(address))
            else:
                # 获取剩余授权时间
                int_usedDay = license_time.get('usedDay')
                int_left_time = int_totalDay - int_usedDay
                if int_totalDay <= 31*3:  # 小于3个月
                    int_ret = common.EXIT_STATUS_ERROR
                    str_messgae = "昆仑2板卡IP【%s】授权剩余时间【%d】天到期，授权申请时间【%s】,授权申请的总天数【%d】,已使用天数【%d】" % (
                        common.GetUtf8Str(address),int_left_time, str(str_createTime), int_totalDay, int_usedDay)
                else:
                    int_ret = common.EXIT_STATUS_INFO
                    str_messgae = "昆仑2板卡IP【%s】授权剩余时间【%d】天到期，授权申请时间【%s】,授权申请的总天数【%d】,已使用天数【%d】"% (
                        common.GetUtf8Str(address),int_left_time, str(str_createTime), int_totalDay, int_usedDay)
                common.Log(str_messgae)
            if int_ret > int_total_ret: #等级信息，取最高等级的
                int_total_ret = int_ret
            if str_total_messgae: #不为空
                str_total_messgae = str_total_messgae + ";" + str_messgae
            else:
                str_total_messgae = str_messgae
    else:
        int_total_ret = common.EXIT_STATUS_WARNING
        str_total_messgae = "获取IAEngineInfo为空，无法获取KUNLUN2授权时间到期信息，巡检KUNLUN2授权是否到期异常退出"
        common.Log(str_total_messgae)
    # 返回数据
    return int_total_ret,str_total_messgae

#主函数
def mainCheck():
    int_ret = common.EXIT_STATUS_INFO
    str_message = ""
    #获取IAM授权时间
    int_iam_ret,str_iam_messgae=checkIamTimeExpires()
    if int_iam_ret > int_ret:
        int_ret = int_iam_ret
    #获取iae的授权时间
    int_iae_ret, str_iae_messgae = checkIaeTimeExpires()
    if int_iae_ret > int_ret:
        int_ret = int_iae_ret
    #汇总数据
    str_message += "IAM的授权时间检查："
    str_message += str_iam_messgae
    str_message += ";"
    str_message += "KUNLUN2的授权时间检查："
    str_message += str_iae_messgae
    str_message += ";"
    #返回给shell
    print str_message
    sys.exit(int_ret)

mainCheck()