﻿import sys
import common

VEHICLE_QEUE_STAT_INFO_LIST = {}
PERSON_QEUE_STAT_INFO_LIST = {}
FACE_QEUE_STAT_INFO_LIST = {}
NONMOTOR_QEUE_STAT_INFO_LIST = {}
ALLSTRUCT_QEUE_STAT_INFO_LIST = {}

VEHICLE_CONSUMER_NUM = 0
PERSON_CONSUMER_NUM = 0
NONMOTOR_CONSUMER_NUM = 0
FACE_CONSUMER_NUM = 0
ALLSTRUCT_CONSUMER_NUM = 0

MD_IP = common.GetMdIp()

def VasQeueCheck(k2_vas_chip_num, oth_vas_chip_num, object_type) : 
    return_message = ""
    object_type_chinese = ""
    total_chip_num = k2_vas_chip_num + oth_vas_chip_num
    
    if object_type == "PERSON" :
        object_type_chinese = "人体"
    elif object_type == "VEHICLE" :
        object_type_chinese = "机动车"
    elif object_type == "NONMOTOR" :
        object_type_chinese = "非机动车"
    else :
        return_message = "传参错误, 目标类型：%s" % (object_type)
        common.Log(return_message)
        return common.EXIT_STATUS_WARNING, return_message
    
    # 如果没有分配结构化的芯片，则不需要检查队列数配置
    if 0 == total_chip_num : 
        return_message = "无机非人芯片，不需要检查%s消费者队列" % (object_type_chinese)
        common.Log(return_message)
        return common.EXIT_STATUS_INFO, return_message
        
    vas_url = "http://%s:8050/consumer/queryTopicByConsumer.query?consumerGroup=C_%s_STK_IA12500-VAS_f_PIC" % (MD_IP, object_type)
    vas_consumer_info = common.GetDataFromUrl(vas_url)
    
    if vas_consumer_info == "" :
        return_message = "MQ 访问异常，%s队列配置检查失败" % (object_type_chinese)
        log_message = "%s，url：%s，访问结果：%s" % (return_message, vas_url, vas_consumer_info)
        common.Log(log_message)
        return common.EXIT_STATUS_WARNING, return_message
    
    vas_consumer_info_json = common.ParseJsonData(vas_consumer_info)
    vas_consumer_data_arr_json = vas_consumer_info_json["data"]
    if len(vas_consumer_data_arr_json) <= 1:
        return_message = "无%s消费者队列，不需要检查%s消费者队列" % (object_type, object_type)
        common.Log(return_message)
        return common.EXIT_STATUS_ERROR, return_message
    vas_qeue_stat_info_list = vas_consumer_data_arr_json[1]["queueStatInfoList"]
    vas_qeue_length = len(vas_qeue_stat_info_list)
    
    # 如果芯片数大于队列数，则MQ队列配置一定存在问题
    if total_chip_num > vas_qeue_length :
        return_message = "%sMQ队列数：%d 与芯片数：%d 不匹配。" % (object_type_chinese, vas_qeue_length, total_chip_num)
        return common.EXIT_STATUS_ERROR, return_message
    
    # 如果芯片数小于等于队列数，先判断是否存在无消费者的队列
    # 计数单芯片消费者数量
    chip_consumer_map = {}
    chip_consumer_max_num = 1
    # 遍历 vas_qeue_stat_info_list
    for index_v in range(vas_qeue_length) :
        client_info = vas_qeue_stat_info_list[index_v]["clientInfo"]
        if "" == client_info or client_info is None:
            # 如果存在无消费者的队列，则认为队列数配置存在异常
            return_message = "%sMQ队列数配置存在异常，当前MQ队列数：%d，昆仑2芯片数：%d，其他芯片数：%d" % (object_type_chinese, vas_qeue_length, k2_vas_chip_num, oth_vas_chip_num)
            return common.EXIT_STATUS_ERROR, return_message
        # 计算一颗芯片有一个消费者还是4个消费者。计算逻辑：IP+芯片号一致，但是消费者ID不同，则认为一颗芯片有4个消费者
        # 51及以前的版本：一颗昆仑2芯片，1个消费者，一个昆仑3或者GPU芯片有4个消费者
        # 51之后的版本：一颗芯片一个消费者
        # 以下计算依据MQ实际在线消费者统计，排除了版本之前的差异
        # 先在消费者ID中分离IP及芯片号，并做计数。消费者ID格式：@10760#consumer_PE_206.206.124.18#10248#2_VasPicTask-YK9SAApf5jmZ
        client_info_split = client_info.split("#")
        if len(client_info_split) < 4 :
            return_message = "%s消费者数计算异常，clientInfo：%s" % (object_type_chinese,client_info)
            common.Log(return_message)
            return common.EXIT_STATUS_WARNING, return_message
            
        consumer_ip = client_info_split[1]
        chip_port = client_info_split[2]
        consumer_ip_split = consumer_ip.split("_")
        if len(consumer_ip_split) < 3 :
            return_message = "%s消费者数计算异常，clientInfo：%s" % (object_type_chinese,client_info)
            common.Log(return_message)
            return common.EXIT_STATUS_WARNING, return_message
            
        iae_ip = consumer_ip_split[2]
        key="%s#%s" % (iae_ip, chip_port)
        if key in chip_consumer_map :
            if client_info != chip_consumer_map[key] :
                chip_consumer_max_num = 4
                break
        else :
            chip_consumer_map[key] = client_info
            
    # 计数完成后，统计消费者数量，如果存在单芯片在线消费者数量大于1的，
    consumer_total_num = k2_vas_chip_num + oth_vas_chip_num * chip_consumer_max_num
    if consumer_total_num > vas_qeue_length :
        return_message = "%sMQ队列数配置存在异常，当前MQ队列数：%d, 消费者数：%d" % (object_type_chinese, vas_qeue_length, consumer_total_num)
        return common.EXIT_STATUS_ERROR, return_message
    elif consumer_total_num == 0 :
        return_message = "%sMQ队列消费者数为0，当前MQ队列数：%d, 消费者数：%d" % (object_type_chinese, vas_qeue_length, consumer_total_num)
        return common.EXIT_STATUS_ERROR, return_message
    elif vas_qeue_length % consumer_total_num != 0 :
        return_message = "%sMQ队列数配置存在异常，MQ队列数需要是消费者数的整数倍，当前MQ队列数：%d, 消费者数：%d" % (object_type_chinese, vas_qeue_length, consumer_total_num)
        return common.EXIT_STATUS_WARNING, return_message
    else :
       return_message = "MQ队列配置正常"
       log_message = "%sMQ队列数：%d ，消费者数：%d" % (object_type_chinese, vas_qeue_length, consumer_total_num)
       common.Log(log_message)
       return common.EXIT_STATUS_INFO, return_message

def FsQeueCheck(fs_chip_num) :
    return_message = ""
    # 如果没有分配结构化的芯片，则不需要检查队列数配置
    if 0 == fs_chip_num : 
        return_message = "无人脸芯片，不需要检查人脸消费者队列"
        common.Log(return_message)
        return common.EXIT_STATUS_INFO, return_message
        
    fs_url = "http://%s:8050/consumer/queryTopicByConsumer.query?consumerGroup=IA_FACE_BALANCE_STK_" % (MD_IP)
    fs_consumer_info = common.GetDataFromUrl(fs_url)
    if fs_consumer_info == "" :
        return_message = "MQ 访问异常，人脸队列配置检查失败"
        log_message = "%s，url：%s，访问结果：%s" % (return_message, fs_url, fs_consumer_info)
        common.Log(log_message)
        return common.EXIT_STATUS_WARNING, return_message
    
    fs_consumer_info_json = common.ParseJsonData(fs_consumer_info)
    fs_consumer_data_arr_json = fs_consumer_info_json["data"]
    if len(fs_consumer_data_arr_json) <= 1:
        return_message = "无人脸消费者队列，不需要检查人脸消费者队列"
        common.Log(return_message)
        return common.EXIT_STATUS_ERROR, return_message
    fs_qeue_stat_info_list = fs_consumer_data_arr_json[1]["queueStatInfoList"]
    fs_qeue_length = len(fs_qeue_stat_info_list)
    
    # 如果芯片数大于队列数，则MQ队列配置一定存在问题
    # 人脸负载均衡模式下，一颗芯片只有一个消费者
    if fs_chip_num > fs_qeue_length :
        return_message = "人脸MQ队列数：%d 与芯片数：%d 不匹配。" % (fs_qeue_length, fs_chip_num)
        return common.EXIT_STATUS_ERROR, return_message
    elif fs_chip_num == 0 :
        return_message = "%sMQ队列消费者数为0，当前MQ队列数：%d, 消费者数：%d" % (object_type_chinese, fs_qeue_length, consumer_total_num)
        return common.EXIT_STATUS_ERROR, return_message
    elif fs_qeue_length % fs_chip_num != 0 : 
        return_message = "人脸MQ队列数配置存在异常,MQ队列数需要是消费者数的整数倍，当前MQ队列数：%d, 消费者数：%d" % (fs_qeue_length, fs_chip_num)
        return common.EXIT_STATUS_WARNING, return_message
    else :
        return_message = "人脸MQ队列数检查正常"
        log_message = "人脸MQ队列数：%d ，芯片数：%d" % (fs_qeue_length, fs_chip_num)
        common.Log(log_message)
        return common.EXIT_STATUS_INFO, return_message

def AllStructQeueCheck(allstruct_chip_num) :
    return_message = ""
    # 如果没有分配全量解析的芯片，则不需要检查队列数配置
    if 0 == allstruct_chip_num : 
        return_message = "无全量解析芯片，不需要检查全量解析消费者队列"
        common.Log(return_message)
        return common.EXIT_STATUS_INFO, return_message
        
    allstruct_url = "http://%s:8050/consumer/queryTopicByConsumer.query?consumerGroup=C_ALLSTRUCT_STK_IA12500-VAS_f_PIC" % (MD_IP)
    allstruct_consumer_info = common.GetDataFromUrl(allstruct_url)
    if allstruct_consumer_info == "" :
        return_message = "MQ 访问异常，全量解析队列配置检查失败"
        log_message = "%s，url：%s，访问结果：%s" % (return_message, allstruct_url, allstruct_consumer_info)
        common.Log(log_message)
        return common.EXIT_STATUS_WARNING, return_message
    
    allstruct_consumer_info_json = common.ParseJsonData(allstruct_consumer_info)
    parse_status = allstruct_consumer_info_json["status"]
    allstruct_consumer_data_arr_json = allstruct_consumer_info_json["data"]
    if -1 == parse_status or "" == allstruct_consumer_data_arr_json or len(allstruct_consumer_data_arr_json) <= 1:
        return_message = "无全量解析消费者队列，不需要检查全量解析消费者队列"
        common.Log(return_message)
        return common.EXIT_STATUS_WARNING, return_message
    allstruct_qeue_stat_info_list = allstruct_consumer_data_arr_json[1]["queueStatInfoList"]
    allstruct_qeue_length = len(allstruct_qeue_stat_info_list)
    
    # 如果芯片数大于队列数，则MQ队列配置一定存在问题
    # 人脸负载均衡模式下，一颗芯片只有一个消费者
    if allstruct_chip_num > allstruct_qeue_length :
        return_message = "全量解析MQ队列数：%d 与芯片数：%d 不匹配。" % (allstruct_qeue_length, allstruct_chip_num)
        return common.EXIT_STATUS_ERROR, return_message
    elif allstruct_chip_num == 0 :
        return_message = "%sMQ队列消费者数为0，当前MQ队列数：%d, 消费者数：%d" % (object_type_chinese, allstruct_qeue_length, consumer_total_num)
        return common.EXIT_STATUS_ERROR, return_message
    elif allstruct_qeue_length % allstruct_chip_num != 0 : 
        return_message = "全量解析MQ队列数配置存在异常,MQ队列数需要是消费者数的整数倍，当前MQ队列数：%d, 消费者数：%d" % (allstruct_qeue_length, allstruct_chip_num)
        return common.EXIT_STATUS_ERROR, return_message
    else :
        return_message = "全量解析MQ队列检查正常"
        log_message = "全量解析MQ队列数：%d ，芯片数：%d" % (allstruct_qeue_length, allstruct_chip_num)
        common.Log(log_message)
        return common.EXIT_STATUS_INFO, return_message

def MainCheck() :
    if len(sys.argv) < 5:
        common.Log(sys.argv[0])
        print "MQ队列数检查失败，传参异常，参数列表长度：%d" % (len(sys.argv))
        sys.exit(common.EXIT_STATUS_WARNING)
    face_chip_num=0
    vas_chip_num=0
    allstruct_chip_num=0
    try:
        k2_vas_chip_num = int(sys.argv[1])
        oth_vas_chip_num = int(sys.argv[2])
        allstruct_chip_num = int(sys.argv[3])
        face_chip_num = int(sys.argv[4])
    except Exception as e :
        common.Log("Exception type: %s, Description: %s" % (e.__class__.__name__, str(e)))
        print "MQ队列数检查失败，传参异常"
        sys.exit(common.EXIT_STATUS_WARNING)
    person_check_status, person_error_message = VasQeueCheck(k2_vas_chip_num, oth_vas_chip_num, "PERSON")
    ve_check_status, vehicle_error_message = VasQeueCheck(k2_vas_chip_num, oth_vas_chip_num, "VEHICLE")
    non_check_status, nonmotor_error_message = VasQeueCheck(k2_vas_chip_num, oth_vas_chip_num, "NONMOTOR")
    all_check_status, allstruct_error_message = AllStructQeueCheck(allstruct_chip_num)
    face_check_status, face_error_message = FsQeueCheck(face_chip_num)
    no_error_flag = True
    error_message = ""
    if person_check_status != common.EXIT_STATUS_INFO :
        no_error_flag = False
        error_message="%s，" % (person_error_message)

    if ve_check_status != common.EXIT_STATUS_INFO :
        no_error_flag = False
        error_message="%s%s，" % (error_message, vehicle_error_message)
        
    if non_check_status != common.EXIT_STATUS_INFO :
        no_error_flag = False
        error_message="%s%s，" % (error_message, nonmotor_error_message)
    
    if all_check_status != common.EXIT_STATUS_INFO :
        no_error_flag = False
        error_message="%s%s" % (error_message, allstruct_error_message)
    
    if face_check_status != common.EXIT_STATUS_INFO :
        no_error_flag = False
        error_message="%s%s，" % (error_message, face_error_message)
    
    error_message = error_message.rstrip('，')

    if no_error_flag :
        error_message = "MQ队列数配置正常"
        print error_message
        sys.exit(common.EXIT_STATUS_INFO)
    else:
        print error_message
        sys.exit(common.EXIT_STATUS_ERROR)
    
### 执行脚本
MainCheck()